@@ -4,7 +4,7 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief Context implementations
  *
- * Copyright (c) 2015 - 2024 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -36,6 +36,7 @@
 #include "hash_table.h"
 #include "in.h"
 #include "ly_common.h"
+#include "lyb.h"
 #include "parser_data.h"
 #include "plugins_internal.h"
 #include "plugins_types.h"
@@ -48,14 +49,15 @@
 #include "tree_schema_free.h"
 #include "tree_schema_internal.h"
 
-#include "../models/ietf-datastores@2018-02-14.h"
-#include "../models/ietf-inet-types@2013-07-15.h"
-#include "../models/ietf-yang-library@2019-01-04.h"
-#include "../models/ietf-yang-metadata@2016-08-05.h"
-#include "../models/ietf-yang-schema-mount@2019-01-14.h"
-#include "../models/ietf-yang-structure-ext@2020-06-17.h"
-#include "../models/ietf-yang-types@2013-07-15.h"
-#include "../models/yang@2025-01-29.h"
+#include "../modules/default@2025-06-18.h"
+#include "../modules/ietf-datastores@2018-02-14.h"
+#include "../modules/ietf-inet-types@2013-07-15.h"
+#include "../modules/ietf-yang-library@2019-01-04.h"
+#include "../modules/ietf-yang-metadata@2016-08-05.h"
+#include "../modules/ietf-yang-schema-mount@2019-01-14.h"
+#include "../modules/ietf-yang-structure-ext@2020-06-17.h"
+#include "../modules/ietf-yang-types@2013-07-15.h"
+#include "../modules/yang@2025-01-29.h"
 #define IETF_YANG_LIB_REV "2019-01-04"
 
 static struct internal_modules_s {
@@ -65,8 +67,9 @@ static struct internal_modules_s {
     ly_bool implemented;
     LYS_INFORMAT format;
 } internal_modules[] = {
-    {"ietf-yang-metadata", "2016-08-05", (const char *)ietf_yang_metadata_2016_08_05_yang, 0, LYS_IN_YANG},
+    {"ietf-yang-metadata", "2016-08-05", (const char *)ietf_yang_metadata_2016_08_05_yang, 1, LYS_IN_YANG},
     {"yang", "2025-01-29", (const char *)yang_2025_01_29_yang, 1, LYS_IN_YANG},
+    {"default", "2025-06-18", (const char *)default_2025_06_18_yang, 1, LYS_IN_YANG},
     {"ietf-inet-types", "2013-07-15", (const char *)ietf_inet_types_2013_07_15_yang, 0, LYS_IN_YANG},
     {"ietf-yang-types", "2013-07-15", (const char *)ietf_yang_types_2013_07_15_yang, 0, LYS_IN_YANG},
     {"ietf-yang-schema-mount", "2019-01-14", (const char *)ietf_yang_schema_mount_2019_01_14_yang, 1, LYS_IN_YANG},
@@ -88,7 +91,7 @@ ly_ctx_set_searchdir(struct ly_ctx *ctx, const char *search_dir)
     LY_ARRAY_COUNT_TYPE u;
     struct lys_module *mod;
 
-    LY_CHECK_ARG_RET(ctx, ctx, LY_EINVAL);
+    LY_CHECK_ARG_RET(ctx, ctx, !(ctx->opts & LY_CTX_INT_IMMUTABLE), LY_EINVAL);
 
     if (!search_dir) {
         /* fine, ignore */
@@ -134,8 +137,8 @@ ly_ctx_set_searchdir(struct ly_ctx *ctx, const char *search_dir)
     }
 
     /* new searchdir - reset latests flags (possibly new revisions available) */
-    for (i = 0; i < ctx->list.count; ++i) {
-        mod = ctx->list.objs[i];
+    for (i = 0; i < ctx->modules.count; ++i) {
+        mod = ctx->modules.objs[i];
 
         mod->latest_revision &= ~LYS_MOD_LATEST_SEARCHDIRS;
         if (mod->parsed && mod->parsed->includes) {
@@ -177,7 +180,7 @@ ly_ctx_get_searchdirs(const struct ly_ctx *ctx)
 LIBYANG_API_DEF LY_ERR
 ly_ctx_unset_searchdir(struct ly_ctx *ctx, const char *value)
 {
-    LY_CHECK_ARG_RET(ctx, ctx, LY_EINVAL);
+    LY_CHECK_ARG_RET(ctx, ctx, !(ctx->opts & LY_CTX_INT_IMMUTABLE), LY_EINVAL);
 
     if (!ctx->search_paths.count) {
         return LY_SUCCESS;
@@ -210,7 +213,7 @@ ly_ctx_unset_searchdir(struct ly_ctx *ctx, const char *value)
 LIBYANG_API_DEF LY_ERR
 ly_ctx_unset_searchdir_last(struct ly_ctx *ctx, uint32_t count)
 {
-    LY_CHECK_ARG_RET(ctx, ctx, LY_EINVAL);
+    LY_CHECK_ARG_RET(ctx, ctx, !(ctx->opts & LY_CTX_INT_IMMUTABLE), LY_EINVAL);
 
     for ( ; count > 0 && ctx->search_paths.count; --count) {
         LY_CHECK_RET(ly_set_rm_index(&ctx->search_paths, ctx->search_paths.count - 1, free))
@@ -225,7 +228,7 @@ ly_ctx_load_module(struct ly_ctx *ctx, const char *name, const char *revision, c
     struct lys_module *mod = NULL;
     LY_ERR ret = LY_SUCCESS;
 
-    LY_CHECK_ARG_RET(ctx, ctx, name, NULL);
+    LY_CHECK_ARG_RET(ctx, ctx, !(ctx->opts & LY_CTX_INT_IMMUTABLE), name, NULL);
 
     /* load and parse */
     ret = lys_parse_load(ctx, name, revision, &ctx->unres.creating, &mod);
@@ -235,7 +238,7 @@ ly_ctx_load_module(struct ly_ctx *ctx, const char *name, const char *revision, c
     ret = _lys_set_implemented(mod, features, &ctx->unres);
     LY_CHECK_GOTO(ret, cleanup);
 
-    if (!(ctx->flags & LY_CTX_EXPLICIT_COMPILE)) {
+    if (!(ctx->opts & LY_CTX_EXPLICIT_COMPILE)) {
         /* create dep set for the module and mark all the modules that will be (re)compiled */
         LY_CHECK_GOTO(ret = lys_unres_dep_sets_create(ctx, &ctx->unres.dep_sets, mod), cleanup);
 
@@ -244,10 +247,10 @@ ly_ctx_load_module(struct ly_ctx *ctx, const char *name, const char *revision, c
 
         /* unres resolved */
         lys_unres_glob_erase(&ctx->unres);
-    }
 
-    /* new context state */
-    ly_ctx_new_change(ctx);
+        /* new context state */
+        ly_ctx_new_change(ctx);
+    }
 
 cleanup:
     if (ret) {
@@ -258,44 +261,8 @@ cleanup:
     return mod;
 }
 
-/**
- * @brief Hash table value-equal callback for comparing context error hash table record.
- */
-static ly_bool
-ly_ctx_ht_err_equal_cb(void *val1_p, void *val2_p, ly_bool UNUSED(mod), void *UNUSED(cb_data))
-{
-    struct ly_ctx_err_rec *err1 = val1_p, *err2 = val2_p;
-
-    return !memcmp(&err1->tid, &err2->tid, sizeof err1->tid);
-}
-
-/**
- * @brief Hash table value-equal callback for comparing leafref links hash table record.
- */
-static ly_bool
-ly_ctx_ht_leafref_links_equal_cb(void *val1_p, void *val2_p, ly_bool UNUSED(mod), void *UNUSED(cb_data))
-{
-    struct lyd_leafref_links_rec **rec1 = val1_p, **rec2 = val2_p;
-
-    return (*rec1)->node == (*rec2)->node;
-}
-
-/**
- * @brief Callback for freeing leafref links recorcd internal resources.
- *
- * @param[in] val_p Pointer to leafref links record
- */
-static void
-ly_ctx_ht_leafref_links_rec_free(void *val_p)
-{
-    struct lyd_leafref_links_rec **rec = val_p;
-
-    lyd_free_leafref_links_rec(*rec);
-    free(*rec);
-}
-
 LIBYANG_API_DEF LY_ERR
-ly_ctx_new(const char *search_dir, uint16_t options, struct ly_ctx **new_ctx)
+ly_ctx_new(const char *search_dir, uint32_t options, struct ly_ctx **new_ctx)
 {
     struct ly_ctx *ctx = NULL;
     struct lys_module *module;
@@ -305,7 +272,7 @@ ly_ctx_new(const char *search_dir, uint16_t options, struct ly_ctx **new_ctx)
     struct ly_in *in = NULL;
     LY_ERR rc = LY_SUCCESS;
     struct lys_glob_unres unres = {0};
-    ly_bool builtin_plugins_only;
+    ly_bool builtin_plugins_only, static_plugins_only;
 
     LY_CHECK_ARG_RET(NULL, new_ctx, LY_EINVAL);
 
@@ -317,27 +284,17 @@ ly_ctx_new(const char *search_dir, uint16_t options, struct ly_ctx **new_ctx)
 
     /* plugins */
     builtin_plugins_only = (options & LY_CTX_BUILTIN_PLUGINS_ONLY) ? 1 : 0;
-    LY_CHECK_ERR_GOTO(lyplg_init(builtin_plugins_only), LOGINT(NULL); rc = LY_EINT, cleanup);
+    static_plugins_only = (options & LY_CTX_STATIC_PLUGINS_ONLY) ? 1 : 0;
+    LY_CHECK_ERR_GOTO(lyplg_init(builtin_plugins_only, static_plugins_only), LOGINT(NULL); rc = LY_EINT, cleanup);
 
-    if (options & LY_CTX_LEAFREF_LINKING) {
-        /**
-         * storing the pointer instead of record itself is needed to avoid invalid memory reads. Hash table can reallocate
-         * its memory completely during various manipulation function (e.g. remove, insert). In case of using pointers, the
-         * pointer can be reallocated safely, while record itself remains untouched and can be accessed/modified freely
-         * */
-        ctx->leafref_links_ht = lyht_new(1, sizeof(struct lyd_leafref_links_rec *), ly_ctx_ht_leafref_links_equal_cb, NULL, 1);
-        LY_CHECK_ERR_GOTO(!ctx->leafref_links_ht, rc = LY_EMEM, cleanup);
-    }
+    /* options */
+    ctx->opts = options;
 
-    /* initialize thread-specific error hash table */
-    ctx->err_ht = lyht_new(1, sizeof(struct ly_ctx_err_rec), ly_ctx_ht_err_equal_cb, NULL, 1);
-    LY_CHECK_ERR_GOTO(!ctx->err_ht, rc = LY_EMEM, cleanup);
-
-    /* init LYB hash lock */
-    pthread_mutex_init(&ctx->lyb_hash_lock, NULL);
+    /* ctx data */
+    rc = ly_ctx_data_add(ctx);
+    LY_CHECK_GOTO(rc, cleanup);
 
     /* modules list */
-    ctx->flags = options;
     if (search_dir) {
         search_dir_list = strdup(search_dir);
         LY_CHECK_ERR_GOTO(!search_dir_list, LOGMEM(NULL); rc = LY_EMEM, cleanup);
@@ -366,7 +323,7 @@ ly_ctx_new(const char *search_dir, uint16_t options, struct ly_ctx **new_ctx)
 
     if (!(options & LY_CTX_EXPLICIT_COMPILE)) {
         /* use it for creating the initial context */
-        ctx->flags |= LY_CTX_EXPLICIT_COMPILE;
+        ctx->opts |= LY_CTX_EXPLICIT_COMPILE;
     }
 
     /* create dummy in */
@@ -377,8 +334,8 @@ ly_ctx_new(const char *search_dir, uint16_t options, struct ly_ctx **new_ctx)
     for (i = 0; i < ((options & LY_CTX_NO_YANGLIBRARY) ? (LY_INTERNAL_MODS_COUNT - 2) : LY_INTERNAL_MODS_COUNT); i++) {
         ly_in_memory(in, internal_modules[i].data);
         LY_CHECK_GOTO(rc = lys_parse_in(ctx, in, internal_modules[i].format, NULL, &unres.creating, &module), cleanup);
-        if (internal_modules[i].implemented || (ctx->flags & LY_CTX_ALL_IMPLEMENTED)) {
-            imp_f = (ctx->flags & LY_CTX_ENABLE_IMP_FEATURES) ? all_f : NULL;
+        if (internal_modules[i].implemented || (ctx->opts & LY_CTX_ALL_IMPLEMENTED)) {
+            imp_f = (ctx->opts & LY_CTX_ENABLE_IMP_FEATURES) ? all_f : NULL;
             LY_CHECK_GOTO(rc = lys_implement(module, imp_f, &unres), cleanup);
         }
     }
@@ -386,7 +343,7 @@ ly_ctx_new(const char *search_dir, uint16_t options, struct ly_ctx **new_ctx)
     if (!(options & LY_CTX_EXPLICIT_COMPILE)) {
         /* compile now */
         LY_CHECK_GOTO(rc = ly_ctx_compile(ctx), cleanup);
-        ctx->flags &= ~LY_CTX_EXPLICIT_COMPILE;
+        ctx->opts &= ~LY_CTX_EXPLICIT_COMPILE;
     }
 
 cleanup:
@@ -413,7 +370,12 @@ ly_ctx_new_yl_legacy(struct ly_ctx *ctx, const struct lyd_node *yltree)
     LY_ERR ret = LY_SUCCESS;
     uint32_t i, j;
 
-    LY_CHECK_RET(ret = lyd_find_xpath(yltree, "/ietf-yang-library:yang-library/modules-state/module", &set));
+    LY_CHECK_RET(ret = lyd_find_xpath(yltree, "modules-state/module", &set));
+
+    if (!set->count) {
+        ret = LY_ENOTFOUND;
+        goto cleanup;
+    }
 
     /* process the data tree */
     for (i = 0; i < set->count; ++i) {
@@ -532,7 +494,7 @@ ly_ctx_new_yldata(const char *search_dir, const struct lyd_node *tree, int optio
     ly_bool no_expl_compile = 0;
     uint32_t i, j;
 
-    LY_CHECK_ARG_RET(NULL, tree, ctx, LY_EINVAL);
+    LY_CHECK_ARG_RET(NULL, tree, !strcmp(LYD_NAME(tree), "yang-library"), ctx, LY_EINVAL);
 
     /* create a new context */
     if (*ctx == NULL) {
@@ -543,11 +505,11 @@ ly_ctx_new_yldata(const char *search_dir, const struct lyd_node *tree, int optio
 
     /* redundant to compile modules one-by-one */
     if (!(options & LY_CTX_EXPLICIT_COMPILE)) {
-        ctx_new->flags |= LY_CTX_EXPLICIT_COMPILE;
+        ctx_new->opts |= LY_CTX_EXPLICIT_COMPILE;
         no_expl_compile = 1;
     }
 
-    LY_CHECK_GOTO(ret = lyd_find_xpath(tree, "/ietf-yang-library:yang-library/module-set[1]/module", &set), cleanup);
+    LY_CHECK_GOTO(ret = lyd_find_xpath(tree, "module-set[1]/module", &set), cleanup);
     if (set->count == 0) {
         /* perhaps a legacy data tree? */
         LY_CHECK_GOTO(ret = ly_ctx_new_yl_legacy(ctx_new, tree), cleanup);
@@ -598,7 +560,7 @@ ly_ctx_new_yldata(const char *search_dir, const struct lyd_node *tree, int optio
 
     if (no_expl_compile) {
         /* unset flag */
-        ctx_new->flags &= ~LY_CTX_EXPLICIT_COMPILE;
+        ctx_new->opts &= ~LY_CTX_EXPLICIT_COMPILE;
     }
 
 cleanup:
@@ -619,7 +581,7 @@ ly_ctx_compile(struct ly_ctx *ctx)
 {
     LY_ERR ret = LY_SUCCESS;
 
-    LY_CHECK_ARG_RET(NULL, ctx, LY_EINVAL);
+    LY_CHECK_ARG_RET(NULL, ctx, !(ctx->opts & LY_CTX_INT_IMMUTABLE), LY_EINVAL);
 
     /* create dep sets and mark all the modules that will be (re)compiled */
     LY_CHECK_GOTO(ret = lys_unres_dep_sets_create(ctx, &ctx->unres.dep_sets, NULL), cleanup);
@@ -627,6 +589,9 @@ ly_ctx_compile(struct ly_ctx *ctx)
     /* (re)compile all the dep sets */
     LY_CHECK_GOTO(ret = lys_compile_depset_all(ctx, &ctx->unres), cleanup);
 
+    /* new context state */
+    ly_ctx_new_change(ctx);
+
 cleanup:
     if (ret) {
         /* revert changes of modules */
@@ -636,42 +601,64 @@ cleanup:
     return ret;
 }
 
-LIBYANG_API_DEF uint16_t
+LIBYANG_API_DEF uint32_t
 ly_ctx_get_options(const struct ly_ctx *ctx)
 {
     LY_CHECK_ARG_RET(ctx, ctx, 0);
 
-    return ctx->flags;
+    return ctx->opts;
 }
 
 LIBYANG_API_DEF LY_ERR
-ly_ctx_set_options(struct ly_ctx *ctx, uint16_t option)
+ly_ctx_set_options(struct ly_ctx *ctx, uint32_t option)
 {
     LY_ERR lyrc = LY_SUCCESS;
+    struct ly_ctx_shared_data *ctx_data;
     struct lys_module *mod;
     uint32_t i;
 
-    LY_CHECK_ARG_RET(ctx, ctx, LY_EINVAL);
-    LY_CHECK_ERR_RET((option & LY_CTX_NO_YANGLIBRARY) && !(ctx->flags & LY_CTX_NO_YANGLIBRARY),
+    LY_CHECK_ARG_RET(ctx, ctx, !(ctx->opts & LY_CTX_INT_IMMUTABLE), LY_EINVAL);
+    LY_CHECK_ERR_RET((option & LY_CTX_NO_YANGLIBRARY) && !(ctx->opts & LY_CTX_NO_YANGLIBRARY),
             LOGARG(ctx, option), LY_EINVAL);
 
-    if (!(ctx->flags & LY_CTX_BUILTIN_PLUGINS_ONLY) && (option & LY_CTX_BUILTIN_PLUGINS_ONLY)) {
+    if (!(ctx->opts & LY_CTX_BUILTIN_PLUGINS_ONLY) && (option & LY_CTX_BUILTIN_PLUGINS_ONLY)) {
         LOGERR(ctx, LY_EINVAL,
                 "Invalid argument %s (LY_CTX_BUILTIN_PLUGINS_ONLY can be set only when creating a new context) (%s()).",
                 "option", __func__);
         return LY_EINVAL;
     }
 
-    if (!(ctx->flags & LY_CTX_LEAFREF_LINKING) && (option & LY_CTX_LEAFREF_LINKING)) {
-        ctx->leafref_links_ht = lyht_new(1, sizeof(struct lyd_leafref_links_rec *), ly_ctx_ht_leafref_links_equal_cb, NULL, 1);
-        LY_CHECK_ERR_RET(!ctx->leafref_links_ht, LOGARG(ctx, option), LY_EMEM);
+    if (!(ctx->opts & LY_CTX_STATIC_PLUGINS_ONLY) && (option & LY_CTX_STATIC_PLUGINS_ONLY)) {
+        LOGERR(ctx, LY_EINVAL,
+                "Invalid argument %s (LY_CTX_STATIC_PLUGINS_ONLY can be set only when creating a new context) (%s()).",
+                "option", __func__);
+        return LY_EINVAL;
     }
 
-    if (!(ctx->flags & LY_CTX_SET_PRIV_PARSED) && (option & LY_CTX_SET_PRIV_PARSED)) {
-        ctx->flags |= LY_CTX_SET_PRIV_PARSED;
+    if (!(ctx->opts & LY_CTX_LEAFREF_LINKING) && (option & LY_CTX_LEAFREF_LINKING)) {
+        /* context is not printed (ctx data cant be shared with any other ctx), so no need to hold a lock */
+        ctx_data = ly_ctx_shared_data_get(ctx);
+        ctx_data->leafref_links_ht = lyht_new(1, sizeof(struct lyd_leafref_links_rec *), ly_ctx_ht_leafref_links_equal_cb, NULL, 1);
+        LY_CHECK_ERR_RET(!ctx_data->leafref_links_ht, LOGARG(ctx, option), LY_EMEM);
+    }
+
+    if (!(ctx->opts & LY_CTX_LYB_HASHES) && (option & LY_CTX_LYB_HASHES)) {
+        for (i = 0; i < ctx->modules.count; ++i) {
+            mod = ctx->modules.objs[i];
+            if (!mod->implemented) {
+                continue;
+            }
+
+            /* store all cached hashes for all the nodes */
+            lysc_module_dfs_full(mod, lyb_cache_node_hash_cb, NULL);
+        }
+    }
+
+    if (!(ctx->opts & LY_CTX_SET_PRIV_PARSED) && (option & LY_CTX_SET_PRIV_PARSED)) {
+        ctx->opts |= LY_CTX_SET_PRIV_PARSED;
         /* recompile the whole context to set the priv pointers */
-        for (i = 0; i < ctx->list.count; ++i) {
-            mod = ctx->list.objs[i];
+        for (i = 0; i < ctx->modules.count; ++i) {
+            mod = ctx->modules.objs[i];
             if (mod->implemented) {
                 mod->to_compile = 1;
             }
@@ -684,7 +671,7 @@ ly_ctx_set_options(struct ly_ctx *ctx, uint16_t option)
 
     /* set the option(s) */
     if (!lyrc) {
-        ctx->flags |= option;
+        ctx->opts |= option;
     }
 
     return lyrc;
@@ -698,21 +685,24 @@ lysc_node_clear_priv_dfs_cb(struct lysc_node *node, void *UNUSED(data), ly_bool
 }
 
 LIBYANG_API_DEF LY_ERR
-ly_ctx_unset_options(struct ly_ctx *ctx, uint16_t option)
+ly_ctx_unset_options(struct ly_ctx *ctx, uint32_t option)
 {
     LY_ARRAY_COUNT_TYPE u, v;
     const struct lysc_ext_instance *ext;
     struct lysc_node *root;
+    struct ly_ctx_shared_data *ctx_data;
 
-    LY_CHECK_ARG_RET(ctx, ctx, LY_EINVAL);
+    LY_CHECK_ARG_RET(ctx, ctx, !(ctx->opts & LY_CTX_INT_IMMUTABLE), LY_EINVAL);
     LY_CHECK_ERR_RET(option & LY_CTX_NO_YANGLIBRARY, LOGARG(ctx, option), LY_EINVAL);
 
-    if ((ctx->flags & LY_CTX_LEAFREF_LINKING) && (option & LY_CTX_LEAFREF_LINKING)) {
-        lyht_free(ctx->leafref_links_ht, ly_ctx_ht_leafref_links_rec_free);
-        ctx->leafref_links_ht = NULL;
+    if ((ctx->opts & LY_CTX_LEAFREF_LINKING) && (option & LY_CTX_LEAFREF_LINKING)) {
+        /* context is not printed (ctx data cant be shared with any other ctx), so no need to hold a lock */
+        ctx_data = ly_ctx_shared_data_get(ctx);
+        lyht_free(ctx_data->leafref_links_ht, ly_ctx_ht_leafref_links_rec_free);
+        ctx_data->leafref_links_ht = NULL;
     }
 
-    if ((ctx->flags & LY_CTX_SET_PRIV_PARSED) && (option & LY_CTX_SET_PRIV_PARSED)) {
+    if ((ctx->opts & LY_CTX_SET_PRIV_PARSED) && (option & LY_CTX_SET_PRIV_PARSED)) {
         struct lys_module *mod;
         uint32_t index;
 
@@ -740,7 +730,7 @@ ly_ctx_unset_options(struct ly_ctx *ctx, uint16_t option)
     }
 
     /* unset the option(s) */
-    ctx->flags &= ~option;
+    ctx->opts &= ~option;
 
     return LY_SUCCESS;
 }
@@ -761,38 +751,77 @@ ly_ctx_get_modules_hash(const struct ly_ctx *ctx)
     return ctx->mod_hash;
 }
 
+/**
+ * @brief qsort() alphabetical compare of modules.
+ */
+static int
+ly_ctx_mod_cmp_cb(const void *ptr1, const void *ptr2)
+{
+    const struct lys_module *mod1, *mod2;
+    int r;
+
+    mod1 = *(struct lys_module **)ptr1;
+    mod2 = *(struct lys_module **)ptr2;
+
+    /* module name */
+    r = strcmp(mod1->name, mod2->name);
+    if (!r) {
+        /* revision */
+        if (mod1->revision && !mod2->revision) {
+            r = 1;
+        } else if (!mod1->revision && mod2->revision) {
+            r = -1;
+        } else if (mod1->revision && mod2->revision) {
+            r = strcmp(mod1->revision, mod2->revision);
+        }
+    }
+
+    return r;
+}
+
 void
 ly_ctx_new_change(struct ly_ctx *ctx)
 {
-    const struct lys_module *mod;
-    uint32_t i = ly_ctx_internal_modules_count(ctx), hash = 0, fi = 0;
-    struct lysp_feature *f = NULL;
+    uint32_t i, hash = 0;
+    const struct lys_module **mods = NULL;
+    LY_ARRAY_COUNT_TYPE u;
 
     /* change counter */
     ctx->change_count++;
 
-    /* module hash */
-    while ((mod = ly_ctx_get_module_iter(ctx, &i))) {
+    /* collect modules into an array */
+    mods = malloc(ctx->modules.count * sizeof *mods);
+    LY_CHECK_ERR_GOTO(!mods, LOGMEM(ctx), cleanup);
+    memcpy(mods, ctx->modules.objs, ctx->modules.count * sizeof *mods);
+
+    /* sort modules alphabetically */
+    qsort(mods, ctx->modules.count, sizeof *mods, ly_ctx_mod_cmp_cb);
+
+    /* calculate module hash */
+    for (i = 0; i < ctx->modules.count; ++i) {
         /* name */
-        hash = lyht_hash_multi(hash, mod->name, strlen(mod->name));
+        hash = lyht_hash_multi(hash, mods[i]->name, strlen(mods[i]->name));
 
         /* revision */
-        if (mod->revision) {
-            hash = lyht_hash_multi(hash, mod->revision, strlen(mod->revision));
+        if (mods[i]->revision) {
+            hash = lyht_hash_multi(hash, mods[i]->revision, strlen(mods[i]->revision));
         }
 
         /* enabled features */
-        while ((f = lysp_feature_next(f, mod->parsed, &fi))) {
-            if (f->flags & LYS_FENABLED) {
-                hash = lyht_hash_multi(hash, f->name, strlen(f->name));
+        if (mods[i]->implemented) {
+            LY_ARRAY_FOR(mods[i]->compiled->features, u) {
+                hash = lyht_hash_multi(hash, mods[i]->compiled->features[u], strlen(mods[i]->compiled->features[u]));
             }
         }
 
         /* imported/implemented */
-        hash = lyht_hash_multi(hash, (char *)&mod->implemented, sizeof mod->implemented);
+        hash = lyht_hash_multi(hash, (char *)&mods[i]->implemented, sizeof mods[i]->implemented);
     }
 
     ctx->mod_hash = lyht_hash_multi(hash, NULL, 0);
+
+cleanup:
+    free(mods);
 }
 
 LIBYANG_API_DEF ly_module_imp_clb
@@ -809,14 +838,14 @@ ly_ctx_get_module_imp_clb(const struct ly_ctx *ctx, void **user_data)
 LIBYANG_API_DEF void
 ly_ctx_set_module_imp_clb(struct ly_ctx *ctx, ly_module_imp_clb clb, void *user_data)
 {
-    LY_CHECK_ARG_RET(ctx, ctx, );
+    LY_CHECK_ARG_RET(ctx, ctx, !(ctx->opts & LY_CTX_INT_IMMUTABLE), );
 
     ctx->imp_clb = clb;
     ctx->imp_clb_data = user_data;
 
     /* new import callback - reset latests flags (possibly new revisions available) */
-    for (uint32_t v = 0; v < ctx->list.count; ++v) {
-        struct lys_module *mod = ctx->list.objs[v];
+    for (uint32_t v = 0; v < ctx->modules.count; ++v) {
+        struct lys_module *mod = ctx->modules.objs[v];
 
         mod->latest_revision &= ~LYS_MOD_LATEST_IMPCLB;
         if (mod->parsed && mod->parsed->includes) {
@@ -828,15 +857,24 @@ ly_ctx_set_module_imp_clb(struct ly_ctx *ctx, ly_module_imp_clb clb, void *user_
 }
 
 LIBYANG_API_DEF ly_ext_data_clb
-ly_ctx_set_ext_data_clb(struct ly_ctx *ctx, ly_ext_data_clb clb, void *user_data)
+ly_ctx_set_ext_data_clb(const struct ly_ctx *ctx, ly_ext_data_clb clb, void *user_data)
 {
+    struct ly_ctx_shared_data *ctx_data;
     ly_ext_data_clb prev;
 
     LY_CHECK_ARG_RET(ctx, ctx, NULL);
 
-    prev = ctx->ext_clb;
-    ctx->ext_clb = clb;
-    ctx->ext_clb_data = user_data;
+    ctx_data = ly_ctx_shared_data_get(ctx);
+
+    /* EXT CLB LOCK */
+    pthread_mutex_lock(&ctx_data->ext_clb_lock);
+
+    prev = ctx_data->ext_clb;
+    ctx_data->ext_clb = clb;
+    ctx_data->ext_clb_data = user_data;
+
+    /* EXT CLB UNLOCK */
+    pthread_mutex_unlock(&ctx_data->ext_clb_lock);
 
     return prev;
 }
@@ -846,8 +884,8 @@ ly_ctx_get_module_iter(const struct ly_ctx *ctx, uint32_t *index)
 {
     LY_CHECK_ARG_RET(ctx, ctx, index, NULL);
 
-    if (*index < ctx->list.count) {
-        return ctx->list.objs[(*index)++];
+    if (*index < ctx->modules.count) {
+        return ctx->modules.objs[(*index)++];
     } else {
         return NULL;
     }
@@ -872,8 +910,8 @@ ly_ctx_get_module_by_iter(const struct ly_ctx *ctx, const char *key, size_t key_
     struct lys_module *mod;
     const char *value;
 
-    for ( ; *index < ctx->list.count; ++(*index)) {
-        mod = ctx->list.objs[*index];
+    for ( ; *index < ctx->modules.count; ++(*index)) {
+        mod = ctx->modules.objs[*index];
         value = *(const char **)(((int8_t *)(mod)) + key_offset);
         if ((!key_size && !strcmp(key, value)) || (key_size && !strncmp(key, value, key_size) && (value[key_size] == '\0'))) {
             /* increment index for the next run */
@@ -1066,8 +1104,8 @@ _ly_ctx_get_submodule(const struct ly_ctx *ctx, const char *submodule, const cha
 
     LY_CHECK_ARG_RET(ctx, ctx, submodule, NULL);
 
-    for (v = 0; v < ctx->list.count; ++v) {
-        mod = ctx->list.objs[v];
+    for (v = 0; v < ctx->modules.count; ++v) {
+        mod = ctx->modules.objs[v];
         if (!mod->parsed) {
             continue;
         }
@@ -1112,7 +1150,7 @@ ly_ctx_internal_modules_count(const struct ly_ctx *ctx)
         return 0;
     }
 
-    if (ctx->flags & LY_CTX_NO_YANGLIBRARY) {
+    if (ctx->opts & LY_CTX_NO_YANGLIBRARY) {
         return LY_INTERNAL_MODS_COUNT - 2;
     } else {
         return LY_INTERNAL_MODS_COUNT;
@@ -1120,32 +1158,17 @@ ly_ctx_internal_modules_count(const struct ly_ctx *ctx)
 }
 
 static LY_ERR
-ylib_feature(struct lyd_node *parent, const struct lysp_module *pmod)
+ylib_feature(struct lyd_node *parent, const struct lys_module *mod)
 {
     LY_ARRAY_COUNT_TYPE u;
-    struct lysp_feature *f;
 
-    if (!pmod->mod->implemented) {
+    if (!mod->implemented) {
         /* no features can be enabled */
         return LY_SUCCESS;
     }
 
-    LY_ARRAY_FOR(pmod->features, struct lysp_feature, f) {
-        if (!(f->flags & LYS_FENABLED)) {
-            continue;
-        }
-
-        LY_CHECK_RET(lyd_new_term(parent, NULL, "feature", f->name, 0, NULL));
-    }
-
-    LY_ARRAY_FOR(pmod->includes, u) {
-        LY_ARRAY_FOR(pmod->includes[u].submodule->features, struct lysp_feature, f) {
-            if (!(f->flags & LYS_FENABLED)) {
-                continue;
-            }
-
-            LY_CHECK_RET(lyd_new_term(parent, NULL, "feature", f->name, 0, NULL));
-        }
+    LY_ARRAY_FOR(mod->compiled->features, u) {
+        LY_CHECK_RET(lyd_new_term(parent, NULL, "feature", mod->compiled->features[u], 0, NULL));
     }
 
     return LY_SUCCESS;
@@ -1168,8 +1191,7 @@ ylib_deviation(struct lyd_node *parent, const struct lys_module *cur_mod, ly_boo
         if (bis) {
             LY_CHECK_RET(lyd_new_term(parent, NULL, "deviation", mod->name, 0, NULL));
         } else {
-            LY_CHECK_RET(lyd_new_list(parent, NULL, "deviation", 0, NULL, mod->name,
-                    (mod->parsed->revs ? mod->parsed->revs[0].date : "")));
+            LY_CHECK_RET(lyd_new_list(parent, NULL, "deviation", 0, NULL, mod->name, mod->revision ? mod->revision : ""));
         }
     }
 
@@ -1177,32 +1199,32 @@ ylib_deviation(struct lyd_node *parent, const struct lys_module *cur_mod, ly_boo
 }
 
 static LY_ERR
-ylib_submodules(struct lyd_node *parent, const struct lysp_module *pmod, ly_bool bis)
+ylib_submodules(struct lyd_node *parent, const struct lys_module *mod, ly_bool bis)
 {
     LY_ERR ret;
-    LY_ARRAY_COUNT_TYPE i;
+    LY_ARRAY_COUNT_TYPE u;
     struct lyd_node *cont;
-    struct lysp_submodule *submod;
+    struct lysc_submodule *submod;
     int r;
     char *str;
 
-    LY_ARRAY_FOR(pmod->includes, i) {
-        submod = pmod->includes[i].submodule;
+    LY_ARRAY_FOR(mod->submodules, u) {
+        submod = &mod->submodules[u];
 
         if (bis) {
             LY_CHECK_RET(lyd_new_list(parent, NULL, "submodule", 0, &cont, submod->name));
 
-            if (submod->revs) {
-                LY_CHECK_RET(lyd_new_term(cont, NULL, "revision", submod->revs[0].date, 0, NULL));
+            if (submod->revision) {
+                LY_CHECK_RET(lyd_new_term(cont, NULL, "revision", submod->revision, 0, NULL));
             }
         } else {
             LY_CHECK_RET(lyd_new_list(parent, NULL, "submodule", 0, &cont, submod->name,
-                    (submod->revs ? submod->revs[0].date : "")));
+                    submod->revision ? submod->revision : ""));
         }
 
         if (submod->filepath) {
             r = asprintf(&str, "file://%s", submod->filepath);
-            LY_CHECK_ERR_RET(r == -1, LOGMEM(pmod->mod->ctx), LY_EMEM);
+            LY_CHECK_ERR_RET(r == -1, LOGMEM(mod->ctx), LY_EMEM);
 
             ret = lyd_new_term(cont, NULL, bis ? "location" : "schema", str, 0, NULL);
             free(str);
@@ -1230,9 +1252,9 @@ ly_ctx_get_yanglib_data(const struct ly_ctx *ctx, struct lyd_node **root_p, cons
     mod = ly_ctx_get_module_implemented(ctx, "ietf-yang-library");
     LY_CHECK_ERR_RET(!mod, LOGERR(ctx, LY_EINVAL, "Module \"ietf-yang-library\" is not implemented."), LY_EINVAL);
 
-    if (mod->parsed->revs && !strcmp(mod->parsed->revs[0].date, "2016-06-21")) {
+    if (mod->revision && !strcmp(mod->revision, "2016-06-21")) {
         bis = 0;
-    } else if (mod->parsed->revs && !strcmp(mod->parsed->revs[0].date, IETF_YANG_LIB_REV)) {
+    } else if (mod->revision && !strcmp(mod->revision, IETF_YANG_LIB_REV)) {
         bis = 1;
     } else {
         LOGERR(ctx, LY_EINVAL, "Incompatible ietf-yang-library version in context.");
@@ -1246,18 +1268,14 @@ ly_ctx_get_yanglib_data(const struct ly_ctx *ctx, struct lyd_node **root_p, cons
         LY_CHECK_GOTO(ret = lyd_new_list(root_bis, NULL, "module-set", 0, &set_bis, "complete"), error);
     }
 
-    for (i = 0; i < ctx->list.count; ++i) {
-        mod = ctx->list.objs[i];
-        if (!mod->parsed) {
-            LOGERR(ctx, LY_ENOTFOUND, "Parsed module \"%s\" missing in the context.", mod->name);
-            goto error;
-        }
+    for (i = 0; i < ctx->modules.count; ++i) {
+        mod = ctx->modules.objs[i];
 
         /*
          * deprecated legacy
          */
-        LY_CHECK_GOTO(ret = lyd_new_list(root, NULL, "module", 0, &cont, mod->name,
-                (mod->parsed->revs ? mod->parsed->revs[0].date : "")), error);
+        LY_CHECK_GOTO(ret = lyd_new_list(root, NULL, "module", 0, &cont, mod->name, mod->revision ? mod->revision : ""),
+                error);
 
         /* schema */
         if (mod->filepath) {
@@ -1273,7 +1291,7 @@ ly_ctx_get_yanglib_data(const struct ly_ctx *ctx, struct lyd_node **root_p, cons
         LY_CHECK_GOTO(ret = lyd_new_term(cont, NULL, "namespace", mod->ns, 0, NULL), error);
 
         /* feature leaf-list */
-        LY_CHECK_GOTO(ret = ylib_feature(cont, mod->parsed), error);
+        LY_CHECK_GOTO(ret = ylib_feature(cont, mod), error);
 
         /* deviation list */
         LY_CHECK_GOTO(ret = ylib_deviation(cont, mod, 0), error);
@@ -1282,7 +1300,7 @@ ly_ctx_get_yanglib_data(const struct ly_ctx *ctx, struct lyd_node **root_p, cons
         LY_CHECK_GOTO(ret = lyd_new_term(cont, NULL, "conformance-type", mod->implemented ? "implement" : "import", 0, NULL), error);
 
         /* submodule list */
-        LY_CHECK_GOTO(ret = ylib_submodules(cont, mod->parsed, 0), error);
+        LY_CHECK_GOTO(ret = ylib_submodules(cont, mod, 0), error);
 
         /*
          * current revision
@@ -1292,12 +1310,12 @@ ly_ctx_get_yanglib_data(const struct ly_ctx *ctx, struct lyd_node **root_p, cons
             if (mod->implemented) {
                 LY_CHECK_GOTO(ret = lyd_new_list(set_bis, NULL, "module", 0, &cont, mod->name), error);
 
-                if (mod->parsed->revs) {
-                    LY_CHECK_GOTO(ret = lyd_new_term(cont, NULL, "revision", mod->parsed->revs[0].date, 0, NULL), error);
+                if (mod->revision) {
+                    LY_CHECK_GOTO(ret = lyd_new_term(cont, NULL, "revision", mod->revision, 0, NULL), error);
                 }
             } else {
                 LY_CHECK_GOTO(ret = lyd_new_list(set_bis, NULL, "import-only-module", 0, &cont, mod->name,
-                        (mod->parsed->revs ? mod->parsed->revs[0].date : "")), error);
+                        mod->revision ? mod->revision : ""), error);
             }
 
             /* namespace */
@@ -1314,10 +1332,10 @@ ly_ctx_get_yanglib_data(const struct ly_ctx *ctx, struct lyd_node **root_p, cons
             }
 
             /* submodule list */
-            LY_CHECK_GOTO(ret = ylib_submodules(cont, mod->parsed, 1), error);
+            LY_CHECK_GOTO(ret = ylib_submodules(cont, mod, 1), error);
 
             /* feature list */
-            LY_CHECK_GOTO(ret = ylib_feature(cont, mod->parsed), error);
+            LY_CHECK_GOTO(ret = ylib_feature(cont, mod), error);
 
             /* deviation */
             LY_CHECK_GOTO(ret = ylib_deviation(cont, mod, 1), error);
@@ -1361,44 +1379,195 @@ error:
     return ret;
 }
 
-/**
- * @brief Callback for freeing context error hash table values.
- *
- * @param[in] val_p Pointer to a pointer to an error item to free with all the siblings.
- */
-static void
-ly_ctx_ht_err_rec_free(void *val_p)
+LIBYANG_API_DEF void
+ly_ctx_free_parsed(struct ly_ctx *ctx)
 {
-    struct ly_ctx_err_rec *err = val_p;
+    uint32_t i;
+    struct lys_module *mod;
 
-    ly_err_free(err->err);
+    LY_CHECK_ARG_RET(ctx, ctx, !(ctx->opts & LY_CTX_INT_IMMUTABLE), );
+
+    for (i = 0; i < ctx->modules.count; ++i) {
+        mod = ctx->modules.objs[i];
+
+        /* free the parsed modules */
+        lysp_module_free(ctx, mod->parsed);
+        mod->parsed = NULL;
+    }
+}
+
+static ly_bool
+ctxs_ptr_val_equal(void *val1_p, void *val2_p, ly_bool UNUSED(mod), void *UNUSED(cb_data))
+{
+    void *val1, *val2;
+
+    val1 = *(void **)val1_p;
+    val2 = *(void **)val2_p;
+
+    return val1 == val2;
+}
+
+LIBYANG_API_DEF int
+ly_ctx_compiled_size(const struct ly_ctx *ctx)
+{
+    int size = 0;
+    struct ly_ht *ht;
+
+    LY_CHECK_ARG_RET(NULL, ctx, -1);
+
+    /* create hash table for shared structures */
+    ht = lyht_new(0, sizeof(void *), ctxs_ptr_val_equal, NULL, 1);
+    LY_CHECK_RET(!ht, -1);
+
+    ly_ctx_compiled_size_context(ctx, ht, &size);
+
+    /* cleanup */
+    lyht_free(ht, NULL);
+    return size;
+}
+
+static ly_bool
+ctxp_ptr_val_equal(void *val1_p, void *val2_p, ly_bool UNUSED(mod), void *UNUSED(cb_data))
+{
+    struct ctxp_ht_addr *val1, *val2;
+
+    val1 = val1_p;
+    val2 = val2_p;
+
+    return val1->orig_addr == val2->orig_addr;
+}
+
+LIBYANG_API_DEF LY_ERR
+ly_ctx_compiled_print(const struct ly_ctx *ctx, void *mem, void **mem_end)
+{
+    struct ly_ctx *pctx;
+    struct ly_ht *addr_ht = NULL;
+    struct ly_set ptr_set = {0};
+    void **ptr_p;
+    uint32_t i;
+
+    LY_CHECK_ARG_RET(ctx, ctx, mem, LY_EINVAL);
+
+    if (ctx->plugins_types.count || ctx->plugins_extensions.count || !(ctx->opts & LY_CTX_STATIC_PLUGINS_ONLY)) {
+        LOGERR(ctx, LY_EINVAL, "Printing context with dynamic type or extension plugins is not supported.");
+        return LY_EINVAL;
+    }
+
+    /* create hash table for shared structures */
+    addr_ht = lyht_new(0, sizeof(struct ctxp_ht_addr), ctxp_ptr_val_equal, NULL, 1);
+    LY_CHECK_RET(!addr_ht, -1);
+
+    /* context, referenced */
+    pctx = mem;
+    mem = (char *)mem + sizeof *pctx;
+    ly_ctx_compiled_addr_ht_add(addr_ht, ctx, pctx);
+
+    /* members */
+    ly_ctx_compiled_print_context(ctx, pctx, addr_ht, &ptr_set, &mem);
+
+    /* set all the pointers to the printed structures */
+    for (i = 0; i < ptr_set.count; ++i) {
+        ptr_p = ptr_set.objs[i];
+        *ptr_p = ly_ctx_compiled_addr_ht_get(addr_ht, *ptr_p, 0);
+    }
+
+    /* cleanup */
+    lyht_free(addr_ht, NULL);
+    ly_set_erase(&ptr_set, NULL);
+    if (mem_end) {
+        *mem_end = mem;
+    }
+    return LY_SUCCESS;
+}
+
+LIBYANG_API_DEF LY_ERR
+ly_ctx_new_printed(const void *mem, struct ly_ctx **ctx)
+{
+    LY_ERR rc = LY_SUCCESS;
+    ly_bool builtin_plugins_only, static_plugins_only;
+
+    LY_CHECK_ARG_RET(NULL, mem, ctx, LY_EINVAL);
+
+    *ctx = (void *)mem;
+
+    /* ctx data */
+    rc = ly_ctx_data_add(*ctx);
+    if (rc) {
+        goto cleanup;
+    }
+
+    /* plugins */
+    builtin_plugins_only = ((*ctx)->opts & LY_CTX_BUILTIN_PLUGINS_ONLY) ? 1 : 0;
+    static_plugins_only = 1;
+    LY_CHECK_ERR_GOTO(lyplg_init(builtin_plugins_only, static_plugins_only), LOGINT(NULL); rc = LY_EINT, cleanup);
+
+cleanup:
+    if (rc) {
+        if (rc != LY_EEXIST) {
+            /* do not free another context's data */
+            ly_ctx_data_del(*ctx);
+        }
+        *ctx = NULL;
+    }
+    return rc;
+}
+
+LIBYANG_API_DEF ly_bool
+ly_ctx_is_printed(const struct ly_ctx *ctx)
+{
+    if (!ctx) {
+        return 0;
+    }
+
+    return (ctx->opts & LY_CTX_INT_IMMUTABLE) ? 1 : 0;
 }
 
 LIBYANG_API_DEF void
 ly_ctx_destroy(struct ly_ctx *ctx)
 {
-    struct lysf_ctx fctx = {.ctx = ctx};
+    struct lys_module *mod;
+    uint32_t i;
+    LY_ARRAY_COUNT_TYPE u;
 
     if (!ctx) {
         return;
     }
 
-    /* modules list */
-    for ( ; ctx->list.count; ctx->list.count--) {
-        fctx.mod = ctx->list.objs[ctx->list.count - 1];
-
-        /* remove the module */
-        if (fctx.mod->implemented) {
-            fctx.mod->implemented = 0;
-            lysc_module_free(&fctx, fctx.mod->compiled);
-            fctx.mod->compiled = NULL;
+    if (ctx->opts & LY_CTX_INT_IMMUTABLE) {
+        if (!ctx->parent_ctx) {
+            /* ctx data */
+            ly_ctx_data_del(ctx);
         }
-        lys_module_free(&fctx, fctx.mod, 0);
+        lyplg_clean();
+        return;
     }
-    free(ctx->list.objs);
 
-    /* free extensions */
-    lysf_ctx_erase(&fctx);
+    /* free the parsed and compiled modules (both can reference ext instances, which need to be freed, so their
+     * definitions can be freed) */
+    for (i = 0; i < ctx->modules.count; ++i) {
+        mod = ctx->modules.objs[i];
+
+        lysp_module_free(ctx, mod->parsed);
+        mod->parsed = NULL;
+        if (mod->implemented) {
+            mod->implemented = 0;
+            lysc_module_free(ctx, mod->compiled);
+            mod->compiled = NULL;
+        }
+
+        /* even compiled ext definitions can have ext instances, free those, too */
+        LY_ARRAY_FOR(mod->extensions, u) {
+            FREE_ARRAY(ctx, mod->extensions[u].exts, lysc_ext_instance_free);
+            mod->extensions[u].exts = NULL;
+        }
+    }
+
+    /* free the modules */
+    for (i = 0; i < ctx->modules.count; ++i) {
+        mod = ctx->modules.objs[i];
+        lys_module_free(ctx, mod, 0);
+    }
+    free(ctx->modules.objs);
 
     /* search paths list */
     ly_set_erase(&ctx->search_paths, free);
@@ -1406,20 +1575,14 @@ ly_ctx_destroy(struct ly_ctx *ctx)
     /* leftover unres */
     lys_unres_glob_erase(&ctx->unres);
 
-    /* clean the leafref links hash table */
-    if (ctx->leafref_links_ht) {
-        lyht_free(ctx->leafref_links_ht, ly_ctx_ht_leafref_links_rec_free);
+    if (!ctx->parent_ctx) {
+        /* ctx data */
+        ly_ctx_data_del(ctx);
     }
 
-    /* clean the error hash table */
-    lyht_free(ctx->err_ht, ly_ctx_ht_err_rec_free);
-
     /* dictionary */
     lydict_clean(&ctx->dict);
 
-    /* LYB hash lock */
-    pthread_mutex_destroy(&ctx->lyb_hash_lock);
-
     /* context specific plugins */
     ly_set_erase(&ctx->plugins_types, NULL);
     ly_set_erase(&ctx->plugins_extensions, NULL);
