@@ -21,6 +21,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <inttypes.h>
+#include <pthread.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,10 +37,25 @@
 #include <unistd.h>
 
 #include "compat.h"
+#include "schema_compile_node.h"
+#include "tree_data_internal.h"
 #include "tree_schema_internal.h"
 #include "version.h"
 #include "xml.h"
 
+# define _POSIX_C_SOURCE 200809L /* pthread_rwlock_t */
+
+/**< lock for creating and destroying both private & shared context data */
+static pthread_rwlock_t ly_ctx_data_rwlock = PTHREAD_RWLOCK_INITIALIZER;
+
+/**< sized array ([sized array](@ref sizedarrays)) of pointers to private context data (safe realloc).
+ * The context is identified by the thread ID of the thread that created it and its address. */
+static struct ly_ctx_private_data **ly_private_ctx_data;
+
+/**< sized array ([sized array](@ref sizedarrays)) of pointers to shared context data (safe realloc).
+ * The context is identified by the memory address of the context. */
+static struct ly_ctx_shared_data **ly_shared_ctx_data;
+
 LIBYANG_API_DEF uint32_t
 ly_version_so_major(void)
 {
@@ -88,13 +104,538 @@ ly_version_proj_str(void)
     return LY_PROJ_VERSION;
 }
 
+/**
+ * @brief Callback for comparing two leafref links records.
+ */
+ly_bool
+ly_ctx_ht_leafref_links_equal_cb(void *val1_p, void *val2_p, ly_bool UNUSED(mod), void *UNUSED(cb_data))
+{
+    struct lyd_leafref_links_rec **rec1 = val1_p, **rec2 = val2_p;
+
+    return (*rec1)->node == (*rec2)->node;
+}
+
+/**
+ * @brief Callback for freeing two leafref links records.
+ */
+void
+ly_ctx_ht_leafref_links_rec_free(void *val_p)
+{
+    struct lyd_leafref_links_rec **rec = val_p;
+
+    lyd_free_leafref_links_rec(*rec);
+    free(*rec);
+}
+
+/**
+ * @brief Callback for comparing two pattern records.
+ */
+static ly_bool
+ly_ctx_ht_pattern_equal_cb(void *val1_p, void *val2_p, ly_bool UNUSED(mod), void *UNUSED(cb_data))
+{
+    struct ly_pattern_ht_rec *val1 = val1_p;
+    struct ly_pattern_ht_rec *val2 = val2_p;
+
+    /* compare the pattern strings, if they match we can use the stored
+     * serialized value to create the pcre2 code for the pattern */
+    return !strcmp(val1->pattern, val2->pattern);
+}
+
+/**
+ * @brief Callback for freeing a pattern record.
+ */
+static void
+ly_ctx_ht_pattern_free_cb(void *val_p)
+{
+    struct ly_pattern_ht_rec *val = val_p;
+
+    if (val->pcode) {
+        /* free the pcode */
+        pcre2_code_free(val->pcode);
+    }
+}
+
+/**
+ * @brief Remove private context data from the sized array and free its contents.
+ *
+ * @param[in] private_data Private context data to free.
+ */
+static void
+ly_ctx_private_data_remove_and_free(struct ly_ctx_private_data *private_data)
+{
+    LY_ARRAY_COUNT_TYPE u;
+
+    if (!private_data) {
+        return;
+    }
+
+    /* free members */
+    ly_err_free(private_data->errs);
+    free(private_data);
+
+    /* find */
+    LY_ARRAY_FOR(ly_private_ctx_data, u) {
+        if (ly_private_ctx_data[u] == private_data) {
+            break;
+        }
+    }
+    assert(u < LY_ARRAY_COUNT(ly_private_ctx_data));
+
+    /* remove */
+    if (u < LY_ARRAY_COUNT(ly_private_ctx_data) - 1) {
+        /* replace the private data with the last one if it even was added */
+        ly_private_ctx_data[u] = ly_private_ctx_data[LY_ARRAY_COUNT(ly_private_ctx_data) - 1];
+    }
+    LY_ARRAY_DECREMENT_FREE(ly_private_ctx_data);
+}
+
+/**
+ * @brief Create the private context data for a specific context.
+ *
+ * @param[in] ctx Context to create the private data for.
+ * @param[out] private_data Optional created private context data.
+ * @return LY_SUCCESS on success, LY_EMEM on memory allocation failure.
+ */
+static LY_ERR
+ly_ctx_private_data_create(const struct ly_ctx *ctx, struct ly_ctx_private_data **private_data)
+{
+    pthread_t tid = pthread_self();
+    struct ly_ctx_private_data **priv_data = NULL;
+    LY_ERR rc = LY_SUCCESS;
+
+    *private_data = NULL;
+
+    /* create the private context data */
+    LY_ARRAY_NEW_GOTO(ctx, ly_private_ctx_data, priv_data, rc, cleanup);
+    *priv_data = calloc(1, sizeof **priv_data);
+    LY_CHECK_ERR_GOTO(!*priv_data, LOGMEM(ctx); rc = LY_EMEM, cleanup);
+
+    /* fill */
+    (*priv_data)->tid = tid;
+    (*priv_data)->ctx = ctx;
+    (*priv_data)->errs = NULL;
+
+    *private_data = *priv_data;
+
+cleanup:
+    if (rc && priv_data) {
+        ly_ctx_private_data_remove_and_free(*priv_data);
+    }
+    return rc;
+}
+
+/**
+ * @brief Get the private context data for a specific context.
+ *
+ * @param[in] ctx Context whose private data to get.
+ * @param[in] own_data_only Whether to return only the private data of the current thread.
+ * @return Private context data of @p ctx, NULL if not found.
+ */
+static struct ly_ctx_private_data *
+_ly_ctx_private_data_get(const struct ly_ctx *ctx, ly_bool own_data_only)
+{
+    struct ly_ctx_private_data **iter;
+    ly_bool found = 0;
+    pthread_t tid = pthread_self();
+
+    LY_ARRAY_FOR(ly_private_ctx_data, struct ly_ctx_private_data *, iter) {
+        /* either own - ctx and tid match, or "context's" - thread does not matter */
+        if (((*iter)->ctx == ctx) && (!own_data_only || pthread_equal((*iter)->tid, tid))) {
+            found = 1;
+            break;
+        }
+    }
+
+    return found ? *iter : NULL;
+}
+
+struct ly_ctx_private_data *
+ly_ctx_private_data_get_or_create(const struct ly_ctx *ctx)
+{
+    struct ly_ctx_private_data *private_data;
+    LY_ERR r;
+
+    while (ctx->parent_ctx) {
+        /* find the right context */
+        ctx = ctx->parent_ctx;
+    }
+
+    /* RD LOCK */
+    pthread_rwlock_rdlock(&ly_ctx_data_rwlock);
+
+    /* try to find existing data */
+    private_data = _ly_ctx_private_data_get(ctx, 1);
+
+    /* RD UNLOCK */
+    pthread_rwlock_unlock(&ly_ctx_data_rwlock);
+
+    if (private_data) {
+        return private_data;
+    }
+
+    /* WR LOCK */
+    pthread_rwlock_wrlock(&ly_ctx_data_rwlock);
+
+    /* create it */
+    r = ly_ctx_private_data_create(ctx, &private_data);
+    if (r) {
+        /* NULL to avoid infinite loop in LOGERR */
+        LOGERR(NULL, r, "Failed to create context private data.");
+        assert(0);
+    }
+
+    /* WR UNLOCK */
+    pthread_rwlock_unlock(&ly_ctx_data_rwlock);
+
+    return private_data;
+}
+
+/**
+ * @brief Remove shared context data from the sized array and free its contents.
+ *
+ * @param[in] shared_data Shared context data to free.
+ */
+static void
+ly_ctx_shared_data_remove_and_free(struct ly_ctx_shared_data *shared_data)
+{
+    LY_ARRAY_COUNT_TYPE u;
+
+    if (!shared_data) {
+        return;
+    }
+
+    /* free all the cached pattern pcodes */
+    lyht_free(shared_data->pattern_ht, ly_ctx_ht_pattern_free_cb);
+
+    /* free rest of the members */
+    lydict_clean(shared_data->data_dict);
+    free(shared_data->data_dict);
+    lyht_free(shared_data->leafref_links_ht, ly_ctx_ht_leafref_links_rec_free);
+    free(shared_data);
+
+    /* find */
+    LY_ARRAY_FOR(ly_shared_ctx_data, u) {
+        if (ly_shared_ctx_data[u] == shared_data) {
+            break;
+        }
+    }
+    assert(u < LY_ARRAY_COUNT(ly_shared_ctx_data));
+
+    /* remove */
+    if (u < LY_ARRAY_COUNT(ly_shared_ctx_data) - 1) {
+        /* replace the shared data with the last one */
+        ly_shared_ctx_data[u] = ly_shared_ctx_data[LY_ARRAY_COUNT(ly_shared_ctx_data) - 1];
+    }
+    LY_ARRAY_DECREMENT_FREE(ly_shared_ctx_data);
+}
+
+/**
+ * @brief Create the shared context data for a specific context.
+ *
+ * @param[in] ctx Context to create the shared data for.
+ * @param[out] shared_data Optional created shared context data.
+ * @return LY_SUCCESS on success, LY_EMEM on memory allocation failure.
+ */
+static LY_ERR
+ly_ctx_shared_data_create(const struct ly_ctx *ctx, struct ly_ctx_shared_data **shared_data)
+{
+    LY_ERR rc = LY_SUCCESS;
+    struct ly_ctx_shared_data **shrd_data = NULL;
+    pthread_mutexattr_t attr;
+
+    if (shared_data) {
+        *shared_data = NULL;
+    }
+
+    /* create the shared context data */
+    LY_ARRAY_NEW_GOTO(ctx, ly_shared_ctx_data, shrd_data, rc, cleanup);
+    *shrd_data = calloc(1, sizeof **shrd_data);
+    LY_CHECK_ERR_GOTO(!*shrd_data, LOGMEM(ctx); rc = LY_EMEM, cleanup);
+
+    /* fill */
+    (*shrd_data)->ctx = ctx;
+
+    /* pattern hash table */
+    (*shrd_data)->pattern_ht = lyht_new(LYHT_MIN_SIZE, sizeof(struct ly_pattern_ht_rec),
+            ly_ctx_ht_pattern_equal_cb, NULL, 1);
+    LY_CHECK_ERR_GOTO(!(*shrd_data)->pattern_ht, rc = LY_EMEM, cleanup);
+
+    /* data dictionary */
+    (*shrd_data)->data_dict = malloc(sizeof *(*shrd_data)->data_dict);
+    LY_CHECK_ERR_GOTO(!(*shrd_data)->data_dict, rc = LY_EMEM, cleanup);
+    lydict_init((*shrd_data)->data_dict);
+
+    /* leafref set */
+    if (ctx->opts & LY_CTX_LEAFREF_LINKING) {
+        /**
+         * storing the pointer instead of record itself is needed to avoid invalid memory reads. Hash table can reallocate
+         * its memory completely during various manipulation function (e.g. remove, insert). In case of using pointers, the
+         * pointer can be reallocated safely, while record itself remains untouched and can be accessed/modified freely
+         * */
+        (*shrd_data)->leafref_links_ht = lyht_new(1, sizeof(struct lyd_leafref_links_rec *),
+                ly_ctx_ht_leafref_links_equal_cb, NULL, 1);
+        LY_CHECK_ERR_GOTO(!(*shrd_data)->leafref_links_ht, rc = LY_EMEM, cleanup);
+    }
+
+    /* ext clb and leafref links locks */
+    pthread_mutexattr_init(&attr);
+    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
+    pthread_mutex_init(&(*shrd_data)->ext_clb_lock, &attr);
+    pthread_mutexattr_destroy(&attr);
+    pthread_mutex_init(&(*shrd_data)->leafref_links_lock, NULL);
+
+    /* refcount */
+    ATOMIC_STORE_RELAXED((*shrd_data)->refcount, 1);
+
+    if (shared_data) {
+        *shared_data = *shrd_data;
+    }
+
+cleanup:
+    if (rc && shrd_data) {
+        ly_ctx_shared_data_remove_and_free(*shrd_data);
+    }
+    return rc;
+}
+
+/**
+ * @brief Get the shared context data for a specific context.
+ *
+ * @param[in] ctx Context whose shared data to get.
+ * @return Found shared ctx data, NULL if not found.
+ */
+static struct ly_ctx_shared_data *
+_ly_ctx_shared_data_get(const struct ly_ctx *ctx)
+{
+    struct ly_ctx_shared_data **iter;
+    ly_bool found = 0;
+
+    LY_ARRAY_FOR(ly_shared_ctx_data, struct ly_ctx_shared_data *, iter) {
+        if ((*iter)->ctx == ctx) {
+            found = 1;
+            break;
+        }
+    }
+
+    return found ? *iter : NULL;
+}
+
+struct ly_ctx_shared_data *
+ly_ctx_shared_data_get(const struct ly_ctx *ctx)
+{
+    struct ly_ctx_shared_data *shared_data;
+
+    while (ctx->parent_ctx) {
+        /* find the right context */
+        ctx = ctx->parent_ctx;
+    }
+
+    /* RD LOCK */
+    pthread_rwlock_rdlock(&ly_ctx_data_rwlock);
+
+    shared_data = _ly_ctx_shared_data_get(ctx);
+
+    /* RD UNLOCK */
+    pthread_rwlock_unlock(&ly_ctx_data_rwlock);
+
+    if (!shared_data) {
+        /* NULL to avoid infinite loop in LOGERR */
+        LOGERR(NULL, LY_EINT, "Context shared data not found.");
+        assert(0);
+    }
+    return shared_data;
+}
+
+struct ly_dict *
+ly_ctx_data_dict_get(const struct ly_ctx *ctx)
+{
+    struct ly_ctx_shared_data *shared_data;
+
+    shared_data = ly_ctx_shared_data_get(ctx);
+    return shared_data ? shared_data->data_dict : NULL;
+}
+
+LY_ERR
+ly_ctx_data_add(const struct ly_ctx *ctx)
+{
+    LY_ERR rc = LY_SUCCESS;
+    struct ly_ctx_private_data *private_data;
+    struct ly_ctx_shared_data *shared_data;
+
+    /* WR LOCK */
+    pthread_rwlock_wrlock(&ly_ctx_data_rwlock);
+
+    /* check for duplicates in private context data, not allowed */
+    private_data = _ly_ctx_private_data_get(ctx, 1);
+    if (private_data) {
+        /* ctx pointers can match only if the context is printed (they have the same memory address) */
+        assert(private_data->ctx->opts & LY_CTX_INT_IMMUTABLE);
+
+        /* use NULL as ctx to avoid RD lock while holding WR lock */
+        LOGERR(NULL, LY_EEXIST, "Only one printed context per memory chunk and thread is allowed.");
+        rc = LY_EEXIST;
+        goto cleanup;
+    }
+
+    /* create the private context data, freed by the caller in case we fail later on */
+    rc = ly_ctx_private_data_create(ctx, &private_data);
+    if (rc) {
+        /* NULL to avoid infinite loop in LOGERR */
+        LOGERR(NULL, rc, "Failed to create context private data.");
+        goto cleanup;
+    }
+
+    /* check for duplicates in shared context data */
+    shared_data = _ly_ctx_shared_data_get(ctx);
+    if (shared_data) {
+        /* found, we can end */
+        ATOMIC_INC_RELAXED(shared_data->refcount);
+        goto cleanup;
+    }
+
+    /* create the shared context data */
+    rc = ly_ctx_shared_data_create(ctx, NULL);
+    if (rc) {
+        /* NULL to avoid infinite loop in LOGERR */
+        LOGERR(NULL, rc, "Failed to create context shared data.");
+        goto cleanup;
+    }
+
+cleanup:
+    /* WR UNLOCK */
+    pthread_rwlock_unlock(&ly_ctx_data_rwlock);
+    return rc;
+}
+
+void
+ly_ctx_data_del(const struct ly_ctx *ctx)
+{
+    struct ly_ctx_private_data *private_data;
+    struct ly_ctx_shared_data *shared_data;
+
+    /* WR LOCK */
+    pthread_rwlock_wrlock(&ly_ctx_data_rwlock);
+
+    /* free private data of all the threads for this context */
+    while ((private_data = _ly_ctx_private_data_get(ctx, 0))) {
+        /* free the private data */
+        ly_ctx_private_data_remove_and_free(private_data);
+    }
+
+    /* get the shared context data */
+    shared_data = _ly_ctx_shared_data_get(ctx);
+    if (!shared_data) {
+        /* not found, nothing to do */
+        goto cleanup;
+    }
+    assert(ATOMIC_LOAD_RELAXED(shared_data->refcount));
+
+    /* decrease refcount */
+    ATOMIC_DEC_RELAXED(shared_data->refcount);
+    if (ATOMIC_LOAD_RELAXED(shared_data->refcount)) {
+        goto cleanup;
+    }
+
+    /* free the shared members */
+    ly_ctx_shared_data_remove_and_free(shared_data);
+
+cleanup:
+    /* WR UNLOCK */
+    pthread_rwlock_unlock(&ly_ctx_data_rwlock);
+}
+
+LY_ERR
+ly_ctx_shared_data_pattern_get(const struct ly_ctx *ctx, const char *pattern, const pcre2_code **pcode)
+{
+    LY_ERR rc = LY_SUCCESS;
+    struct ly_ctx_shared_data *ctx_data;
+    struct ly_pattern_ht_rec rec = {0}, *found_rec = NULL;
+    uint32_t hash;
+    pcre2_code *pcode_tmp = NULL;
+
+    assert(ctx && pattern);
+
+    if (pcode) {
+        *pcode = NULL;
+    }
+
+    ctx_data = ly_ctx_shared_data_get(ctx);
+    LY_CHECK_RET(!ctx_data, LY_EINT);
+
+    /* try to find the pattern code in the pattern ht */
+    hash = lyht_hash(pattern, strlen(pattern));
+    rec.pattern = pattern;
+    if (!lyht_find(ctx_data->pattern_ht, &rec, hash, (void **)&found_rec)) {
+        /* found it, return it */
+        if (pcode) {
+            *pcode = found_rec->pcode;
+        }
+        goto cleanup;
+    }
+
+    /* didnt find it, need to compile it */
+    LY_CHECK_GOTO(rc = lys_compile_type_pattern_check(ctx, pattern, &pcode_tmp), cleanup);
+
+    /* store the compiled pattern code in the hash table */
+    hash = lyht_hash(pattern, strlen(pattern));
+    rec.pattern = pattern;
+    rec.pcode = pcode_tmp;
+    LY_CHECK_GOTO(rc = lyht_insert_no_check(ctx_data->pattern_ht, &rec, hash, NULL), cleanup);
+
+    if (pcode) {
+        *pcode = pcode_tmp;
+        pcode_tmp = NULL;
+    }
+
+cleanup:
+    if (rc) {
+        /* only free the pcode if we failed, because it belongs to the hash table */
+        pcre2_code_free(pcode_tmp);
+    }
+    return rc;
+}
+
+void
+ly_ctx_shared_data_pattern_del(const struct ly_ctx *ctx, const char *pattern)
+{
+    struct ly_ctx_shared_data *ctx_data;
+    struct ly_pattern_ht_rec rec = {0}, *found_rec = NULL;
+    uint32_t hash;
+
+    assert(ctx && pattern);
+
+    /* get the context shared data */
+    ctx_data = ly_ctx_shared_data_get(ctx);
+    LY_CHECK_ERR_RET(!ctx_data, LOGINT(ctx), );
+
+    /* try to find the pattern code in the pattern ht */
+    hash = lyht_hash(pattern, strlen(pattern));
+    rec.pattern = pattern;
+
+    if (lyht_find(ctx_data->pattern_ht, &rec, hash, (void **)&found_rec)) {
+        /* pattern code not cached yet */
+        return;
+    }
+
+    /* found it, free the pcode */
+    pcre2_code_free(found_rec->pcode);
+
+    /* free the pattern HT record */
+    if (lyht_remove(ctx_data->pattern_ht, &rec, hash)) {
+        LOGINT(ctx);
+    }
+}
+
 void *
 ly_realloc(void *ptr, size_t size)
 {
     void *new_mem;
 
+    /* may cause double-free if NULL is returned */
+    assert(size);
+
     new_mem = realloc(ptr, size);
-    if (!new_mem) {
+    if (!new_mem && size) {
         free(ptr);
     }
 
@@ -159,8 +700,7 @@ LY_ERR
 ly_value_prefix_next(const char *str_begin, const char *str_end, uint32_t *len, ly_bool *is_prefix, const char **str_next)
 {
     const char *stop, *prefix;
-    size_t bytes_read;
-    uint32_t c;
+    uint32_t bytes_read, c;
     ly_bool prefix_found;
     LY_ERR ret = LY_SUCCESS;
 
@@ -223,10 +763,9 @@ ly_value_prefix_next(const char *str_begin, const char *str_end, uint32_t *len,
 }
 
 LY_ERR
-ly_getutf8(const char **input, uint32_t *utf8_char, size_t *bytes_read)
+ly_getutf8(const char **input, uint32_t *utf8_char, uint32_t *bytes_read)
 {
-    uint32_t c, aux;
-    size_t len;
+    uint32_t c, aux, len;
 
     c = (*input)[0];
 
@@ -405,9 +944,9 @@ ly_utf8_greater(const char *input, int bytes, ...)
 }
 
 LY_ERR
-ly_checkutf8(const char *input, size_t in_len, size_t *utf8_len)
+ly_checkutf8(const char *input, uint32_t in_len, uint32_t *utf8_len)
 {
-    size_t len;
+    uint32_t len;
 
     if (!(input[0] & 0x80)) {
         /* one byte character */
@@ -459,7 +998,7 @@ ly_checkutf8(const char *input, size_t in_len, size_t *utf8_len)
 }
 
 LY_ERR
-ly_pututf8(char *dst, uint32_t value, size_t *bytes_written)
+ly_pututf8(char *dst, uint32_t value, uint32_t *bytes_written)
 {
     if (value < 0x80) {
         /* one byte character */
@@ -534,13 +1073,13 @@ static const unsigned char utf8_char_length_table[] = {
     4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 6, 6, 1, 1
 };
 
-size_t
-ly_utf8len(const char *str, size_t bytes)
+uint32_t
+ly_utf8len(const char *str, uint32_t bytes)
 {
-    size_t len = 0;
+    uint32_t len = 0;
     const char *ptr = str;
 
-    while (((size_t)(ptr - str) < bytes) && *ptr) {
+    while (((uint32_t)(ptr - str) < bytes) && *ptr) {
         ++len;
         ptr += utf8_char_length_table[((unsigned char)(*ptr))];
     }
@@ -721,7 +1260,7 @@ ly_strcat(char **dest, const char *format, ...)
 }
 
 LY_ERR
-ly_parse_int(const char *val_str, size_t val_len, int64_t min, int64_t max, int base, int64_t *ret)
+ly_parse_int(const char *val_str, uint32_t val_len, int64_t min, int64_t max, int base, int64_t *ret)
 {
     LY_ERR rc = LY_SUCCESS;
     char *ptr, *str;
@@ -761,7 +1300,7 @@ ly_parse_int(const char *val_str, size_t val_len, int64_t min, int64_t max, int
 }
 
 LY_ERR
-ly_parse_uint(const char *val_str, size_t val_len, uint64_t max, int base, uint64_t *ret)
+ly_parse_uint(const char *val_str, uint32_t val_len, uint64_t max, int base, uint64_t *ret)
 {
     LY_ERR rc = LY_SUCCESS;
     char *ptr, *str;
@@ -827,7 +1366,7 @@ lys_parse_id(const char **id)
 }
 
 LY_ERR
-ly_parse_nodeid(const char **id, const char **prefix, size_t *prefix_len, const char **name, size_t *name_len)
+ly_parse_nodeid(const char **id, const char **prefix, uint32_t *prefix_len, const char **name, uint32_t *name_len)
 {
     assert(id && *id);
     assert(prefix && prefix_len);
@@ -858,13 +1397,12 @@ ly_parse_nodeid(const char **id, const char **prefix, size_t *prefix_len, const
 }
 
 LY_ERR
-ly_parse_instance_predicate(const char **pred, size_t limit, LYD_FORMAT format,
-        const char **prefix, size_t *prefix_len, const char **id, size_t *id_len, const char **value, size_t *value_len,
-        const char **errmsg)
+ly_parse_instance_predicate(const char **pred, uint32_t limit, LYD_FORMAT format, const char **prefix, uint32_t *prefix_len,
+        const char **id, uint32_t *id_len, const char **value, uint32_t *value_len, const char **errmsg)
 {
     LY_ERR ret = LY_EVALID;
     const char *in = *pred;
-    size_t offset = 1;
+    uint32_t offset = 1;
     uint8_t expr = 0; /* 0 - position predicate; 1 - leaf-list-predicate; 2 - key-predicate */
     char quot;
 
