@@ -4,7 +4,7 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief Data parsers for libyang
  *
- * Copyright (c) 2015-2023 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -149,7 +149,21 @@ struct ly_in;
 #define LYD_PARSE_OPAQ      0x040000        /**< Instead of silently ignoring data without definition, parse them into
                                                  an opaq node. Do not combine with ::LYD_PARSE_STRICT (except for ::LYD_LYB). */
 #define LYD_PARSE_NO_STATE  0x080000        /**< Forbid state data in the parsed data. Usually used with ::LYD_VALIDATE_NO_STATE. */
-#define LYD_PARSE_LYB_MOD_UPDATE  0x100000  /**< Deprecated, ignored. */
+#define LYD_PARSE_LYB_SKIP_CTX_CHECK 0x100000   /**< Normally, when printing LYB data, the context hash is written into
+                                                     the data that describes the context (see ::ly_ctx_get_modules_hash()).
+                                                     Then, when parsing the data, the hash is compared with the current
+                                                     context and an error is generated if they do not match. The reason
+                                                     for that is that the data may be parsed incorrectly possibly
+                                                     leading to a crash. However, this hash restriction is too severe
+                                                     and the hash must match only with regard to the modules with actual
+                                                     YANG data in the LYB data to guarantee correct data parsing. In other
+                                                     words, if there is a module A in the printing context but no data
+                                                     of this module in the LYB data, the parsing context does not have
+                                                     to have module A loaded but the hash check will fail in that case.
+                                                     So, if you are able to guarantee that the parsing context will
+                                                     have all the required YANG modules in the same state as in the
+                                                     printed context you can use this flag to effectively soften the
+                                                     context restriction. */
 #define LYD_PARSE_ORDERED 0x200000          /**< Do not search for the correct place of each node but instead expect
                                                  that the nodes are being parsed in the correct schema-based order,
                                                  which is always true if the data were printed by libyang and not
@@ -218,8 +232,9 @@ struct ly_in;
                                                  (such as NP containers) are still added. Validation will fail if a
                                                  default node is required for it to pass. */
 #define LYD_VALIDATE_NOT_FINAL 0x0020       /**< Skip final validation tasks that require for all the data nodes to
-                                                 either exist or not, based on the YANG constraints. Once the data
-                                                 satisfy this requirement, the final validation should be performed. */
+                                                 either exist or not, based on the YANG constraints (including skipping
+                                                 type plugin validate_tree callbacks). Once the data satisfy this
+                                                 requirement, the final validation should be performed. */
 
 #define LYD_VALIDATE_OPTS_MASK  0x0000FFFF  /**< Mask for all the LYD_VALIDATE_* options. */
 
@@ -423,13 +438,15 @@ enum lyd_type {
  * @param[in] in Input handle to read the input from.
  * @param[in] format Expected format of the data in @p in.
  * @param[in] data_type Expected operation to parse (@ref datatype).
+ * @param[in] parse_options Options for parser are set based on @p data_type but either of ::LYD_PARSE_STRICT and
+ * ::LYD_PARSE_OPAQ can be added, see @ref dataparseroptions.
  * @param[out] tree Optional full parsed data tree. If @p parent is set, set to NULL.
  * @param[out] op Optional pointer to the operation (action/RPC/notification) node.
  * @return LY_ERR value.
  * @return LY_ENOT if @p data_type is a NETCONF message and the root XML element is not the expected one.
  */
 LIBYANG_API_DECL LY_ERR lyd_parse_op(const struct ly_ctx *ctx, struct lyd_node *parent, struct ly_in *in, LYD_FORMAT format,
-        enum lyd_type data_type, struct lyd_node **tree, struct lyd_node **op);
+        enum lyd_type data_type, uint32_t parse_options, struct lyd_node **tree, struct lyd_node **op);
 
 /**
  * @brief Parse extension data into an operation data tree following only the specification from the given extension instance.
@@ -466,13 +483,30 @@ LIBYANG_API_DECL LY_ERR lyd_parse_op(const struct ly_ctx *ctx, struct lyd_node *
  * @param[in] in Input handle to read the input from.
  * @param[in] format Expected format of the data in @p in.
  * @param[in] data_type Expected operation to parse (@ref datatype).
+ * @param[in] parse_options Options for parser are set based on @p data_type but either of ::LYD_PARSE_STRICT and
+ * ::LYD_PARSE_OPAQ can be added, see @ref dataparseroptions.
  * @param[out] tree Optional full parsed data tree. If @p parent is set, set to NULL.
  * @param[out] op Optional pointer to the operation (action/RPC) node.
  * @return LY_ERR value.
  * @return LY_ENOT if @p data_type is a NETCONF message and the root XML element is not the expected one.
  */
 LIBYANG_API_DECL LY_ERR lyd_parse_ext_op(const struct lysc_ext_instance *ext, struct lyd_node *parent, struct ly_in *in,
-        LYD_FORMAT format, enum lyd_type data_type, struct lyd_node **tree, struct lyd_node **op);
+        LYD_FORMAT format, enum lyd_type data_type, uint32_t parse_options, struct lyd_node **tree, struct lyd_node **op);
+
+/**
+ * @brief Validate a data subtree of an extension instance, which is assumed to be a separate data tree independent of
+ * normal YANG data.
+ *
+ * @param[in,out] ext_tree Ext data tree to validate. May be changed by validation, might become NULL.
+ * @param[in] ext Extension instance whose data to validate.
+ * @param[in] val_opts Validation options (@ref datavalidationoptions).
+ * @param[in] dep_tree Tree to be used for validating references from the operation subtree.
+ * @param[out] diff Optional diff with any changes made by the validation.
+ * @return LY_SUCCESS on success.
+ * @return LY_ERR error on error.
+ */
+LIBYANG_API_DECL LY_ERR lyd_validate_ext(struct lyd_node **ext_tree, const struct lysc_ext_instance *ext,
+        uint32_t val_opts, const struct lyd_node *dep_tree, struct lyd_node **diff);
 
 /**
  * @brief Fully validate a data tree.
