@@ -16,6 +16,7 @@
 #ifndef LY_PARSER_INTERNAL_H_
 #define LY_PARSER_INTERNAL_H_
 
+#include "lyb.h"
 #include "parser_data.h"
 #include "set.h"
 
@@ -61,8 +62,9 @@ typedef void (*lyd_ctx_free_clb)(struct lyd_ctx *ctx);
 #define LYD_INTOPT_NOTIF            0x08    /**< Notification is being parsed. */
 #define LYD_INTOPT_ANY              0x10    /**< Anydata/anyxml content is being parsed, there can be anything. */
 #define LYD_INTOPT_WITH_SIBLINGS    0x20    /**< Parse the whole input with any siblings. */
-#define LYD_INTOPT_NO_SIBLINGS      0x40    /**< If there are any siblings, return an error. */
-#define LYD_INTOPT_EVENTTIME        0x80    /**< Parse notification eventTime node. */
+#define LYD_INTOPT_SKIP_SIBLINGS    0x40    /**< Perform the validation task only for the specfic subtree, skip other siblings. */
+#define LYD_INTOPT_NO_SIBLINGS      0x80    /**< If there are any siblings, return an error. */
+#define LYD_INTOPT_EVENTTIME        0x0100  /**< Parse notification eventTime node. */
 
 /**
  * @brief Internal (common) context for YANG data parsers.
@@ -81,7 +83,6 @@ struct lyd_ctx {
     struct ly_set node_when;       /**< set of nodes with "when" conditions */
     struct ly_set node_types;      /**< set of nodes validated with LY_EINCOMPLETE result */
     struct ly_set meta_types;      /**< set of metadata validated with LY_EINCOMPLETE result */
-    struct ly_set ext_node;        /**< set of nodes with extension instances to validate */
     struct ly_set ext_val;         /**< set of nested subtrees parsed by extensions to validate */
     struct lyd_node *op_node;      /**< if an RPC/action/notification is being parsed, store the pointer to it */
     const struct lys_module *val_getnext_ht_mod;    /**< module of the cached schema nodes in getnext HT */
@@ -110,7 +111,6 @@ struct lyd_xml_ctx {
     struct ly_set node_when;
     struct ly_set node_types;
     struct ly_set meta_types;
-    struct ly_set ext_node;
     struct ly_set ext_val;
     struct lyd_node *op_node;
     const struct lys_module *val_getnext_ht_mod;
@@ -135,7 +135,6 @@ struct lyd_json_ctx {
     struct ly_set node_when;
     struct ly_set node_types;
     struct ly_set meta_types;
-    struct ly_set ext_node;
     struct ly_set ext_val;
     struct lyd_node *op_node;
     const struct lys_module *val_getnext_ht_mod;
@@ -167,7 +166,6 @@ struct lyd_lyb_ctx {
     struct ly_set node_when;
     struct ly_set node_types;
     struct ly_set meta_types;
-    struct ly_set ext_node;
     struct ly_set ext_val;
     struct lyd_node *op_node;
     const struct lys_module *val_getnext_ht_mod;
@@ -176,7 +174,10 @@ struct lyd_lyb_ctx {
     /* callbacks */
     lyd_ctx_free_clb free;
 
-    struct lylyb_ctx *lybctx;      /* LYB context */
+    union {
+        struct lylyb_print_ctx *print_ctx;
+        struct lylyb_parse_ctx *parse_ctx;
+    };
 };
 
 /**
@@ -187,14 +188,6 @@ struct lyd_ctx_ext_val {
     struct lyd_node *sibling;
 };
 
-/**
- * @brief Parsed data node with extension instance to validate.
- */
-struct lyd_ctx_ext_node {
-    struct lysc_ext_instance *ext;
-    struct lyd_node *node;
-};
-
 /**
  * @brief Common part to supplement the specific ::lyd_ctx_free_clb callbacks.
  */
@@ -390,7 +383,7 @@ LY_ERR lyd_parser_check_schema(struct lyd_ctx *lydctx, const struct lysc_node *s
  * @param[in] lydctx Data parser context.
  * @param[in] schema Schema node of the new data node.
  * @param[in] value String value to be parsed.
- * @param[in] value_len Length of @p value, must be set correctly.
+ * @param[in] value_size_bits Size of @p value in bits, must be set correctly.
  * @param[in,out] dynamic Flag if @p value is dynamically allocated, is adjusted when @p value is consumed.
  * @param[in] format Input format of @p value.
  * @param[in] prefix_data Format-specific data for resolving any prefixes (see ::ly_resolve_prefix).
@@ -400,8 +393,9 @@ LY_ERR lyd_parser_check_schema(struct lyd_ctx *lydctx, const struct lysc_node *s
  * @return LY_EINCOMPLETE in case data tree is needed to finish the validation.
  * @return LY_ERR value if an error occurred.
  */
-LY_ERR lyd_parser_create_term(struct lyd_ctx *lydctx, const struct lysc_node *schema, const void *value, size_t value_len,
-        ly_bool *dynamic, LY_VALUE_FORMAT format, void *prefix_data, uint32_t hints, struct lyd_node **node);
+LY_ERR lyd_parser_create_term(struct lyd_ctx *lydctx, const struct lysc_node *schema, const void *value,
+        uint32_t value_size_bits, ly_bool *dynamic, LY_VALUE_FORMAT format, void *prefix_data, uint32_t hints,
+        struct lyd_node **node);
 
 /**
  * @brief Wrapper around ::lyd_create_meta() for data parsers.
@@ -413,7 +407,7 @@ LY_ERR lyd_parser_create_term(struct lyd_ctx *lydctx, const struct lysc_node *sc
  * @param[in] name Metadata name.
  * @param[in] name_len Length of @p name.
  * @param[in] value Metadata value.
- * @param[in] value_len Length of @p value.
+ * @param[in] value_size_bits Size of @p value in bits.
  * @param[in,out] dynamic Whether the @p value is dynamically allocated, is adjusted once the value is assigned.
  * @param[in] format Prefix format.
  * @param[in] prefix_data Prefix format data (see ::ly_resolve_prefix()).
@@ -422,7 +416,7 @@ LY_ERR lyd_parser_create_term(struct lyd_ctx *lydctx, const struct lysc_node *sc
  * @return LY_ERR value.
  */
 LY_ERR lyd_parser_create_meta(struct lyd_ctx *lydctx, struct lyd_node *parent, struct lyd_meta **meta,
-        const struct lys_module *mod, const char *name, size_t name_len, const void *value, size_t value_len,
+        const struct lys_module *mod, const char *name, uint32_t name_len, const void *value, uint32_t value_size_bits,
         ly_bool *dynamic, LY_VALUE_FORMAT format, void *prefix_data, uint32_t hints, const struct lysc_node *ctx_node);
 
 /**
