@@ -108,7 +108,7 @@ static pthread_mutex_t plugins_guard = PTHREAD_MUTEX_INITIALIZER;
  * unloaded with the destroy of the last context. Therefore, to reload the list of plugins, all the contexts must be
  * destroyed and with the creation of a first new context after that, the plugins will be reloaded.
  */
-static uint32_t context_refcount = 0;
+static uint32_t context_refcount;
 
 /**
  * @brief Record describing an implemented extension.
@@ -127,17 +127,55 @@ struct lyplg_record {
 };
 
 #ifndef STATIC
-static struct ly_set plugins_handlers = {0};
+static struct ly_set plugins_handlers;
 #endif
-static struct ly_set plugins_types = {0};
-static struct ly_set plugins_extensions = {0};
+
+/* global sets of loaded plugins */
+struct ly_set ly_plugins_types;
+struct ly_set ly_plugins_extensions;
+
+/* global counters for the number of static plugins */
+uint32_t ly_static_type_plugins_count;
+uint32_t ly_static_ext_plugins_count;
+
+LIBYANG_API_DEF struct lyplg_type *
+lysc_get_type_plugin(uintptr_t plugin_ref)
+{
+    if (!plugin_ref) {
+        return NULL;
+    }
+
+    if (plugin_ref <= ly_static_type_plugins_count) {
+        /* plugin is static, fetch it from the global list */
+        return &((struct lyplg_type_record *)ly_plugins_types.objs[plugin_ref - 1])->plugin;
+    } else {
+        /* plugin is dynamic, return the pointer */
+        return (struct lyplg_type *)plugin_ref;
+    }
+}
+
+LIBYANG_API_DEF struct lyplg_ext *
+lysc_get_ext_plugin(uintptr_t plugin_ref)
+{
+    if (!plugin_ref) {
+        return NULL;
+    }
+
+    if (plugin_ref <= ly_static_ext_plugins_count) {
+        /* plugin is static, fetch it from the global list */
+        return &((struct lyplg_ext_record *)ly_plugins_extensions.objs[plugin_ref - 1])->plugin;
+    } else {
+        /* plugin is dynamic, return the pointer */
+        return (struct lyplg_ext *)plugin_ref;
+    }
+}
 
 /**
  * @brief Iterate over list of loaded plugins of the given @p type.
  *
- * @param[in] ctx The context for which the plugin is searched for
+ * @param[in] ctx Context for which the plugin is searched for.
  * @param[in] type Type of the plugins to iterate.
- * @param[in,out] index The iterator - set to 0 for the first call.
+ * @param[in,out] index Iterator - set to 0 for the first call.
  * @return The plugin records, NULL if no more record is available.
  */
 static struct lyplg_record *
@@ -148,9 +186,9 @@ plugins_iter(const struct ly_ctx *ctx, enum LYPLG type, uint32_t *index)
     assert(index);
 
     if (type == LYPLG_EXTENSION) {
-        plugins = ctx ? &ctx->plugins_extensions : &plugins_extensions;
+        plugins = ctx ? &ctx->plugins_extensions : &ly_plugins_extensions;
     } else {
-        plugins = ctx ? &ctx->plugins_types : &plugins_types;
+        plugins = ctx ? &ctx->plugins_types : &ly_plugins_types;
     }
 
     if (*index == plugins->count) {
@@ -161,8 +199,20 @@ plugins_iter(const struct ly_ctx *ctx, enum LYPLG type, uint32_t *index)
     return plugins->objs[*index - 1];
 }
 
+/**
+ * @brief Find the given @p type plugin record.
+ *
+ * @param[in] ctx Context for which the plugin record is searched for, NULL for static plugins.
+ * @param[in] type Type of the plugin record to find.
+ * @param[in] module Module name of the plugin record.
+ * @param[in] revision Revision of the @p module.
+ * @param[in] name Name of the plugin record.
+ * @param[out] record_idx Optional output parameter to get the index of the found plugin record.
+ * @return Found plugin record or NULL if not found.
+ */
 static void *
-lyplg_record_find(const struct ly_ctx *ctx, enum LYPLG type, const char *module, const char *revision, const char *name)
+lyplg_record_find(const struct ly_ctx *ctx, enum LYPLG type, const char *module, const char *revision, const char *name,
+        uint32_t *record_idx)
 {
     uint32_t i = 0;
     struct lyplg_record *item;
@@ -170,6 +220,10 @@ lyplg_record_find(const struct ly_ctx *ctx, enum LYPLG type, const char *module,
     assert(module);
     assert(name);
 
+    if (record_idx) {
+        *record_idx = 0;
+    }
+
     while ((item = plugins_iter(ctx, type, &i)) != NULL) {
         if (!strcmp(item->module, module) && !strcmp(item->name, name)) {
             if (item->revision && revision && strcmp(item->revision, revision)) {
@@ -178,6 +232,11 @@ lyplg_record_find(const struct ly_ctx *ctx, enum LYPLG type, const char *module,
                 continue;
             }
 
+            if (record_idx) {
+                /* return the record idx, -1 because the iterator is already increased */
+                *record_idx = i - 1;
+            }
+
             return item;
         }
     }
@@ -185,40 +244,59 @@ lyplg_record_find(const struct ly_ctx *ctx, enum LYPLG type, const char *module,
     return NULL;
 }
 
-struct lyplg_type *
-lyplg_type_plugin_find(const struct ly_ctx *ctx, const char *module, const char *revision, const char *name)
+/**
+ * @brief Find the plugin of the given @p type.
+ *
+ * @param[in] ctx Context for which the plugin record is searched for, NULL for static plugins.
+ * @param[in] type Type of the plugin record to find.
+ * @param[in] module Module name of the plugin.
+ * @param[in] revision Revision of the @p module.
+ * @param[in] name Name of the plugin.
+ * @return Reference to the callbacks plugin structure. Use ::LYSC_GET_TYPE_PLG
+ * or ::LYSC_GET_EXT_PLG on the returned value to get the actual plugin. 0 if not found.
+ */
+static uintptr_t
+lyplg_plugin_find(const struct ly_ctx *ctx, enum LYPLG type, const char *module, const char *revision, const char *name)
 {
-    struct lyplg_record *record = NULL;
+    struct lyplg_type_record *record = NULL;
+    uint32_t record_idx = 0, static_plugin_count;
 
     if (ctx) {
         /* try to find context specific plugin */
-        record = lyplg_record_find(ctx, LYPLG_TYPE, module, revision, name);
+        record = lyplg_record_find(ctx, type, module, revision, name, &record_idx);
+        if (record) {
+            /* plugin found in the context, hence it is dynamic so return the ptr to it */
+            return (uintptr_t)&record->plugin;
+        }
     }
 
+    /* try to find shared plugin */
+    record = lyplg_record_find(NULL, type, module, revision, name, &record_idx);
     if (!record) {
-        /* try to find shared plugin */
-        record = lyplg_record_find(NULL, LYPLG_TYPE, module, revision, name);
+        /* not found */
+        return 0;
     }
 
-    return record ? &((struct lyplg_type_record *)record)->plugin : NULL;
+    static_plugin_count = (type == LYPLG_TYPE) ? ly_static_type_plugins_count : ly_static_ext_plugins_count;
+    if (record_idx < static_plugin_count) {
+        /* static plugin, return an index with an offset of +1 in order to keep 0 as an invalid index (a NULL ptr) */
+        return record_idx + 1;
+    } else {
+        /* dynamic plugin, return the pointer to it */
+        return (uintptr_t)&record->plugin;
+    }
 }
 
-struct lyplg_ext_record *
-lyplg_ext_record_find(const struct ly_ctx *ctx, const char *module, const char *revision, const char *name)
+uintptr_t
+lyplg_type_plugin_find(const struct ly_ctx *ctx, const char *module, const char *revision, const char *name)
 {
-    struct lyplg_ext_record *record = NULL;
+    return lyplg_plugin_find(ctx, LYPLG_TYPE, module, revision, name);
+}
 
-    if (ctx) {
-        /* try to find context specific plugin */
-        record = lyplg_record_find(ctx, LYPLG_EXTENSION, module, revision, name);
-    }
-
-    if (!record) {
-        /* try to find shared plugin */
-        record = lyplg_record_find(NULL, LYPLG_EXTENSION, module, revision, name);
-    }
-
-    return record;
+uintptr_t
+lyplg_ext_plugin_find(const struct ly_ctx *ctx, const char *module, const char *revision, const char *name)
+{
+    return lyplg_plugin_find(ctx, LYPLG_EXTENSION, module, revision, name);
 }
 
 /**
@@ -245,7 +323,7 @@ plugins_insert(struct ly_ctx *ctx, enum LYPLG type, const void *recs)
     if (type == LYPLG_EXTENSION) {
         const struct lyplg_ext_record *rec = (const struct lyplg_ext_record *)recs;
 
-        plugins = ctx ? &ctx->plugins_extensions : &plugins_extensions;
+        plugins = ctx ? &ctx->plugins_extensions : &ly_plugins_extensions;
 
         for (uint32_t i = 0; rec[i].name; i++) {
             LY_CHECK_RET(ly_set_add(plugins, (void *)&rec[i], 0, NULL));
@@ -253,7 +331,7 @@ plugins_insert(struct ly_ctx *ctx, enum LYPLG type, const void *recs)
     } else { /* LYPLG_TYPE */
         const struct lyplg_type_record *rec = (const struct lyplg_type_record *)recs;
 
-        plugins = ctx ? &ctx->plugins_types : &plugins_types;
+        plugins = ctx ? &ctx->plugins_types : &ly_plugins_types;
 
         for (uint32_t i = 0; rec[i].name; i++) {
             LY_CHECK_RET(ly_set_add(plugins, (void *)&rec[i], 0, NULL));
@@ -279,9 +357,10 @@ lyplg_clean_(void)
         return;
     }
 
-    ly_set_erase(&plugins_types, NULL);
-    ly_set_erase(&plugins_extensions, NULL);
+    ly_set_erase(&ly_plugins_types, NULL);
+    ly_set_erase(&ly_plugins_extensions, NULL);
     ly_set_erase(&plugins_handlers, lyplg_close_cb);
+    ly_static_type_plugins_count = ly_static_ext_plugins_count = 0;
 }
 
 #endif
@@ -396,8 +475,8 @@ plugins_load_module(const char *pathname)
     }
 
     /* remember the current plugins lists for recovery */
-    types_count = plugins_types.count;
-    extensions_count = plugins_extensions.count;
+    types_count = ly_plugins_types.count;
+    extensions_count = ly_plugins_extensions.count;
 
     /* type plugin */
     ret = plugins_load(dlhandler, pathname, LYPLG_TYPE);
@@ -417,11 +496,11 @@ error:
     dlclose(dlhandler);
 
     /* revert changes in the lists */
-    while (plugins_types.count > types_count) {
-        ly_set_rm_index(&plugins_types, plugins_types.count - 1, NULL);
+    while (ly_plugins_types.count > types_count) {
+        ly_set_rm_index(&ly_plugins_types, ly_plugins_types.count - 1, NULL);
     }
-    while (plugins_extensions.count > extensions_count) {
-        ly_set_rm_index(&plugins_extensions, plugins_extensions.count - 1, NULL);
+    while (ly_plugins_extensions.count > extensions_count) {
+        ly_set_rm_index(&ly_plugins_extensions, ly_plugins_extensions.count - 1, NULL);
     }
 
     return ret;
@@ -480,12 +559,13 @@ plugins_insert_dir(enum LYPLG type)
 #endif
 
 LY_ERR
-lyplg_init(ly_bool builtin_type_plugins_only)
+lyplg_init(ly_bool builtin_type_plugins_only, ly_bool static_plugins_only)
 {
     LY_ERR ret;
 
-    pthread_mutex_lock(&plugins_guard);
     /* let only the first context to initiate plugins, but let others wait for finishing the initiation */
+    pthread_mutex_lock(&plugins_guard);
+
     if (context_refcount++) {
         /* already initiated */
         pthread_mutex_unlock(&plugins_guard);
@@ -532,20 +612,27 @@ lyplg_init(ly_bool builtin_type_plugins_only)
 
         /* ietf-netconf-acm */
         LY_CHECK_GOTO(ret = plugins_insert(NULL, LYPLG_TYPE, plugins_node_instanceid), error);
-
-        /* internal extensions */
-        LY_CHECK_GOTO(ret = plugins_insert(NULL, LYPLG_EXTENSION, plugins_nacm), error);
-        LY_CHECK_GOTO(ret = plugins_insert(NULL, LYPLG_EXTENSION, plugins_yangdata), error);
-        LY_CHECK_GOTO(ret = plugins_insert(NULL, LYPLG_EXTENSION, plugins_schema_mount), error);
-        LY_CHECK_GOTO(ret = plugins_insert(NULL, LYPLG_EXTENSION, plugins_structure), error);
     }
 
-#ifndef STATIC
-    /* external types */
-    LY_CHECK_GOTO(ret = plugins_insert_dir(LYPLG_TYPE), error);
+    /* internal extensions */
+    LY_CHECK_GOTO(ret = plugins_insert(NULL, LYPLG_EXTENSION, plugins_nacm), error);
+    LY_CHECK_GOTO(ret = plugins_insert(NULL, LYPLG_EXTENSION, plugins_yangdata), error);
+    LY_CHECK_GOTO(ret = plugins_insert(NULL, LYPLG_EXTENSION, plugins_schema_mount), error);
+    LY_CHECK_GOTO(ret = plugins_insert(NULL, LYPLG_EXTENSION, plugins_structure), error);
 
-    /* external extensions */
-    LY_CHECK_GOTO(ret = plugins_insert_dir(LYPLG_EXTENSION), error);
+    /* the global plugin sets contain only static plugins at this point, so assign to the counters here.
+     * the counters are used to determine whether a plugin is static or not */
+    ly_static_type_plugins_count = ly_plugins_types.count;
+    ly_static_ext_plugins_count = ly_plugins_extensions.count;
+
+#ifndef STATIC
+    if (!static_plugins_only) {
+        /* external types */
+        LY_CHECK_GOTO(ret = plugins_insert_dir(LYPLG_TYPE), error);
+
+        /* external extensions */
+        LY_CHECK_GOTO(ret = plugins_insert_dir(LYPLG_EXTENSION), error);
+    }
 #endif
 
     /* initiation done, wake-up possibly waiting threads creating another contexts */
