@@ -4,7 +4,7 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief helper functions for extension plugins
  *
- * Copyright (c) 2019 - 2024 CESNET, z.s.p.o.
+ * Copyright (c) 2019 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -67,7 +67,7 @@ lyplg_ext_parse_extension_instance(struct lysp_ctx *pctx, struct lysp_ext_instan
                 continue;
             }
 
-            if ((rc = lys_parse_ext_instance_stmt(pctx, &ext->substmts[u], stmt))) {
+            if ((rc = lys_parser_ext_instance_stmt(pctx, &ext->substmts[u], stmt))) {
                 goto cleanup;
             }
         }
@@ -77,18 +77,43 @@ cleanup:
     return rc;
 }
 
+LIBYANG_API_DEF struct ly_ctx *
+lyplg_ext_compile_get_ctx(const struct lysc_ctx *ctx)
+{
+    return ctx->ctx;
+}
+
+LIBYANG_API_DEF uint32_t *
+lyplg_ext_compile_get_options(const struct lysc_ctx *ctx)
+{
+    return &((struct lysc_ctx *)ctx)->compile_opts;
+}
+
+LIBYANG_API_DEF const struct lys_module *
+lyplg_ext_compile_get_cur_mod(const struct lysc_ctx *ctx)
+{
+    return ctx->cur_mod;
+}
+
+LIBYANG_API_DEF struct lysp_module *
+lyplg_ext_compile_get_pmod(const struct lysc_ctx *ctx)
+{
+    return ctx->pmod;
+}
+
 /**
  * @brief Compile an instance extension statement.
  *
  * @param[in] ctx Compile context.
  * @param[in] parsed_p Parsed ext instance substatement structure.
  * @param[in] ext Compiled ext instance.
+ * @param[in] parent Optional parent of all the compiled schema nodes.
  * @param[in] substmt Compled ext instance substatement info.
  * @return LY_ERR value.
  */
 static LY_ERR
 lys_compile_ext_instance_stmt(struct lysc_ctx *ctx, void **parsed_p, struct lysc_ext_instance *ext,
-        struct lysc_ext_substmt *substmt)
+        struct lysc_node *parent, struct lysc_ext_substmt *substmt)
 {
     LY_ERR rc = LY_SUCCESS;
     ly_bool length_restr = 0;
@@ -144,8 +169,8 @@ lys_compile_ext_instance_stmt(struct lysc_ctx *ctx, void **parsed_p, struct lysc
                 LY_CHECK_GOTO(rc = lys_compile_node_action_inout(ctx, pnode, node), cleanup);
                 LY_CHECK_GOTO(rc = lys_compile_node_connect(ctx, NULL, node), cleanup);
             } else {
-                /* ctx->ext substatement storage is used as the document root */
-                LY_CHECK_GOTO(rc = lys_compile_node(ctx, pnode, NULL, flags, NULL), cleanup);
+                /* if parent is NULL, ctx->ext substatement storage is used as the document root */
+                LY_CHECK_GOTO(rc = lys_compile_node(ctx, pnode, parent, flags, NULL), cleanup);
             }
         }
         break;
@@ -163,7 +188,7 @@ lys_compile_ext_instance_stmt(struct lysc_ctx *ctx, void **parsed_p, struct lysc
     case LY_STMT_REFERENCE:
     case LY_STMT_UNITS:
         /* just make a copy */
-        LY_CHECK_GOTO(rc = lydict_insert(ctx->ctx, *parsed_p, 0, (const char **)substmt->storage_p), cleanup);
+        LY_CHECK_GOTO(rc = lysdict_insert(ctx->ctx, *parsed_p, 0, (const char **)substmt->storage_p), cleanup);
         break;
 
     case LY_STMT_BIT:
@@ -325,7 +350,7 @@ cleanup:
 
 LIBYANG_API_DEF LY_ERR
 lyplg_ext_compile_extension_instance(struct lysc_ctx *ctx, const struct lysp_ext_instance *extp,
-        struct lysc_ext_instance *ext)
+        struct lysc_ext_instance *ext, struct lysc_node *parent)
 {
     LY_ERR rc = LY_SUCCESS;
     LY_ARRAY_COUNT_TYPE u, v;
@@ -352,7 +377,7 @@ lyplg_ext_compile_extension_instance(struct lysc_ctx *ctx, const struct lysp_ext
                 continue;
             }
 
-            if ((rc = lys_compile_ext_instance_stmt(ctx, storagep, ext, &ext->substmts[v]))) {
+            if ((rc = lys_compile_ext_instance_stmt(ctx, storagep, ext, parent, &ext->substmts[v]))) {
                 goto cleanup;
             }
 
@@ -370,28 +395,24 @@ cleanup:
     return rc;
 }
 
-LIBYANG_API_DEF struct ly_ctx *
-lyplg_ext_compile_get_ctx(const struct lysc_ctx *ctx)
+LIBYANG_API_DEF LY_ERR
+lyplg_ext_compiled_node_augments(struct lysc_ctx *ctx, struct lysc_ext_instance *ext, struct lysc_node *node)
 {
-    return ctx->ctx;
-}
+    LY_ERR rc = LY_SUCCESS;
 
-LIBYANG_API_DEF uint32_t *
-lyplg_ext_compile_get_options(const struct lysc_ctx *ctx)
-{
-    return &((struct lysc_ctx *)ctx)->compile_opts;
-}
+    LY_CHECK_ARG_RET(ctx ? ctx->ctx : NULL, ctx, ext, node, LY_EINVAL);
 
-LIBYANG_API_DEF const struct lys_module *
-lyplg_ext_compile_get_cur_mod(const struct lysc_ctx *ctx)
-{
-    return ctx->cur_mod;
-}
+    assert(!ctx->ext);
 
-LIBYANG_API_DEF struct lysp_module *
-lyplg_ext_compile_get_pmod(const struct lysc_ctx *ctx)
-{
-    return ctx->pmod;
+    /* note into the compile context that we are processing extension now */
+    ctx->ext = ext;
+
+    /* connect any augments */
+    LY_CHECK_GOTO(rc = lys_compile_node_augments(ctx, node), cleanup);
+
+cleanup:
+    ctx->ext = NULL;
+    return rc;
 }
 
 LIBYANG_API_DEF struct ly_out **
@@ -412,7 +433,7 @@ lyplg_ext_print_get_level(const struct lyspr_ctx *ctx)
     return &((struct lyspr_ctx *)ctx)->level;
 }
 
-LIBYANG_API_DECL LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_ext_sprinter_ctree_add_ext_nodes(const struct lyspr_tree_ctx *ctx, struct lysc_ext_instance *ext,
         lyplg_ext_sprinter_ctree_override_clb clb)
 {
@@ -450,7 +471,7 @@ lyplg_ext_sprinter_ctree_add_ext_nodes(const struct lyspr_tree_ctx *ctx, struct
     return rc;
 }
 
-LIBYANG_API_DECL LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_ext_sprinter_ptree_add_ext_nodes(const struct lyspr_tree_ctx *ctx, struct lysp_ext_instance *ext,
         lyplg_ext_sprinter_ptree_override_clb clb)
 {
@@ -488,7 +509,7 @@ lyplg_ext_sprinter_ptree_add_ext_nodes(const struct lyspr_tree_ctx *ctx, struct
     return rc;
 }
 
-LIBYANG_API_DECL LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_ext_sprinter_ctree_add_nodes(const struct lyspr_tree_ctx *ctx, struct lysc_node *nodes,
         lyplg_ext_sprinter_ctree_override_clb clb)
 {
@@ -508,7 +529,7 @@ lyplg_ext_sprinter_ctree_add_nodes(const struct lyspr_tree_ctx *ctx, struct lysc
     return LY_SUCCESS;
 }
 
-LIBYANG_API_DECL LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_ext_sprinter_ptree_add_nodes(const struct lyspr_tree_ctx *ctx, struct lysp_node *nodes,
         lyplg_ext_sprinter_ptree_override_clb clb)
 {
@@ -528,7 +549,7 @@ lyplg_ext_sprinter_ptree_add_nodes(const struct lyspr_tree_ctx *ctx, struct lysp
     return LY_SUCCESS;
 }
 
-LIBYANG_API_DECL LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_ext_sprinter_tree_set_priv(const struct lyspr_tree_ctx *ctx, void *plugin_priv, void (*free_clb)(void *plugin_priv))
 {
     LY_CHECK_ARG_RET(NULL, ctx, LY_EINVAL);
@@ -539,6 +560,39 @@ lyplg_ext_sprinter_tree_set_priv(const struct lyspr_tree_ctx *ctx, void *plugin_
     return LY_SUCCESS;
 }
 
+LIBYANG_API_DEF int
+lyplg_ext_compiled_stmts_storage_size(const struct lysc_ext_substmt *substmts, struct ly_ht *addr_ht)
+{
+    LY_CHECK_ARG_RET(NULL, substmts, addr_ht, -1);
+
+    return ly_ctx_compiled_ext_stmts_storage_size(substmts, addr_ht);
+}
+
+LIBYANG_API_DEF LY_ERR
+lyplg_ext_compiled_stmts_storage_print(const struct lysc_ext_substmt *orig_substmts, struct lysc_ext_substmt *substmts,
+        struct ly_ht *addr_ht, struct ly_set *ptr_set, void **mem)
+{
+    LY_CHECK_ARG_RET(NULL, orig_substmts, substmts, addr_ht, ptr_set, mem, LY_EINVAL);
+
+    return ly_ctx_compiled_ext_stmts_storage_print(orig_substmts, substmts, addr_ht, ptr_set, mem);
+}
+
+LIBYANG_API_DEF void *
+lyplg_ext_compiled_print_get_addr(const struct ly_ht *addr_ht, const void *addr)
+{
+    LY_CHECK_ARG_RET(NULL, addr_ht, addr, NULL);
+
+    return ly_ctx_compiled_addr_ht_get(addr_ht, addr, 1);
+}
+
+LIBYANG_API_DEF LY_ERR
+lyplg_ext_compiled_print_add_addr(struct ly_ht *addr_ht, const void *orig_addr, const void *addr)
+{
+    LY_CHECK_ARG_RET(NULL, addr_ht, orig_addr, addr, LY_EINVAL);
+
+    return ly_ctx_compiled_addr_ht_add(addr_ht, orig_addr, addr);
+}
+
 LIBYANG_API_DEF const char *
 lyplg_ext_stmt2str(enum ly_stmt stmt)
 {
@@ -633,6 +687,7 @@ lyplg_ext_parsed_get_storage(const struct lysc_ext_instance *ext, int stmt, uint
 {
     LY_ARRAY_COUNT_TYPE u;
     const struct lysp_ext_instance *extp = NULL;
+    const char *extp_name;
     enum ly_stmt match = 0;
     void **s_p = NULL;
 
@@ -641,8 +696,9 @@ lyplg_ext_parsed_get_storage(const struct lysc_ext_instance *ext, int stmt, uint
     /* find the parsed ext instance */
     LY_ARRAY_FOR(ext->module->parsed->exts, u) {
         extp = &ext->module->parsed->exts[u];
+        extp_name = strchr(extp->name, ':') + 1;
 
-        if (ext->def == extp->def->compiled) {
+        if (!strcmp(ext->def->name, extp_name)) {
             break;
         }
         extp = NULL;
@@ -673,17 +729,72 @@ lyplg_ext_parsed_get_storage(const struct lysc_ext_instance *ext, int stmt, uint
 }
 
 LIBYANG_API_DEF LY_ERR
-lyplg_ext_get_data(const struct ly_ctx *ctx, const struct lysc_ext_instance *ext, void **ext_data, ly_bool *ext_data_free)
+lyplg_ext_get_data(const struct ly_ctx *ctx, const struct lysc_ext_instance *ext, const struct lyd_node *parent,
+        void **ext_data, ly_bool *ext_data_free)
 {
     LY_ERR rc;
+    struct ly_ctx_shared_data *ctx_data;
 
-    if (!ctx->ext_clb) {
+    LY_CHECK_ARG_RET(ctx, ctx, ext, ext_data, LY_EINVAL);
+
+    ctx_data = ly_ctx_shared_data_get(ctx);
+
+    /* EXT CLB LOCK */
+    pthread_mutex_lock(&ctx_data->ext_clb_lock);
+
+    if (!ctx_data->ext_clb) {
         lyplg_ext_compile_log(NULL, ext, LY_LLERR, LY_EINVAL, "Failed to get extension data, no callback set.");
-        return LY_EINVAL;
+        rc = LY_EINVAL;
+        goto cleanup;
     }
 
-    if ((rc = ctx->ext_clb(ext, ctx->ext_clb_data, ext_data, ext_data_free))) {
+    if ((rc = ctx_data->ext_clb(ext, parent, ctx_data->ext_clb_data, ext_data, ext_data_free))) {
         lyplg_ext_compile_log(NULL, ext, LY_LLERR, rc, "Callback for getting ext data failed.");
+        goto cleanup;
     }
+
+cleanup:
+    /* EXT CLB UNLOCK */
+    pthread_mutex_unlock(&ctx_data->ext_clb_lock);
     return rc;
 }
+
+LIBYANG_API_DEF LY_ERR
+lyplg_ext_set_parent_ctx(struct ly_ctx *ctx, const struct ly_ctx *parent_ctx)
+{
+    const struct ly_ctx *c;
+
+    LY_CHECK_ARG_RET(ctx, ctx, LY_EINVAL);
+
+    if (ctx->parent_ctx == parent_ctx) {
+        return LY_SUCCESS;
+    }
+
+    /* prevent circular reference chain */
+    c = parent_ctx;
+    while (c) {
+        if (c->parent_ctx == ctx) {
+            LOGERR(ctx, LY_EDENIED, "Circular references of parent contexts.");
+            return LY_EDENIED;
+        }
+
+        c = c->parent_ctx;
+    }
+
+    if (!ctx->parent_ctx) {
+        if (ly_ctx_is_printed(ctx) && ly_ctx_is_printed(parent_ctx)) {
+            /* completely destroy the printed context, assume it will not be destroyed (in case of schema-mount
+             * contexts, there is no ext callback for freeing the compiled extension data with the contexts) */
+            ly_ctx_destroy(ctx);
+        } else {
+            /* remove its shared and private data */
+            ly_ctx_data_del(ctx);
+        }
+    } else if (!parent_ctx) {
+        /* create its shared and private data */
+        LY_CHECK_RET(ly_ctx_data_add(ctx));
+    }
+
+    ctx->parent_ctx = (struct ly_ctx *)parent_ctx;
+    return LY_SUCCESS;
+}
