@@ -4,7 +4,7 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief libyang support for YANG extensions implementation.
  *
- * Copyright (c) 2015 - 2022 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -109,7 +109,7 @@ extern "C" {
 /**
  * @brief Extensions API version
  */
-#define LYPLG_EXT_API_VERSION 8
+#define LYPLG_EXT_API_VERSION 10
 
 /**
  * @brief Mask for an operation statement.
@@ -405,7 +405,7 @@ struct lysp_ext_instance {
     const char *argument;                   /**< optional value of the extension's argument */
     LY_VALUE_FORMAT format;                 /**< prefix format of the extension name/argument (::LY_VALUE_XML is YIN format) */
     void *prefix_data;                      /**< format-specific data for prefix resolution (see ly_resolve_prefix()) */
-    struct lysp_ext *def;                   /**< pointer to the extension definition */
+    uintptr_t plugin_ref;                   /**< reference to extension plugin, use ::lysc_get_ext_plugin() to get the plugin */
 
     void *parent;                           /**< pointer to the parent statement holding the extension instance(s), use
                                                  ::lysp_ext_instance#parent_stmt to access the value/structure */
@@ -414,7 +414,6 @@ struct lysp_ext_instance {
                                                  statement directly and it is an array */
     uint16_t flags;                         /**< ::LYS_INTERNAL value (@ref snodeflags) */
 
-    const struct lyplg_ext_record *record;  /**< extension definition plugin record, if any */
     struct lysp_ext_substmt *substmts;      /**< list of supported known YANG statements with the pointer to their
                                                  parsed data ([sized array](@ref sizedarrays)) */
     void *parsed;                           /**< private plugin parsed data */
@@ -645,12 +644,27 @@ LIBYANG_API_DECL struct lysp_module *lyplg_ext_compile_get_pmod(const struct lys
  * @param[in] extp Parsed representation of the extension instance being processed.
  * @param[in,out] ext Compiled extension instance with the prepared ::lysc_ext_instance.substmts array, which will be updated
  * by storing the compiled data.
+ * @param[in] parent Optional parent of all the compiled schema nodes.
  * @return LY_SUCCESS on success.
  * @return LY_EVALID if compilation of the substatements fails.
  * @return LY_ENOT if the extension is disabled (by if-feature) and should be ignored.
  */
 LIBYANG_API_DECL LY_ERR lyplg_ext_compile_extension_instance(struct lysc_ctx *ctx, const struct lysp_ext_instance *extp,
-        struct lysc_ext_instance *ext);
+        struct lysc_ext_instance *ext, struct lysc_node *parent);
+
+/**
+ * @brief Compile augments for a specific node in an extension instance.
+ *
+ * May be useful for cases when there are virtual nodes that cannot be compiled using
+ * ::lyplg_ext_compile_extension_instance().
+ *
+ * @param[in] ctx Compile context.
+ * @param[in] ext Compiled extension instance.
+ * @param[in,out] node Compiled schema node to apply any matching augments for.
+ * @return LY_ERR value.
+ */
+LIBYANG_API_DECL LY_ERR lyplg_ext_compiled_node_augments(struct lysc_ctx *ctx, struct lysc_ext_instance *ext,
+        struct lysc_node *node);
 
 /** @} pluginsExtensionsCompile */
 
@@ -822,56 +836,68 @@ LIBYANG_API_DECL LY_ERR lyplg_ext_sprinter_ptree_add_nodes(const struct lyspr_tr
  */
 
 /**
- * @brief Callback called for all data nodes connected to the extension instance.
- *
- * Can be used for additional data node validation. Is called only after the whole data tree is created and standard
- * validation succeeds. Not called when parsing data and ::LYD_PARSE_ONLY is used.
+ * @brief Callback for getting the first child data node of an XPath document root of the extension instance.
  *
  * @param[in] ext Compiled extension instance.
- * @param[in] node Data node to process.
- * @param[in] validate_options Options used for the validation phase, see @ref datavalidationoptions.
- * @return LY_SUCCESS on success.
- * @return LY_ERR on error.
+ * @param[in] tree Whole data tree.
+ * @param[out] node First XPath document root child node, if any.
  */
-typedef LY_ERR (*lyplg_ext_data_node_clb)(struct lysc_ext_instance *ext, struct lyd_node *node, uint32_t validate_options);
+typedef void (*lyplg_ext_data_node_xpath_clb)(struct lysc_ext_instance *ext, const struct lyd_node *tree,
+        const struct lyd_node **node);
 
 /*
- * snode
+ * data snode
  */
 
 /**
- * @brief Callback for getting a schema node for a new YANG instance data described by an extension instance.
- * Needed only if the extension instance supports some nested standard YANG data.
+ * @brief Callback for getting a schema node for new YANG instance data described by an extension instance.
+ * Needed only if the extension instance defines some instantiable YANG data nodes.
+ *
+ * 1) Schema nodes are compiled and connected to each other:
+ *    - this callback is NOT called;
+ *    - storage of a data-def-stmt is retrieved from the ext-inst and the nodes connected to it.
+ *
+ * 2) YANG instance data is being parsed:
+ *    - this callback is called with @p in_xpath 0;
+ *    - schema node that can be instantiated in data should be returned (matching any set parameters).
+ *
+ * 3) XPath expression is being evaluated during schema compilation (must, when, leafref path, unique):
+ *    - this callback is called with @p in_xpath 1;
+ *    - first child of the document root of this extension should be returned.
  *
  * @param[in] ext Compiled extension instance.
- * @param[in] parent Parsed parent data node. Set if @p sparent is NULL.
- * @param[in] sparent Schema parent node. Set if @p parent is NULL.
+ * @param[in] parent Parsed parent data node.
+ * @param[in] sparent Schema parent node.
  * @param[in] prefix Element prefix, if any.
  * @param[in] prefix_len Length of @p prefix.
  * @param[in] format Format of @p prefix.
  * @param[in] prefix_data Format-specific prefix data.
- * @param[in] name Element name.
+ * @param[in] name Element name. If NULL, the first node should be returned (top-level, @p parent and @p sparent will
+ * not be set).
  * @param[in] name_len Length of @p name.
+ * @param[in] in_xpath Whether we are in an XPath accessible tree context or standard YANG data handling (parsing).
  * @param[out] snode Schema node to use for parsing the node.
  * @return LY_SUCCESS on success.
  * @return LY_ENOT if the data are not described by @p ext.
  * @return LY_ERR on error.
  */
 typedef LY_ERR (*lyplg_ext_data_snode_clb)(struct lysc_ext_instance *ext, const struct lyd_node *parent,
-        const struct lysc_node *sparent, const char *prefix, size_t prefix_len, LY_VALUE_FORMAT format, void *prefix_data,
-        const char *name, size_t name_len, const struct lysc_node **snode);
+        const struct lysc_node *sparent, const char *prefix, uint32_t prefix_len, LY_VALUE_FORMAT format, void *prefix_data,
+        const char *name, uint32_t name_len, ly_bool in_xpath, const struct lysc_node **snode);
 
 /*
- * validate
+ * data validate
  */
 
 /**
  * @brief Callback for validating parsed YANG instance data described by an extension instance.
  *
- * This callback is used only for nested data definition (with a standard YANG schema parent).
+ * This callback is called both for nested data definition (with a standard YANG schema parent) when all @p sibling
+ * nodes have ::LYD_EXT flag set and for data nodes of schema nodes with the extension instance (no special flag).
  *
  * @param[in] ext Compiled extension instance.
- * @param[in] sibling First sibling with schema node returned by ::lyplg_ext_data_snode_clb.
+ * @param[in] sibling First sibling with schema node returned by ::lyplg_ext_data_snode_clb for nested data, otherwise
+ * the data node with the extension instance.
  * @param[in] dep_tree Tree to be used for validating references from the operation subtree, if operation.
  * @param[in] data_type Validated data type, can be ::LYD_TYPE_DATA_YANG, ::LYD_TYPE_RPC_YANG, ::LYD_TYPE_NOTIF_YANG,
  * or ::LYD_TYPE_REPLY_YANG.
@@ -923,6 +949,89 @@ typedef void (*lyplg_ext_compile_free_clb)(const struct ly_ctx *ctx, struct lysc
  */
 LIBYANG_API_DECL void lyplg_ext_cfree_instance_substatements(const struct ly_ctx *ctx, struct lysc_ext_substmt *substmts);
 
+/*
+ * compiled size
+ */
+
+/**
+ * @brief Callback to return the size of the custom compiled structure and substmts array. If there are none, do not
+ * define this callback.
+ *
+ * @param[in] ext Compiled extension structure.
+ * @param[in,out] addr_ht Hash table with addresses of shared structures that were already accounted for, can be added to.
+ * @return Total size of the custom compiled structures and the substmts array;
+ * @return -1 on error.
+ */
+typedef int (*lyplg_ext_compiled_size_clb)(const struct lysc_ext_instance *ext, struct ly_ht *addr_ht);
+
+/**
+ * @brief Get the size of the compiled substatements storage.
+ *
+ * @param[in] substmts Compiled extension instance substatements array.
+ * @param[in,out] addr_ht Hash table with addresses of shared structures that were already accounted for, can be added to.
+ * @return Total size of the subtmts storage;
+ * @return -1 on error.
+ */
+LIBYANG_API_DECL int lyplg_ext_compiled_stmts_storage_size(const struct lysc_ext_substmt *substmts, struct ly_ht *addr_ht);
+
+/*
+ * compiled print
+ */
+
+/**
+ * @brief Callback to print (serialize) the custom compiled structure and substmts array storage.
+ *
+ * @param[in] orig_ext Compiled extension structure to print.
+ * @param[in,out] ext Printed extension structure to modify.
+ * @param[in,out] mem Memory chunk of the size returned by ::lyplg_ext_compiled_size_clb() to print into, is moved
+ * after all the printed data.
+ * @param[in,out] ptr_set Set with pointers to set to printed addresses.
+ * @param[in,out] addr_ht Hash table with pairs of addresses of shared structures to be printed and their printed
+ * addresses, can be added to.
+ * @return LY_SUCCESS on success.
+ * @return LY_ERR on error.
+ */
+typedef LY_ERR (*lyplg_ext_compiled_print_clb)(const struct lysc_ext_instance *orig_ext, struct lysc_ext_instance *ext,
+        struct ly_ht *addr_ht, struct ly_set *ptr_set, void **mem);
+
+/**
+ * @brief Print the substatements array storage and assign it to the serialized extension instance substatements array.
+ *
+ * @param[in] orig_substmts Extension instance substatements to print.
+ * @param[in,out] substmts Serialized extension structure substatements whose storage to modify.
+ * @param[in,out] addr_ht Hash table with pairs of addresses of shared structures to be printed and their printed
+ * addresses, can be added to.
+ * @param[in,out] ptr_set Set with pointers to set to printed addresses.
+ * @param[in,out] mem Memory chunk to print into, is moved after all the printed data.
+ * @return LY_SUCCESS on success;
+ * @return LY_ERR on error.
+ */
+LIBYANG_API_DECL LY_ERR lyplg_ext_compiled_stmts_storage_print(const struct lysc_ext_substmt *orig_substmts,
+        struct lysc_ext_substmt *substmts, struct ly_ht *addr_ht, struct ly_set *ptr_set, void **mem);
+
+/**
+ * @brief Get the printed address of a shared structure, in case it has already been printed.
+ *
+ * @param[in] addr_ht Hash table with all the printed addresses of shared structures.
+ * @param[in] addr Address of the original shared structure to be printed.
+ * @return Address of the printed shared structure;
+ * @return NULL if the address was not found.
+ */
+LIBYANG_API_DECL void *lyplg_ext_compiled_print_get_addr(const struct ly_ht *addr_ht, const void *addr);
+
+/**
+ * @brief Add a printed address of a shared structure to be reused by later prints.
+ *
+ * @param[in,out] addr_ht Hash table with pairs of addresses of shared structures to be printed and their printed
+ * addresses, is added to.
+ * @param[in] orig_addr Address of the original shared structure that was printed.
+ * @param[in] addr Address of the printed structure to be reused.
+ * @return LY_SUCCESS on success;
+ * @return LY_ERR on error.
+ */
+LIBYANG_API_DECL LY_ERR lyplg_ext_compiled_print_add_addr(struct ly_ht *addr_ht, const void *orig_addr,
+        const void *addr);
+
 /**
  * @brief Extension plugin implementing various aspects of a YANG extension.
  *
@@ -934,20 +1043,18 @@ struct lyplg_ext {
                                                  of the plugins for external tools) */
     lyplg_ext_parse_clb parse;              /**< callback to parse the extension instance substatements */
     lyplg_ext_compile_clb compile;          /**< callback to compile extension instance from the parsed data */
-    lyplg_ext_sprinter_info_clb printer_info;   /**< callback to print the compiled content (info format) of the extension
-                                                     instance */
-    lyplg_ext_sprinter_ctree_clb printer_ctree; /**< callback to print tree format of compiled node containing the
-                                                     compiled content of the extension instance */
-    lyplg_ext_sprinter_ptree_clb printer_ptree; /**< callback to print tree format of parsed node containing the
-                                                     parsed content of the extension instance */
-    lyplg_ext_data_node_clb node;           /**< callback to validate most relevant data instance for the extension
-                                                 instance */
-    lyplg_ext_data_snode_clb snode;         /**< callback to get schema node for nested YANG data */
-    lyplg_ext_data_validate_clb validate;   /**< callback to validate parsed data instances according to the extension
-                                                 definition */
+    lyplg_ext_sprinter_info_clb printer_info;   /**< callback to print the compiled content (info format) */
+    lyplg_ext_sprinter_ctree_clb printer_ctree; /**< callback to print tree format of a compiled node */
+    lyplg_ext_sprinter_ptree_clb printer_ptree; /**< callback to print tree format of a parsed node */
+    lyplg_ext_data_node_xpath_clb node_xpath;   /**< callback to get first XPath document root child node */
+    lyplg_ext_data_snode_clb snode;         /**< callback to get schema node in various use-cases */
+    lyplg_ext_data_validate_clb validate;   /**< callback to validate parsed data instances */
 
     lyplg_ext_parse_free_clb pfree;         /**< free the extension-specific data created by its parsing */
     lyplg_ext_compile_free_clb cfree;       /**< free the extension-specific data created by its compilation */
+    lyplg_ext_compiled_size_clb compiled_size;  /**< callback to get size of the custom compiled structure and substmts */
+    lyplg_ext_compiled_print_clb compiled_print;    /**< callback to print the compiled structure into a pre-allocated
+                                                         memory chunk */
 };
 
 struct lyplg_ext_record {
@@ -1013,13 +1120,24 @@ LIBYANG_API_DECL LY_ERR lyplg_ext_parsed_get_storage(const struct lysc_ext_insta
  *
  * @param[in] ctx Context with the callback.
  * @param[in] ext Compiled extension instance.
+ * @param[in] parent Data parent node instance of a schema node with @p ext instance.
  * @param[out] ext_data Provided extension instance data.
  * @param[out] ext_data_free Whether the extension instance should free @p ext_data or not.
  * @return LY_SUCCESS on success.
  * @return LY_ERR on error.
  */
-LIBYANG_API_DECL LY_ERR lyplg_ext_get_data(const struct ly_ctx *ctx, const struct lysc_ext_instance *ext, void **ext_data,
-        ly_bool *ext_data_free);
+LIBYANG_API_DECL LY_ERR lyplg_ext_get_data(const struct ly_ctx *ctx, const struct lysc_ext_instance *ext,
+        const struct lyd_node *parent, void **ext_data, ly_bool *ext_data_free);
+
+/**
+ * @brief Set parent context of a context. Errors and callbacks of the parent context will then always be used.
+ *
+ * @param[in] ctx Context to change.
+ * @param[in] parent_ctx Parent context to set.
+ * @return LY_SUCCESS on success.
+ * @return LY_ERR on error.
+ */
+LIBYANG_API_DECL LY_ERR lyplg_ext_set_parent_ctx(struct ly_ctx *ctx, const struct ly_ctx *parent_ctx);
 
 /**
  * @brief Insert extension instance data into a parent.
@@ -1035,20 +1153,66 @@ LIBYANG_API_DECL LY_ERR lyplg_ext_insert(struct lyd_node *parent, struct lyd_nod
  * @brief Expand parent-reference xpath expressions
  *
  * @param[in] ext Context allocated for extension.
+ * @param[in] parent Data parent node instance of a schema node with @p ext instance.
  * @param[out] refs Set of schema node matching parent-reference XPaths.
  * @return LY_ERR value.
  */
-LIBYANG_API_DECL LY_ERR lyplg_ext_schema_mount_get_parent_ref(const struct lysc_ext_instance *ext, struct ly_set **refs);
+LIBYANG_API_DECL LY_ERR lyplg_ext_schema_mount_get_parent_ref(const struct lysc_ext_instance *ext,
+        const struct lyd_node *parent, struct ly_set **refs);
 
 /**
  * @brief Allocate a new context for a particular instance of the yangmnt:mount-point extension.
  * Caller is responsible for **freeing** the created context.
  *
  * @param[in] ext Compiled extension instance.
+ * @param[in] parent Data parent node instance of a schema node with @p ext instance.
  * @param[out] ctx Context with modules loaded from the list found in the extension data.
  * @return LY_ERR value.
  */
-LIBYANG_API_DECL LY_ERR lyplg_ext_schema_mount_create_context(const struct lysc_ext_instance *ext, struct ly_ctx **ctx);
+LIBYANG_API_DECL LY_ERR lyplg_ext_schema_mount_create_context(const struct lysc_ext_instance *ext,
+        const struct lyd_node *parent, struct ly_ctx **ctx);
+
+/**
+ * @brief Create a shared schema mount context for a schema mount point.
+ *
+ * Does nothing if the mount point is not shared or if the context already exists.
+ *
+ * For printed contexts (read-only), all shared contexts must be created beforehand
+ * by calling this function as they cannot be created once the context is printed.
+ *
+ * @param[in] ext Compiled extension instance of a schema mount point.
+ * @param[in] ext_data ietf-yang-schema-mount and ietf-yang-library YANG data for the @p ext mount point.
+ * @return LY_ENOT if @p ext mount point data was not found in @p ext_data, other LY_ERR value otherwise.
+*/
+LIBYANG_API_DECL LY_ERR lyplg_ext_schema_mount_create_shared_context(struct lysc_ext_instance *ext,
+        const struct lyd_node *ext_data);
+
+/**
+ * @brief Destroy all the shared schema mount contexts for a given libyang context.
+ *
+ * For standard contexts this is done automatically when the context is destroyed.
+ *
+ * For printed contexts (read-only), all shared contexts must be destroyed
+ * by calling this function as otherwise they will be lost once the original context is destroyed.
+ *
+ * @param[in] ext Compiled extension instance of a schema mount point. All the shared schema mount
+ * contexts that belong to the same libyang context as @p ext will be destroyed.
+ */
+LIBYANG_API_DECL void lyplg_ext_schema_mount_destroy_shared_contexts(struct lysc_ext_instance *ext);
+
+/**
+ * @brief Destroy all the inline contexts for a given libyang context.
+ *
+ * For standard contexts this is done automatically when the context is destroyed.
+ * Useful when using a long-lived libyang context for cleaning up memory after completing
+ * operations with schema mount extension data that require inline contexts.
+ *
+ * Inline contexts are currently not supported in printed contexts.
+ *
+ * @param[in] ext Compiled extension instance of a schema mount point. All the inline schema mount
+ * contexts that belong to the same libyang context as @p ext will be destroyed.
+ */
+LIBYANG_API_DECL void lyplg_ext_schema_mount_destroy_inline_contexts(struct lysc_ext_instance *ext);
 
 /** @} pluginsExtensions */
 
