@@ -3,7 +3,7 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief libyang extension plugin - structure (RFC 8791)
  *
- * Copyright (c) 2022 CESNET, z.s.p.o.
+ * Copyright (c) 2022 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -19,7 +19,10 @@
 
 #include "compat.h"
 #include "libyang.h"
+#include "ly_common.h"
 #include "plugins_exts.h"
+#include "tree_data_internal.h"
+#include "xpath.h"
 
 struct lysp_ext_instance_structure {
     struct lysp_restr *musts;
@@ -32,11 +35,7 @@ struct lysp_ext_instance_structure {
 };
 
 struct lysc_ext_instance_structure {
-    struct lysc_must *musts;
-    uint16_t flags;
-    const char *dsc;
-    const char *ref;
-    struct lysc_node *child;
+    struct lysc_node_container *top_cont;
 };
 
 struct lysp_ext_instance_augment_structure {
@@ -47,6 +46,8 @@ struct lysp_ext_instance_augment_structure {
     struct lysp_node_augment *aug;
 };
 
+static void structure_cfree(const struct ly_ctx *ctx, struct lysc_ext_instance *ext);
+
 /**
  * @brief Parse structure extension instances.
  *
@@ -163,7 +164,7 @@ structure_compile(struct lysc_ctx *cctx, const struct lysp_ext_instance *extp, s
 {
     LY_ERR rc;
     struct lysc_module *mod_c;
-    const struct lysc_node *child;
+    struct lysc_node *child;
     struct lysc_ext_instance_structure *struct_cdata;
     uint32_t prev_options = *lyplg_ext_compile_get_options(cctx);
 
@@ -186,75 +187,95 @@ structure_compile(struct lysc_ctx *cctx, const struct lysp_ext_instance *extp, s
     }
     ext->compiled = struct_cdata;
 
+    /* add the top-level container with the extension instance name, connect all the other substatements into it */
+    struct_cdata->top_cont = calloc(1, sizeof *struct_cdata->top_cont);
+    if (!struct_cdata->top_cont) {
+        goto emem;
+    }
+
+    struct_cdata->top_cont->name = ext->argument;
+    struct_cdata->top_cont->nodetype = LYS_CONTAINER;
+    struct_cdata->top_cont->module = (struct lys_module *)lyplg_ext_compile_get_cur_mod(cctx);
+    struct_cdata->top_cont->prev = &struct_cdata->top_cont->node;
+
     /* compile substatements */
     LY_ARRAY_CREATE_GOTO(cctx->ctx, ext->substmts, 14, rc, emem);
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[0].stmt = LY_STMT_MUST;
-    ext->substmts[0].storage_p = (void **)&struct_cdata->musts;
+    ext->substmts[0].storage_p = (void **)&struct_cdata->top_cont->musts;
 
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[1].stmt = LY_STMT_STATUS;
-    ext->substmts[1].storage_p = (void **)&struct_cdata->flags;
+    ext->substmts[1].storage_p = (void **)&struct_cdata->top_cont->flags;
 
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[2].stmt = LY_STMT_DESCRIPTION;
-    ext->substmts[2].storage_p = (void **)&struct_cdata->dsc;
+    ext->substmts[2].storage_p = (void **)&struct_cdata->top_cont->dsc;
 
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[3].stmt = LY_STMT_REFERENCE;
-    ext->substmts[3].storage_p = (void **)&struct_cdata->ref;
+    ext->substmts[3].storage_p = (void **)&struct_cdata->top_cont->ref;
 
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[4].stmt = LY_STMT_TYPEDEF;
-    ext->substmts[4].storage_p = 0;
+    ext->substmts[4].storage_p = NULL;
 
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[5].stmt = LY_STMT_GROUPING;
-    ext->substmts[5].storage_p = 0;
+    ext->substmts[5].storage_p = NULL;
 
     /* data-def-stmt */
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[6].stmt = LY_STMT_CONTAINER;
-    ext->substmts[6].storage_p = (void **)&struct_cdata->child;
+    ext->substmts[6].storage_p = (void **)&struct_cdata->top_cont->child;
 
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[7].stmt = LY_STMT_LEAF;
-    ext->substmts[7].storage_p = (void **)&struct_cdata->child;
+    ext->substmts[7].storage_p = (void **)&struct_cdata->top_cont->child;
 
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[8].stmt = LY_STMT_LEAF_LIST;
-    ext->substmts[8].storage_p = (void **)&struct_cdata->child;
+    ext->substmts[8].storage_p = (void **)&struct_cdata->top_cont->child;
 
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[9].stmt = LY_STMT_LIST;
-    ext->substmts[9].storage_p = (void **)&struct_cdata->child;
+    ext->substmts[9].storage_p = (void **)&struct_cdata->top_cont->child;
 
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[10].stmt = LY_STMT_CHOICE;
-    ext->substmts[10].storage_p = (void **)&struct_cdata->child;
+    ext->substmts[10].storage_p = (void **)&struct_cdata->top_cont->child;
 
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[11].stmt = LY_STMT_ANYDATA;
-    ext->substmts[11].storage_p = (void **)&struct_cdata->child;
+    ext->substmts[11].storage_p = (void **)&struct_cdata->top_cont->child;
 
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[12].stmt = LY_STMT_ANYXML;
-    ext->substmts[12].storage_p = (void **)&struct_cdata->child;
+    ext->substmts[12].storage_p = (void **)&struct_cdata->top_cont->child;
 
     LY_ARRAY_INCREMENT(ext->substmts);
     ext->substmts[13].stmt = LY_STMT_USES;
-    ext->substmts[13].storage_p = (void **)&struct_cdata->child;
+    ext->substmts[13].storage_p = (void **)&struct_cdata->top_cont->child;
 
     *lyplg_ext_compile_get_options(cctx) |= LYS_COMPILE_NO_CONFIG | LYS_COMPILE_NO_DISABLED;
-    rc = lyplg_ext_compile_extension_instance(cctx, extp, ext);
+    rc = lyplg_ext_compile_extension_instance(cctx, extp, ext, (struct lysc_node *)struct_cdata->top_cont);
     *lyplg_ext_compile_get_options(cctx) = prev_options;
     if (rc) {
         return rc;
     }
 
+    /* compile config properly even though it is ignored */
+    if (!(struct_cdata->top_cont->flags & LYS_CONFIG_MASK)) {
+        struct_cdata->top_cont->flags |= LYS_CONFIG_W;
+    }
+
+    /* connect any augments */
+    LY_CHECK_RET(lyplg_ext_compiled_node_augments(cctx, ext, (struct lysc_node *)struct_cdata->top_cont));
+
     return LY_SUCCESS;
 
 emem:
+    structure_cfree(lyplg_ext_compile_get_ctx(cctx), ext);
     lyplg_ext_compile_log(cctx, ext, LY_LLERR, LY_EMEM, "Memory allocation failed (%s()).", __func__);
     return LY_EMEM;
 }
@@ -291,8 +312,80 @@ structure_pfree(const struct ly_ctx *ctx, struct lysp_ext_instance *ext)
 static void
 structure_cfree(const struct ly_ctx *ctx, struct lysc_ext_instance *ext)
 {
+    struct lysc_ext_instance_structure *struct_cdata = ext->compiled;
+
     lyplg_ext_cfree_instance_substatements(ctx, ext->substmts);
-    free(ext->compiled);
+    if (struct_cdata) {
+        free(struct_cdata->top_cont);
+        free(struct_cdata);
+    }
+}
+
+static int
+structure_compiled_size(const struct lysc_ext_instance *ext, struct ly_ht *addr_ht)
+{
+    struct lysc_ext_instance_structure *struct_cdata;
+    int size = 0;
+
+    struct_cdata = ext->compiled;
+
+    size += sizeof *struct_cdata;
+    size += lyplg_ext_compiled_stmts_storage_size(ext->substmts, addr_ht);
+    size += sizeof *struct_cdata->top_cont;
+
+    return size;
+}
+
+static LY_ERR
+structure_compiled_print(const struct lysc_ext_instance *orig_ext, struct lysc_ext_instance *ext, struct ly_ht *addr_ht,
+        struct ly_set *ptr_set, void **mem)
+{
+    LY_ERR r;
+    struct lysc_ext_instance_structure *struct_cdata, *orig_cdata;
+
+    orig_cdata = orig_ext->compiled;
+
+    /* ext structure */
+    struct_cdata = ext->compiled = *mem;
+    *mem = (char *)*mem + sizeof *struct_cdata;
+    memset(struct_cdata, 0, sizeof *struct_cdata);
+
+    /* top_cont */
+    struct_cdata->top_cont = *mem;
+    *mem = (char *)*mem + sizeof *struct_cdata->top_cont;
+    lyplg_ext_compiled_print_add_addr(addr_ht, orig_cdata->top_cont, struct_cdata->top_cont);
+    memset(struct_cdata->top_cont, 0, sizeof *struct_cdata->top_cont);
+
+    /* substatements */
+    ext->substmts[0].storage_p = (void **)&struct_cdata->top_cont->musts;
+    ext->substmts[1].storage_p = (void **)&struct_cdata->top_cont->flags;
+    ext->substmts[2].storage_p = (void **)&struct_cdata->top_cont->dsc;
+    ext->substmts[3].storage_p = (void **)&struct_cdata->top_cont->ref;
+
+    ext->substmts[6].storage_p = (void **)&struct_cdata->top_cont->child;
+    ext->substmts[7].storage_p = (void **)&struct_cdata->top_cont->child;
+    ext->substmts[8].storage_p = (void **)&struct_cdata->top_cont->child;
+    ext->substmts[9].storage_p = (void **)&struct_cdata->top_cont->child;
+    ext->substmts[10].storage_p = (void **)&struct_cdata->top_cont->child;
+    ext->substmts[11].storage_p = (void **)&struct_cdata->top_cont->child;
+    ext->substmts[12].storage_p = (void **)&struct_cdata->top_cont->child;
+    ext->substmts[13].storage_p = (void **)&struct_cdata->top_cont->child;
+
+    r = lyplg_ext_compiled_stmts_storage_print(orig_ext->substmts, ext->substmts, addr_ht, ptr_set, mem);
+    if (r) {
+        return r;
+    }
+
+    /* top_cont substatements that are now in addr_ht */
+    struct_cdata->top_cont->name = lyplg_ext_compiled_print_get_addr(addr_ht, orig_cdata->top_cont->name);
+    assert(struct_cdata->top_cont->name);
+    struct_cdata->top_cont->nodetype = orig_cdata->top_cont->nodetype;
+    struct_cdata->top_cont->module = lyplg_ext_compiled_print_get_addr(addr_ht, orig_cdata->top_cont->module);
+    assert(struct_cdata->top_cont->module);
+    struct_cdata->top_cont->prev = lyplg_ext_compiled_print_get_addr(addr_ht, orig_cdata->top_cont->prev);
+    assert(struct_cdata->top_cont->prev);
+
+    return LY_SUCCESS;
 }
 
 /**
@@ -318,7 +411,8 @@ structure_aug_parse(struct lysp_ctx *pctx, struct lysp_ext_instance *ext)
 
     /* augment-structure must define some data-def-stmt */
     LY_LIST_FOR(ext->child, stmt) {
-        if (stmt->kw & LY_STMT_DATA_NODE_MASK) {
+        if (stmt->kw & (LY_STMT_CONTAINER | LY_STMT_LEAF | LY_STMT_LEAF_LIST | LY_STMT_LIST | LY_STMT_CHOICE |
+                LY_STMT_ANYDATA | LY_STMT_ANYXML | LY_STMT_USES)) {
             break;
         }
     }
@@ -402,7 +496,7 @@ structure_aug_parse(struct lysp_ctx *pctx, struct lysp_ext_instance *ext)
     }
     aug_pdata->aug->nodetype = LYS_AUGMENT;
     aug_pdata->aug->flags = aug_pdata->flags;
-    if (lydict_insert(ctx, ext->argument, 0, &aug_pdata->aug->nodeid)) {
+    if (lysdict_insert(ctx, ext->argument, 0, &aug_pdata->aug->nodeid)) {
         goto emem;
     }
     aug_pdata->aug->child = aug_pdata->child;
@@ -462,6 +556,54 @@ structure_sprinter_ptree(struct lysp_ext_instance *ext, const struct lyspr_tree_
     return rc;
 }
 
+/**
+ * @brief Node xpath callback for structure.
+ */
+static void
+structure_node_xpath(struct lysc_ext_instance *ext, const struct lyd_node *tree, const struct lyd_node **node)
+{
+    *node = NULL;
+
+    if (!tree) {
+        return;
+    }
+
+    /* virtual top-level container expected */
+    assert(!strcmp(ext->argument, LYD_NAME(tree)));
+    (void)ext;
+
+    /* return the child */
+    *node = lyd_child(tree);
+    return;
+}
+
+/**
+ * @brief Snode callback for structure.
+ */
+static LY_ERR
+structure_snode(struct lysc_ext_instance *ext, const struct lyd_node *parent, const struct lysc_node *sparent,
+        const char *prefix, uint32_t UNUSED(prefix_len), LY_VALUE_FORMAT UNUSED(format), void *UNUSED(prefix_data),
+        const char *name, uint32_t UNUSED(name_len), ly_bool in_xpath, const struct lysc_node **snode)
+{
+    struct lysc_ext_instance_structure *struct_cdata = ext->compiled;
+
+    assert(!parent && !sparent && !prefix && !name);
+    (void)parent;
+    (void)sparent;
+    (void)prefix;
+    (void)name;
+
+    if (in_xpath) {
+        /* XPath starts at the substatements */
+        *snode = struct_cdata->top_cont->child;
+    } else {
+        /* data tree start at the top-level virtual container */
+        *snode = &struct_cdata->top_cont->node;
+    }
+
+    return *snode ? LY_SUCCESS : LY_ENOT;
+}
+
 /**
  * @brief Augment structure schema parsed tree printer.
  *
@@ -525,34 +667,38 @@ const struct lyplg_ext_record plugins_structure[] = {
         .revision = "2020-06-17",
         .name = "structure",
 
-        .plugin.id = "ly2 structure v1",
+        .plugin.id = "ly2 structure",
         .plugin.parse = structure_parse,
         .plugin.compile = structure_compile,
         .plugin.printer_info = structure_printer_info,
         .plugin.printer_ctree = structure_sprinter_ctree,
         .plugin.printer_ptree = structure_sprinter_ptree,
-        .plugin.node = NULL,
-        .plugin.snode = NULL,
+        .plugin.node_xpath = structure_node_xpath,
+        .plugin.snode = structure_snode,
         .plugin.validate = NULL,
         .plugin.pfree = structure_pfree,
-        .plugin.cfree = structure_cfree
+        .plugin.cfree = structure_cfree,
+        .plugin.compiled_size = structure_compiled_size,
+        .plugin.compiled_print = structure_compiled_print
     },
     {
         .module = "ietf-yang-structure-ext",
         .revision = "2020-06-17",
         .name = "augment-structure",
 
-        .plugin.id = "ly2 structure v1",
+        .plugin.id = "ly2 structure",
         .plugin.parse = structure_aug_parse,
         .plugin.compile = NULL,
         .plugin.printer_info = NULL,
         .plugin.printer_ctree = structure_aug_sprinter_ctree,
         .plugin.printer_ptree = structure_aug_sprinter_ptree,
-        .plugin.node = NULL,
+        .plugin.node_xpath = NULL,
         .plugin.snode = NULL,
         .plugin.validate = NULL,
         .plugin.pfree = structure_pfree,
-        .plugin.cfree = NULL
+        .plugin.cfree = NULL,
+        .plugin.compiled_size = NULL,
+        .plugin.compiled_print = NULL
     },
     {0}     /* terminating zeroed record */
 };
