@@ -1,9 +1,10 @@
 /**
  * @file integer.c
  * @author Radek Krejci <rkrejci@cesnet.cz>
+ * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief Built-in integer types plugin.
  *
- * Copyright (c) 2019-2021 CESNET, z.s.p.o.
+ * Copyright (c) 2019 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -16,6 +17,7 @@
 
 #include "plugins_types.h"
 
+#include <assert.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -26,34 +28,30 @@
 #include "compat.h"
 #include "ly_common.h"
 #include "plugins_internal.h" /* LY_TYPE_*_STR */
+#include "tree_schema_internal.h"
 
 /**
  * @page howtoDataLYB LYB Binary Format
  * @subsection howtoDataLYBTypesInteger (u)int(8/16/32/64) (built-in)
  *
- * | Size (B) | Mandatory | Type | Meaning |
+ * | Size (b) | Mandatory | Type | Meaning |
  * | :------  | :-------: | :--: | :-----: |
- * | 1/2/4/8 | yes | pointer to the specific integer type | little-endian integer value |
+ * | variable | yes | pointer to the specific integer type | little-endian integer value |
  */
 
-static LY_ERR lyplg_type_validate_int(const struct ly_ctx *UNUSED(ctx), const struct lysc_type *type, const struct lyd_node *UNUSED(ctx_node), const struct lyd_node *UNUSED(tree), struct lyd_value *storage, struct ly_err_item **err);
-static LY_ERR lyplg_type_validate_uint(const struct ly_ctx *UNUSED(ctx), const struct lysc_type *type, const struct lyd_node *UNUSED(ctx_node), const struct lyd_node *UNUSED(tree), struct lyd_value *storage, struct ly_err_item **err);
+static LY_ERR lyplg_type_validate_value_int(const struct ly_ctx *ctx, const struct lysc_type *type,
+        struct lyd_value *storage, struct ly_err_item **err);
+static LY_ERR lyplg_type_validate_value_uint(const struct ly_ctx *ctx, const struct lysc_type *type,
+        struct lyd_value *storage, struct ly_err_item **err);
 
-/**
- * @brief LYB value size of each integer type.
- */
-static size_t integer_lyb_size[] = {
-    [LY_TYPE_INT8] = 1, [LY_TYPE_INT16] = 2, [LY_TYPE_INT32] = 4, [LY_TYPE_INT64] = 8,
-    [LY_TYPE_UINT8] = 1, [LY_TYPE_UINT16] = 2, [LY_TYPE_UINT32] = 4, [LY_TYPE_UINT64] = 8
-};
-
-LIBYANG_API_DEF LY_ERR
-lyplg_type_store_int(const struct ly_ctx *ctx, const struct lysc_type *type, const void *value, size_t value_len,
+static LY_ERR
+lyplg_type_store_int(const struct ly_ctx *ctx, const struct lysc_type *type, const void *value, uint32_t value_size_bits,
         uint32_t options, LY_VALUE_FORMAT format, void *UNUSED(prefix_data), uint32_t hints,
-        const struct lysc_node *UNUSED(ctx_node), struct lyd_value *storage, struct lys_glob_unres *UNUSED(unres),
-        struct ly_err_item **err)
+        const struct lysc_node *UNUSED(ctx_node), const struct lysc_ext_instance *UNUSED(top_ext),
+        struct lyd_value *storage, struct lys_glob_unres *UNUSED(unres), struct ly_err_item **err)
 {
     LY_ERR ret = LY_SUCCESS;
+    uint32_t value_size;
     int64_t num = 0;
     int base = 1;
     char *canon = NULL;
@@ -62,36 +60,34 @@ lyplg_type_store_int(const struct ly_ctx *ctx, const struct lysc_type *type, con
     memset(storage, 0, sizeof *storage);
     storage->realtype = type;
 
-    if (format == LY_VALUE_LYB) {
-        /* validation */
-        if (value_len != integer_lyb_size[type->basetype]) {
-            ret = ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL, "Invalid LYB signed integer value size %zu (expected %zu).",
-                    value_len, integer_lyb_size[type->basetype]);
-            goto cleanup;
-        }
+    /* check value length */
+    ret = lyplg_type_check_value_size(lys_datatype2str(type->basetype), format, value_size_bits,
+            LYPLG_LYB_SIZE_VARIABLE_BITS, 0, &value_size, err);
+    LY_CHECK_GOTO(ret, cleanup);
 
+    if (format == LY_VALUE_LYB) {
         /* copy the integer and correct the byte order */
-        memcpy(&num, value, value_len);
+        memcpy(&num, value, value_size);
         num = le64toh(num);
     } else {
         /* check hints */
-        ret = lyplg_type_check_hints(hints, value, value_len, type->basetype, &base, err);
+        ret = lyplg_type_check_hints(hints, value, value_size, type->basetype, &base, err);
         LY_CHECK_GOTO(ret, cleanup);
 
         /* parse the integer */
         switch (type->basetype) {
         case LY_TYPE_INT8:
-            ret = lyplg_type_parse_int("int8", base, INT64_C(-128), INT64_C(127), value, value_len, &num, err);
+            ret = lyplg_type_parse_int("int8", base, INT64_C(-128), INT64_C(127), value, value_size, &num, err);
             break;
         case LY_TYPE_INT16:
-            ret = lyplg_type_parse_int("int16", base, INT64_C(-32768), INT64_C(32767), value, value_len, &num, err);
+            ret = lyplg_type_parse_int("int16", base, INT64_C(-32768), INT64_C(32767), value, value_size, &num, err);
             break;
         case LY_TYPE_INT32:
-            ret = lyplg_type_parse_int("int32", base, INT64_C(-2147483648), INT64_C(2147483647), value, value_len, &num, err);
+            ret = lyplg_type_parse_int("int32", base, INT64_C(-2147483648), INT64_C(2147483647), value, value_size, &num, err);
             break;
         case LY_TYPE_INT64:
             ret = lyplg_type_parse_int("int64", base, INT64_C(-9223372036854775807) - INT64_C(1),
-                    INT64_C(9223372036854775807), value, value_len, &num, err);
+                    INT64_C(9223372036854775807), value, value_size, &num, err);
             break;
         default:
             LOGINT(ctx);
@@ -129,7 +125,7 @@ lyplg_type_store_int(const struct ly_ctx *ctx, const struct lysc_type *type, con
             options &= ~LYPLG_TYPE_STORE_DYNAMIC;
             LY_CHECK_GOTO(ret, cleanup);
         } else {
-            ret = lydict_insert(ctx, value, value_len, &storage->_canonical);
+            ret = lydict_insert(ctx, value, value_size, &storage->_canonical);
             LY_CHECK_GOTO(ret, cleanup);
         }
     } else {
@@ -158,7 +154,7 @@ lyplg_type_store_int(const struct ly_ctx *ctx, const struct lysc_type *type, con
 
     if (!(options & LYPLG_TYPE_STORE_ONLY)) {
         /* validate value */
-        ret = lyplg_type_validate_int(ctx, type, NULL, NULL, storage, err);
+        ret = lyplg_type_validate_value_int(ctx, type, storage, err);
         LY_CHECK_GOTO(ret, cleanup);
     }
 
@@ -174,11 +170,11 @@ cleanup:
 }
 
 /**
- * @brief Implementation of ::lyplg_type_validate_clb for the signed interger types.
+ * @brief Implementation of ::lyplg_type_validate_value_clb for the signed interger types.
  */
 static LY_ERR
-lyplg_type_validate_int(const struct ly_ctx *UNUSED(ctx), const struct lysc_type *type, const struct lyd_node *UNUSED(ctx_node),
-        const struct lyd_node *UNUSED(tree), struct lyd_value *storage, struct ly_err_item **err)
+lyplg_type_validate_value_int(const struct ly_ctx *UNUSED(ctx), const struct lysc_type *type, struct lyd_value *storage,
+        struct ly_err_item **err)
 {
     LY_ERR ret;
     struct lysc_type_num *type_num = (struct lysc_type_num *)type;
@@ -215,7 +211,7 @@ lyplg_type_validate_int(const struct ly_ctx *UNUSED(ctx), const struct lysc_type
     return LY_SUCCESS;
 }
 
-LIBYANG_API_DEF LY_ERR
+static LY_ERR
 lyplg_type_compare_int(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *val1, const struct lyd_value *val2)
 {
     if (val1->realtype != val2->realtype) {
@@ -249,7 +245,7 @@ lyplg_type_compare_int(const struct ly_ctx *UNUSED(ctx), const struct lyd_value
     return LY_SUCCESS;
 }
 
-LIBYANG_API_DEF int
+static int
 lyplg_type_sort_int(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *val1, const struct lyd_value *val2)
 {
     switch (val1->realtype->basetype) {
@@ -295,9 +291,9 @@ lyplg_type_sort_int(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *va
     return 0;
 }
 
-LIBYANG_API_DEF const void *
+static const void *
 lyplg_type_print_int(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *value, LY_VALUE_FORMAT format,
-        void *UNUSED(prefix_data), ly_bool *dynamic, size_t *value_len)
+        void *UNUSED(prefix_data), ly_bool *dynamic, uint32_t *value_size_bits)
 {
     int64_t prev_num = 0, num = 0;
     void *buf;
@@ -319,24 +315,26 @@ lyplg_type_print_int(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *v
         default:
             break;
         }
+
         num = htole64(num);
         if (num == prev_num) {
             /* values are equal, little-endian or int8 */
             *dynamic = 0;
-            if (value_len) {
-                *value_len = integer_lyb_size[value->realtype->basetype];
+            if (value_size_bits) {
+                /* the least amount of bits that can hold the number */
+                *value_size_bits = lyplg_type_get_highest_set_bit_pos(num);
             }
             return &value->int64;
         } else {
             /* values differ, big-endian */
-            buf = calloc(1, integer_lyb_size[value->realtype->basetype]);
+            buf = calloc(1, LYPLG_BITS2BYTES(lyplg_type_get_highest_set_bit_pos(num)));
             LY_CHECK_RET(!buf, NULL);
 
             *dynamic = 1;
-            if (value_len) {
-                *value_len = integer_lyb_size[value->realtype->basetype];
+            if (value_size_bits) {
+                *value_size_bits = lyplg_type_get_highest_set_bit_pos(num);
             }
-            memcpy(buf, &num, integer_lyb_size[value->realtype->basetype]);
+            memcpy(buf, &num, LYPLG_BITS2BYTES(lyplg_type_get_highest_set_bit_pos(num)));
             return buf;
         }
     }
@@ -345,19 +343,20 @@ lyplg_type_print_int(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *v
     if (dynamic) {
         *dynamic = 0;
     }
-    if (value_len) {
-        *value_len = strlen(value->_canonical);
+    if (value_size_bits) {
+        *value_size_bits = strlen(value->_canonical) * 8;
     }
     return value->_canonical;
 }
 
-LIBYANG_API_DEF LY_ERR
-lyplg_type_store_uint(const struct ly_ctx *ctx, const struct lysc_type *type, const void *value, size_t value_len,
+static LY_ERR
+lyplg_type_store_uint(const struct ly_ctx *ctx, const struct lysc_type *type, const void *value, uint32_t value_size_bits,
         uint32_t options, LY_VALUE_FORMAT format, void *UNUSED(prefix_data), uint32_t hints,
-        const struct lysc_node *UNUSED(ctx_node), struct lyd_value *storage, struct lys_glob_unres *UNUSED(unres),
-        struct ly_err_item **err)
+        const struct lysc_node *UNUSED(ctx_node), const struct lysc_ext_instance *UNUSED(top_ext),
+        struct lyd_value *storage, struct lys_glob_unres *UNUSED(unres), struct ly_err_item **err)
 {
     LY_ERR ret = LY_SUCCESS;
+    uint32_t value_size;
     uint64_t num = 0;
     int base = 0;
     char *canon;
@@ -366,35 +365,33 @@ lyplg_type_store_uint(const struct ly_ctx *ctx, const struct lysc_type *type, co
     memset(storage, 0, sizeof *storage);
     storage->realtype = type;
 
-    if (format == LY_VALUE_LYB) {
-        /* validation */
-        if (value_len != integer_lyb_size[type->basetype]) {
-            ret = ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL, "Invalid LYB unsigned integer value size %zu (expected %zu).",
-                    value_len, integer_lyb_size[type->basetype]);
-            goto cleanup;
-        }
+    /* check value length */
+    ret = lyplg_type_check_value_size(lys_datatype2str(type->basetype), format, value_size_bits,
+            LYPLG_LYB_SIZE_VARIABLE_BITS, 0, &value_size, err);
+    LY_CHECK_GOTO(ret, cleanup);
 
+    if (format == LY_VALUE_LYB) {
         /* copy the integer and correct the byte order */
-        memcpy(&num, value, value_len);
+        memcpy(&num, value, value_size);
         num = le64toh(num);
     } else {
         /* check hints */
-        ret = lyplg_type_check_hints(hints, value, value_len, type->basetype, &base, err);
+        ret = lyplg_type_check_hints(hints, value, value_size, type->basetype, &base, err);
         LY_CHECK_GOTO(ret, cleanup);
 
         /* parse the integer */
         switch (type->basetype) {
         case LY_TYPE_UINT8:
-            ret = lyplg_type_parse_uint("uint8", base, UINT64_C(255), value, value_len, &num, err);
+            ret = lyplg_type_parse_uint("uint8", base, UINT64_C(255), value, value_size, &num, err);
             break;
         case LY_TYPE_UINT16:
-            ret = lyplg_type_parse_uint("uint16", base, UINT64_C(65535), value, value_len, &num, err);
+            ret = lyplg_type_parse_uint("uint16", base, UINT64_C(65535), value, value_size, &num, err);
             break;
         case LY_TYPE_UINT32:
-            ret = lyplg_type_parse_uint("uint32", base, UINT64_C(4294967295), value, value_len, &num, err);
+            ret = lyplg_type_parse_uint("uint32", base, UINT64_C(4294967295), value, value_size, &num, err);
             break;
         case LY_TYPE_UINT64:
-            ret = lyplg_type_parse_uint("uint64", base, UINT64_C(18446744073709551615), value, value_len, &num, err);
+            ret = lyplg_type_parse_uint("uint64", base, UINT64_C(18446744073709551615), value, value_size, &num, err);
             break;
         default:
             LOGINT(ctx);
@@ -428,7 +425,7 @@ lyplg_type_store_uint(const struct ly_ctx *ctx, const struct lysc_type *type, co
             options &= ~LYPLG_TYPE_STORE_DYNAMIC;
             LY_CHECK_GOTO(ret, cleanup);
         } else {
-            ret = lydict_insert(ctx, value, value_len, &storage->_canonical);
+            ret = lydict_insert(ctx, value, value_size, &storage->_canonical);
             LY_CHECK_GOTO(ret, cleanup);
         }
     } else {
@@ -442,7 +439,7 @@ lyplg_type_store_uint(const struct ly_ctx *ctx, const struct lysc_type *type, co
 
     if (!(options & LYPLG_TYPE_STORE_ONLY)) {
         /* validate value */
-        ret = lyplg_type_validate_uint(ctx, type, NULL, NULL, storage, err);
+        ret = lyplg_type_validate_value_uint(ctx, type, storage, err);
         LY_CHECK_GOTO(ret, cleanup);
     }
 
@@ -458,11 +455,11 @@ cleanup:
 }
 
 /**
- * @brief Implementation of ::lyplg_type_validate_clb for the unsigned interger types.
+ * @brief Implementation of ::lyplg_type_validate_value_clb for the unsigned interger types.
  */
 static LY_ERR
-lyplg_type_validate_uint(const struct ly_ctx *UNUSED(ctx), const struct lysc_type *type, const struct lyd_node *UNUSED(ctx_node),
-        const struct lyd_node *UNUSED(tree), struct lyd_value *storage, struct ly_err_item **err)
+lyplg_type_validate_value_uint(const struct ly_ctx *UNUSED(ctx), const struct lysc_type *type, struct lyd_value *storage,
+        struct ly_err_item **err)
 {
     LY_ERR ret;
     struct lysc_type_num *type_num = (struct lysc_type_num *)type;
@@ -499,7 +496,7 @@ lyplg_type_validate_uint(const struct ly_ctx *UNUSED(ctx), const struct lysc_typ
     return LY_SUCCESS;
 }
 
-LIBYANG_API_DEF LY_ERR
+static LY_ERR
 lyplg_type_compare_uint(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *val1, const struct lyd_value *val2)
 {
     switch (val1->realtype->basetype) {
@@ -529,7 +526,7 @@ lyplg_type_compare_uint(const struct ly_ctx *UNUSED(ctx), const struct lyd_value
     return LY_SUCCESS;
 }
 
-LIBYANG_API_DEF int
+static int
 lyplg_type_sort_uint(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *val1, const struct lyd_value *val2)
 {
     switch (val1->realtype->basetype) {
@@ -575,9 +572,9 @@ lyplg_type_sort_uint(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *v
     return 0;
 }
 
-LIBYANG_API_DEF const void *
+static const void *
 lyplg_type_print_uint(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *value, LY_VALUE_FORMAT format,
-        void *UNUSED(prefix_data), ly_bool *dynamic, size_t *value_len)
+        void *UNUSED(prefix_data), ly_bool *dynamic, uint32_t *value_size_bits)
 {
     uint64_t num = 0;
     void *buf;
@@ -603,20 +600,21 @@ lyplg_type_print_uint(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *
         if (num == value->uint64) {
             /* values are equal, little-endian or uint8 */
             *dynamic = 0;
-            if (value_len) {
-                *value_len = integer_lyb_size[value->realtype->basetype];
+            if (value_size_bits) {
+                /* the least amount of bits that can hold the number */
+                *value_size_bits = lyplg_type_get_highest_set_bit_pos(num);
             }
             return &value->uint64;
         } else {
             /* values differ, big-endian */
-            buf = calloc(1, integer_lyb_size[value->realtype->basetype]);
+            buf = calloc(1, LYPLG_BITS2BYTES(lyplg_type_get_highest_set_bit_pos(num)));
             LY_CHECK_RET(!buf, NULL);
 
             *dynamic = 1;
-            if (value_len) {
-                *value_len = integer_lyb_size[value->realtype->basetype];
+            if (value_size_bits) {
+                *value_size_bits = lyplg_type_get_highest_set_bit_pos(num);
             }
-            memcpy(buf, &num, integer_lyb_size[value->realtype->basetype]);
+            memcpy(buf, &num, LYPLG_BITS2BYTES(lyplg_type_get_highest_set_bit_pos(num)));
             return buf;
         }
     }
@@ -625,8 +623,8 @@ lyplg_type_print_uint(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *
     if (dynamic) {
         *dynamic = 0;
     }
-    if (value_len) {
-        *value_len = strlen(value->_canonical);
+    if (value_size_bits) {
+        *value_size_bits = strlen(value->_canonical) * 8;
     }
     return value->_canonical;
 }
@@ -644,113 +642,121 @@ const struct lyplg_type_record plugins_integer[] = {
         .revision = NULL,
         .name = LY_TYPE_UINT8_STR,
 
-        .plugin.id = "libyang 2 - integers, version 1",
+        .plugin.id = "ly2 integers",
+        .plugin.lyb_size = lyplg_type_lyb_size_variable_bits,
         .plugin.store = lyplg_type_store_uint,
-        .plugin.validate = lyplg_type_validate_uint,
+        .plugin.validate_value = lyplg_type_validate_value_uint,
+        .plugin.validate_tree = NULL,
         .plugin.compare = lyplg_type_compare_uint,
         .plugin.sort = lyplg_type_sort_uint,
         .plugin.print = lyplg_type_print_uint,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
-        .plugin.lyb_data_len = 1,
     }, {
         .module = "",
         .revision = NULL,
         .name = LY_TYPE_UINT16_STR,
 
-        .plugin.id = "libyang 2 - integers, version 1",
+        .plugin.id = "ly2 integers",
+        .plugin.lyb_size = lyplg_type_lyb_size_variable_bits,
         .plugin.store = lyplg_type_store_uint,
-        .plugin.validate = lyplg_type_validate_uint,
+        .plugin.validate_value = lyplg_type_validate_value_uint,
+        .plugin.validate_tree = NULL,
         .plugin.compare = lyplg_type_compare_uint,
         .plugin.sort = lyplg_type_sort_uint,
         .plugin.print = lyplg_type_print_uint,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
-        .plugin.lyb_data_len = 2,
     }, {
         .module = "",
         .revision = NULL,
         .name = LY_TYPE_UINT32_STR,
 
-        .plugin.id = "libyang 2 - integers, version 1",
+        .plugin.id = "ly2 integers",
+        .plugin.lyb_size = lyplg_type_lyb_size_variable_bits,
         .plugin.store = lyplg_type_store_uint,
-        .plugin.validate = lyplg_type_validate_uint,
+        .plugin.validate_value = lyplg_type_validate_value_uint,
+        .plugin.validate_tree = NULL,
         .plugin.compare = lyplg_type_compare_uint,
         .plugin.sort = lyplg_type_sort_uint,
         .plugin.print = lyplg_type_print_uint,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
-        .plugin.lyb_data_len = 4,
     }, {
         .module = "",
         .revision = NULL,
         .name = LY_TYPE_UINT64_STR,
 
-        .plugin.id = "libyang 2 - integers, version 1",
+        .plugin.id = "ly2 integers",
+        .plugin.lyb_size = lyplg_type_lyb_size_variable_bits,
         .plugin.store = lyplg_type_store_uint,
-        .plugin.validate = lyplg_type_validate_uint,
+        .plugin.validate_value = lyplg_type_validate_value_uint,
+        .plugin.validate_tree = NULL,
         .plugin.compare = lyplg_type_compare_uint,
         .plugin.sort = lyplg_type_sort_uint,
         .plugin.print = lyplg_type_print_uint,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
-        .plugin.lyb_data_len = 8,
     }, {
         .module = "",
         .revision = NULL,
         .name = LY_TYPE_INT8_STR,
 
-        .plugin.id = "libyang 2 - integers, version 1",
+        .plugin.id = "ly2 integers",
+        .plugin.lyb_size = lyplg_type_lyb_size_variable_bits,
         .plugin.store = lyplg_type_store_int,
-        .plugin.validate = lyplg_type_validate_int,
+        .plugin.validate_value = lyplg_type_validate_value_int,
+        .plugin.validate_tree = NULL,
         .plugin.compare = lyplg_type_compare_int,
         .plugin.sort = lyplg_type_sort_int,
         .plugin.print = lyplg_type_print_int,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
-        .plugin.lyb_data_len = 1,
     }, {
         .module = "",
         .revision = NULL,
         .name = LY_TYPE_INT16_STR,
 
-        .plugin.id = "libyang 2 - integers, version 1",
+        .plugin.id = "ly2 integers",
+        .plugin.lyb_size = lyplg_type_lyb_size_variable_bits,
         .plugin.store = lyplg_type_store_int,
-        .plugin.validate = lyplg_type_validate_int,
+        .plugin.validate_value = lyplg_type_validate_value_int,
+        .plugin.validate_tree = NULL,
         .plugin.compare = lyplg_type_compare_int,
         .plugin.sort = lyplg_type_sort_int,
         .plugin.print = lyplg_type_print_int,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
-        .plugin.lyb_data_len = 2,
     }, {
         .module = "",
         .revision = NULL,
         .name = LY_TYPE_INT32_STR,
 
-        .plugin.id = "libyang 2 - integers, version 1",
+        .plugin.id = "ly2 integers",
+        .plugin.lyb_size = lyplg_type_lyb_size_variable_bits,
         .plugin.store = lyplg_type_store_int,
-        .plugin.validate = lyplg_type_validate_int,
+        .plugin.validate_value = lyplg_type_validate_value_int,
+        .plugin.validate_tree = NULL,
         .plugin.compare = lyplg_type_compare_int,
         .plugin.sort = lyplg_type_sort_int,
         .plugin.print = lyplg_type_print_int,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
-        .plugin.lyb_data_len = 4,
     }, {
         .module = "",
         .revision = NULL,
         .name = LY_TYPE_INT64_STR,
 
-        .plugin.id = "libyang 2 - integers, version 1",
+        .plugin.id = "ly2 integers",
+        .plugin.lyb_size = lyplg_type_lyb_size_variable_bits,
         .plugin.store = lyplg_type_store_int,
-        .plugin.validate = lyplg_type_validate_int,
+        .plugin.validate_value = lyplg_type_validate_value_int,
+        .plugin.validate_tree = NULL,
         .plugin.compare = lyplg_type_compare_int,
         .plugin.sort = lyplg_type_sort_int,
         .plugin.print = lyplg_type_print_int,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
-        .plugin.lyb_data_len = 8,
     },
     {0}
 };
