@@ -3,7 +3,7 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief ietf-yang-types xpath1.0 type plugin.
  *
- * Copyright (c) 2021 - 2023 CESNET, z.s.p.o.
+ * Copyright (c) 2021 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -32,9 +32,9 @@
  * @page howtoDataLYB LYB Binary Format
  * @subsection howtoDataLYBTypesXpath10 xpath1.0 (ietf-yang-types)
  *
- * | Size (B) | Mandatory | Type | Meaning |
+ * | Size (b) | Mandatory | Type | Meaning |
  * | :------  | :-------: | :--: | :-----: |
- * | string length | yes | `char *` | string JSON format of the XPath expression |
+ * | variable, full bytes | yes | `char *` | string JSON format of the XPath expression |
  */
 
 LIBYANG_API_DEF LY_ERR
@@ -242,13 +242,14 @@ lyplg_type_print_xpath10_value(const struct lyd_value_xpath10 *xp_val, LY_VALUE_
     return ret;
 }
 
-LIBYANG_API_DEF LY_ERR
-lyplg_type_store_xpath10(const struct ly_ctx *ctx, const struct lysc_type *type, const void *value, size_t value_len,
+static LY_ERR
+lyplg_type_store_xpath10(const struct ly_ctx *ctx, const struct lysc_type *type, const void *value, uint32_t value_size_bits,
         uint32_t options, LY_VALUE_FORMAT format, void *prefix_data, uint32_t hints, const struct lysc_node *ctx_node,
-        struct lyd_value *storage, struct lys_glob_unres *UNUSED(unres), struct ly_err_item **err)
+        const struct lysc_ext_instance *UNUSED(top_ext), struct lyd_value *storage, struct lys_glob_unres *UNUSED(unres),
+        struct ly_err_item **err)
 {
     LY_ERR ret = LY_SUCCESS;
-    struct lysc_type_str *type_str = (struct lysc_type_str *)type;
+    uint32_t value_size;
     struct lyd_value_xpath10 *val;
     char *canon;
 
@@ -258,23 +259,17 @@ lyplg_type_store_xpath10(const struct ly_ctx *ctx, const struct lysc_type *type,
     LY_CHECK_ERR_GOTO(!val, ret = LY_EMEM, cleanup);
     storage->realtype = type;
 
-    /* check hints */
-    ret = lyplg_type_check_hints(hints, value, value_len, type->basetype, NULL, err);
+    /* check value length */
+    ret = lyplg_type_check_value_size("xpath1.0", format, value_size_bits, LYPLG_LYB_SIZE_VARIABLE_BYTES, 0,
+            &value_size, err);
     LY_CHECK_GOTO(ret, cleanup);
 
-    /* length restriction of the string */
-    if (type_str->length) {
-        /* value_len is in bytes, but we need number of characters here */
-        ret = lyplg_type_validate_range(LY_TYPE_STRING, type_str->length, ly_utf8len(value, value_len), value, value_len, err);
-        LY_CHECK_GOTO(ret, cleanup);
-    }
-
-    /* pattern restrictions */
-    ret = lyplg_type_validate_patterns(type_str->patterns, value, value_len, err);
+    /* check hints */
+    ret = lyplg_type_check_hints(hints, value, value_size, type->basetype, NULL, err);
     LY_CHECK_GOTO(ret, cleanup);
 
     /* parse */
-    ret = lyxp_expr_parse(ctx, value_len ? value : "", value_len, 1, &val->exp);
+    ret = lyxp_expr_parse(ctx, value_size ? value : "", value_size, 1, &val->exp);
     LY_CHECK_GOTO(ret, cleanup);
     val->ctx = ctx;
 
@@ -285,7 +280,7 @@ lyplg_type_store_xpath10(const struct ly_ctx *ctx, const struct lysc_type *type,
         LY_CHECK_GOTO(ret, cleanup);
     } else {
         /* store format-specific data and context for later prefix resolution */
-        ret = lyplg_type_prefix_data_new(ctx, value, value_len, format, prefix_data, &val->format, &val->prefix_data);
+        ret = lyplg_type_prefix_data_new(ctx, value, value_size, format, prefix_data, &val->format, &val->prefix_data);
         LY_CHECK_GOTO(ret, cleanup);
     }
 
@@ -300,7 +295,7 @@ lyplg_type_store_xpath10(const struct ly_ctx *ctx, const struct lysc_type *type,
             options &= ~LYPLG_TYPE_STORE_DYNAMIC;
             LY_CHECK_GOTO(ret, cleanup);
         } else {
-            ret = lydict_insert(ctx, value_len ? value : "", value_len, &storage->_canonical);
+            ret = lydict_insert(ctx, value_size ? value : "", value_size, &storage->_canonical);
             LY_CHECK_GOTO(ret, cleanup);
         }
         break;
@@ -374,12 +369,12 @@ cleanup:
 }
 
 /**
- * @brief Implementation of ::lyplg_type_validate_clb for the xpath1.0 ietf-yang-types type.
+ * @brief Implementation of ::lyplg_type_validate_tree_clb for the xpath1.0 ietf-yang-types type.
  */
 static LY_ERR
-lyplg_type_validate_xpath10(const struct ly_ctx *UNUSED(ctx), const struct lysc_type *UNUSED(type),
-        const struct lyd_node *ctx_node, const struct lyd_node *UNUSED(tree), struct lyd_value *storage,
-        struct ly_err_item **err)
+lyplg_type_validate_tree_xpath10(const struct ly_ctx *UNUSED(ctx), const struct lysc_type *UNUSED(type),
+        const struct lyd_node *ctx_node, const struct lyd_node *UNUSED(tree), const struct lysc_ext_instance *UNUSED(top_ext),
+        struct lyd_value *storage, struct ly_err_item **err)
 {
     LY_ERR ret = LY_SUCCESS;
     struct lyd_value_xpath10 *val;
@@ -438,7 +433,7 @@ cleanup:
 
 LIBYANG_API_DEF const void *
 lyplg_type_print_xpath10(const struct ly_ctx *ctx, const struct lyd_value *value, LY_VALUE_FORMAT format,
-        void *prefix_data, ly_bool *dynamic, size_t *value_len)
+        void *prefix_data, ly_bool *dynamic, uint32_t *value_size_bits)
 {
     struct lyd_value_xpath10 *val;
     char *ret;
@@ -453,8 +448,8 @@ lyplg_type_print_xpath10(const struct ly_ctx *ctx, const struct lyd_value *value
         if (dynamic) {
             *dynamic = 0;
         }
-        if (value_len) {
-            *value_len = strlen(value->_canonical);
+        if (value_size_bits) {
+            *value_size_bits = strlen(value->_canonical) * 8;
         }
         return value->_canonical;
     }
@@ -469,8 +464,8 @@ lyplg_type_print_xpath10(const struct ly_ctx *ctx, const struct lyd_value *value
     }
 
     *dynamic = 1;
-    if (value_len) {
-        *value_len = strlen(ret);
+    if (value_size_bits) {
+        *value_size_bits = strlen(ret) * 8;
     }
     return ret;
 }
@@ -516,7 +511,7 @@ lyplg_type_free_xpath10(const struct ly_ctx *ctx, struct lyd_value *value)
     value->_canonical = NULL;
     LYD_VALUE_GET(value, val);
     if (val) {
-        lyxp_expr_free(ctx, val->exp);
+        lyxp_expr_free(val->exp);
         lyplg_type_prefix_data_free(val->format, val->prefix_data);
 
         LYPLG_TYPE_VAL_INLINE_DESTROY(val);
@@ -536,15 +531,16 @@ const struct lyplg_type_record plugins_xpath10[] = {
         .revision = "2013-07-15",
         .name = "xpath1.0",
 
-        .plugin.id = "libyang 2 - xpath1.0, version 1",
+        .plugin.id = "ly2 xpath1.0",
+        .plugin.lyb_size = lyplg_type_lyb_size_variable_bytes,
         .plugin.store = lyplg_type_store_xpath10,
-        .plugin.validate = lyplg_type_validate_xpath10,
+        .plugin.validate_value = NULL,
+        .plugin.validate_tree = lyplg_type_validate_tree_xpath10,
         .plugin.compare = lyplg_type_compare_simple,
         .plugin.sort = lyplg_type_sort_simple,
         .plugin.print = lyplg_type_print_xpath10,
         .plugin.duplicate = lyplg_type_dup_xpath10,
         .plugin.free = lyplg_type_free_xpath10,
-        .plugin.lyb_data_len = -1,
     },
     {0}
 };
