@@ -25,27 +25,29 @@
 #include "out_internal.h"
 #include "parser_data.h"
 #include "plugins_exts/metadata.h"
+#include "plugins_internal.h"
 #include "plugins_types.h"
 #include "printer_data.h"
 #include "printer_internal.h"
 #include "set.h"
 #include "tree.h"
 #include "tree_data.h"
+#include "tree_data_internal.h"
 #include "tree_schema.h"
 
 /**
  * @brief JSON printer context.
  */
 struct jsonpr_ctx {
-    struct ly_out *out;         /**< output specification */
+    struct ly_out *out;             /**< output specification */
     const struct lyd_node *root;    /**< root node of the subtree being printed */
     const struct lyd_node *parent;  /**< parent of the node being printed */
-    uint16_t level;             /**< current indentation level: 0 - no formatting, >= 1 indentation levels */
-    uint32_t options;           /**< [Data printer flags](@ref dataprinterflags) */
-    const struct ly_ctx *ctx;   /**< libyang context */
+    uint16_t level;                 /**< current indentation level: 0 - no formatting, >= 1 indentation levels */
+    uint32_t options;               /**< [Data printer flags](@ref dataprinterflags) */
+    const struct ly_ctx *ctx;       /**< libyang context */
 
-    uint16_t level_printed;     /* level where some data were already printed */
-    struct ly_set open;         /* currently open array(s) */
+    uint16_t level_printed;         /**< level where some data were already printed */
+    struct ly_set open;             /**< currently open array(s) */
     const struct lyd_node *first_leaflist;  /**< first printed leaf-list instance, used when printing its metadata/attributes */
 };
 
@@ -60,6 +62,10 @@ struct jsonpr_ctx {
         ly_print_(pctx->out, ",%s", (DO_FORMAT ? "\n" : "")); \
     }
 
+static LY_ERR json_print_leaf_list_empty(struct jsonpr_ctx *pctx, const struct lysc_node *snode);
+static const struct lysc_node *json_print_next_empty_leaf_list(struct jsonpr_ctx *pctx, const struct lysc_node *snode,
+        const struct lyd_node *siblings);
+
 static LY_ERR json_print_node(struct jsonpr_ctx *pctx, const struct lyd_node *node);
 
 /**
@@ -143,34 +149,48 @@ json_print_array_close(struct jsonpr_ctx *pctx)
  * @brief Get the node's module name to use as the @p node prefix in JSON.
  *
  * @param[in] node Node to process.
- * @return The name of the module where the @p node belongs, it can be NULL in case the module name
- * cannot be determined (source format is XML and the refered namespace is unknown/not implemented in the current context).
+ * @param[in] snode Schema node to process.
+ * @param[out] mod_name Module name of @p node, can be NULL (format is XML and the module with NS is not in the context).
+ * @param[out] data_dict Whether @p mod_name is from the schema or data dictionary.
  */
-static const char *
-node_prefix(const struct lyd_node *node)
+static void
+node_prefix(const struct lyd_node *node, const struct lysc_node *snode, const char **mod_name, ly_bool *data_dict)
 {
-    if (node->schema) {
-        return node->schema->module->name;
-    } else {
-        struct lyd_node_opaq *onode = (struct lyd_node_opaq *)node;
-        const struct lys_module *mod;
+    struct lyd_node_opaq *onode;
+    const struct lys_module *mod;
+
+    *mod_name = NULL;
+
+    if (snode) {
+        *mod_name = snode->module->name;
+        if (data_dict) {
+            *data_dict = 0;
+        }
+    } else if (node) {
+        onode = (struct lyd_node_opaq *)node;
 
         switch (onode->format) {
         case LY_VALUE_JSON:
-            return onode->name.module_name;
+            *mod_name = onode->name.module_name;
+            if (data_dict) {
+                *data_dict = 1;
+            }
+            break;
         case LY_VALUE_XML:
             mod = ly_ctx_get_module_implemented_ns(onode->ctx, onode->name.module_ns);
-            if (!mod) {
-                return NULL;
+            if (mod) {
+                *mod_name = mod->name;
+                if (data_dict) {
+                    *data_dict = 0;
+                }
             }
-            return mod->name;
+            break;
         default:
             /* cannot be created */
             LOGINT(LYD_CTX(node));
+            break;
         }
     }
-
-    return NULL;
 }
 
 /**
@@ -178,20 +198,24 @@ node_prefix(const struct lyd_node *node)
  *
  * Accepts both regulard a well as opaq nodes.
  *
- * @param[in] node1 The first node to compare.
- * @param[in] node2 The second node to compare.
+ * @param[in] node1 First data node to compare.
+ * @param[in] snode1 First schema node to compare, if there is no @p node1.
+ * @param[in] node2 Second data node to compare.
  * @return 0 in case the nodes' modules are the same
  * @return 1 in case the nodes belongs to different modules
  */
-int
-json_nscmp(const struct lyd_node *node1, const struct lyd_node *node2)
+static int
+json_nscmp(const struct lyd_node *node1, const struct lysc_node *snode1, const struct lyd_node *node2)
 {
-    assert(node1 || node2);
+    const char *pref1, *pref2;
+    ly_bool dd1, dd2;
 
-    if (!node1 || !node2) {
+    assert(snode1 || node2);
+
+    if (!snode1 || !node2) {
         return 1;
-    } else if (node1->schema && node2->schema) {
-        if (node1->schema->module == node2->schema->module) {
+    } else if (snode1 && node2->schema) {
+        if (snode1->module == node2->schema->module) {
             /* belongs to the same module */
             return 0;
         } else {
@@ -199,10 +223,10 @@ json_nscmp(const struct lyd_node *node1, const struct lyd_node *node2)
             return 1;
         }
     } else {
-        const char *pref1 = node_prefix(node1);
-        const char *pref2 = node_prefix(node2);
+        node_prefix(node1, snode1, &pref1, &dd1);
+        node_prefix(node2, node2->schema, &pref2, &dd2);
 
-        if ((pref1 && pref2) && (pref1 == pref2)) {
+        if (pref1 && pref2 && (((dd1 == dd2) && (pref1 == pref2)) || ((dd1 != dd2) && !strcmp(pref1, pref2)))) {
             return 0;
         } else {
             return 1;
@@ -263,20 +287,32 @@ json_print_string(struct ly_out *out, const char *text)
  * @brief Print JSON object's member name, ending by ':'. It resolves if the prefix is supposed to be printed.
  *
  * @param[in] ctx JSON printer context.
- * @param[in] node The data node being printed.
+ * @param[in] node Data node being printed.
+ * @param[in] snode Optional schema node if there is no @p node.
  * @param[in] is_attr Flag if the metadata sign (@) is supposed to be added before the identifier.
  * @return LY_ERR value.
  */
 static LY_ERR
-json_print_member(struct jsonpr_ctx *pctx, const struct lyd_node *node, ly_bool is_attr)
+json_print_member(struct jsonpr_ctx *pctx, const struct lyd_node *node, const struct lysc_node *snode, ly_bool is_attr)
 {
-    PRINT_COMMA;
-    if ((LEVEL == 1) || json_nscmp(node, pctx->parent)) {
-        /* print "namespace" */
-        ly_print_(pctx->out, "%*s\"%s%s:%s\":%s", INDENT, is_attr ? "@" : "",
-                node_prefix(node), node->schema->name, DO_FORMAT ? " " : "");
+    const char *pref, *name;
+
+    assert(node || snode);
+
+    if (node) {
+        snode = node->schema;
+        name = LYD_NAME(node);
     } else {
-        ly_print_(pctx->out, "%*s\"%s%s\":%s", INDENT, is_attr ? "@" : "", node->schema->name, DO_FORMAT ? " " : "");
+        name = snode->name;
+    }
+
+    PRINT_COMMA;
+    if ((LEVEL == 1) || json_nscmp(node, snode, pctx->parent)) {
+        /* print "namespace" */
+        node_prefix(node, snode, &pref, NULL);
+        ly_print_(pctx->out, "%*s\"%s%s:%s\":%s", INDENT, is_attr ? "@" : "", pref, name, DO_FORMAT ? " " : "");
+    } else {
+        ly_print_(pctx->out, "%*s\"%s%s\":%s", INDENT, is_attr ? "@" : "", name, DO_FORMAT ? " " : "");
     }
 
     return LY_SUCCESS;
@@ -296,7 +332,8 @@ static LY_ERR
 json_print_member2(struct jsonpr_ctx *pctx, const struct lyd_node *parent, LY_VALUE_FORMAT format,
         const struct ly_opaq_name *name, ly_bool is_attr)
 {
-    const char *module_name = NULL, *name_str;
+    const char *module_name = NULL, *name_str, *pmod_name;
+    const struct lys_module *mod;
 
     PRINT_COMMA;
 
@@ -306,9 +343,8 @@ json_print_member2(struct jsonpr_ctx *pctx, const struct lyd_node *parent, LY_VA
         case LY_VALUE_JSON:
             module_name = name->module_name;
             break;
-        case LY_VALUE_XML: {
-            const struct lys_module *mod = NULL;
-
+        case LY_VALUE_XML:
+            mod = NULL;
             if (name->module_ns) {
                 mod = ly_ctx_get_module_implemented_ns(pctx->ctx, name->module_ns);
             }
@@ -316,7 +352,6 @@ json_print_member2(struct jsonpr_ctx *pctx, const struct lyd_node *parent, LY_VA
                 module_name = mod->name;
             }
             break;
-        }
         default:
             /* cannot be created */
             LOGINT_RET(pctx->ctx);
@@ -327,8 +362,9 @@ json_print_member2(struct jsonpr_ctx *pctx, const struct lyd_node *parent, LY_VA
         name_str = "";
     }
 
-    /* print the member */
-    if (module_name && (!parent || (node_prefix(parent) != module_name))) {
+    /* print the member, strcmp because node prefix is in the schema dict, module_name in data dict */
+    node_prefix(parent, parent ? parent->schema : NULL, &pmod_name, NULL);
+    if (module_name && (!parent || strcmp(pmod_name, module_name))) {
         ly_print_(pctx->out, "%*s\"%s%s:%s\":%s", INDENT, is_attr ? "@" : "", module_name, name_str, DO_FORMAT ? " " : "");
     } else {
         ly_print_(pctx->out, "%*s\"%s%s\":%s", INDENT, is_attr ? "@" : "", name_str, DO_FORMAT ? " " : "");
@@ -354,7 +390,7 @@ json_print_value(struct jsonpr_ctx *pctx, const struct ly_ctx *ctx, const struct
     LY_DATA_TYPE basetype;
     const char *value;
 
-    value = val->realtype->plugin->print(ctx, val, LY_VALUE_JSON, (void *)local_mod, &dynamic, NULL);
+    value = LYSC_GET_TYPE_PLG(val->realtype->plugin_ref)->print(ctx, val, LY_VALUE_JSON, (void *)local_mod, &dynamic, NULL);
     LY_CHECK_RET(!value, LY_EINVAL);
     basetype = val->realtype->basetype;
 
@@ -502,25 +538,26 @@ node_has_printable_meta(const struct lyd_node *node)
 static LY_ERR
 json_print_attributes(struct jsonpr_ctx *pctx, const struct lyd_node *node, ly_bool inner)
 {
-    const struct lys_module *wdmod = NULL;
+    const struct lys_module *df_mod = NULL;
 
     if (node->schema && (node->schema->nodetype != LYS_CONTAINER) && (((node->flags & LYD_DEFAULT) &&
             (pctx->options & (LYD_PRINT_WD_ALL_TAG | LYD_PRINT_WD_IMPL_TAG))) ||
             ((pctx->options & LYD_PRINT_WD_ALL_TAG) && lyd_is_default(node)))) {
-        /* we have implicit OR explicit default node */
-        /* get with-defaults module */
-        wdmod = ly_ctx_get_module_implemented(LYD_CTX(node), "ietf-netconf-with-defaults");
+        /* we have implicit OR explicit default node, print only if we have with-defaults module */
+        if (ly_ctx_get_module_implemented(LYD_CTX(node), "ietf-netconf-with-defaults")) {
+            df_mod = ly_ctx_get_module_implemented(LYD_CTX(node), "default");
+        }
     }
 
-    if (node->schema && (wdmod || node_has_printable_meta(node))) {
+    if (node->schema && (df_mod || node_has_printable_meta(node))) {
         if (inner) {
             LY_CHECK_RET(json_print_member2(pctx, lyd_parent(node), LY_VALUE_JSON, NULL, 1));
         } else {
-            LY_CHECK_RET(json_print_member(pctx, node, 1));
+            LY_CHECK_RET(json_print_member(pctx, node, NULL, 1));
         }
         ly_print_(pctx->out, "{%s", (DO_FORMAT ? "\n" : ""));
         LEVEL_INC;
-        LY_CHECK_RET(json_print_metadata(pctx, node, wdmod));
+        LY_CHECK_RET(json_print_metadata(pctx, node, df_mod));
         LEVEL_DEC;
         ly_print_(pctx->out, "%s%*s}", DO_FORMAT ? "\n" : "", INDENT);
         LEVEL_PRINTED;
@@ -552,7 +589,7 @@ json_print_attributes(struct jsonpr_ctx *pctx, const struct lyd_node *node, ly_b
 static LY_ERR
 json_print_leaf(struct jsonpr_ctx *pctx, const struct lyd_node *node)
 {
-    LY_CHECK_RET(json_print_member(pctx, node, 0));
+    LY_CHECK_RET(json_print_member(pctx, node, NULL, 0));
     LY_CHECK_RET(json_print_value(pctx, LYD_CTX(node), &((const struct lyd_node_term *)node)->value, node->schema->module));
     LEVEL_PRINTED;
 
@@ -575,30 +612,13 @@ json_print_any_content(struct jsonpr_ctx *pctx, struct lyd_node_any *any)
     LY_ERR ret = LY_SUCCESS;
     struct lyd_node *iter;
     const struct lyd_node *prev_parent;
-    uint32_t prev_opts, *prev_lo, temp_lo = 0;
+    uint32_t prev_opts;
 
     assert(any->schema->nodetype & LYD_NODE_ANY);
 
     if ((any->schema->nodetype == LYS_ANYDATA) && (any->value_type != LYD_ANYDATA_DATATREE)) {
         LOGINT_RET(pctx->ctx);
     }
-    if (any->value_type == LYD_ANYDATA_LYB) {
-        uint32_t parser_options = LYD_PARSE_ONLY | LYD_PARSE_OPAQ | LYD_PARSE_STRICT;
-
-        /* turn logging off */
-        prev_lo = ly_temp_log_options(&temp_lo);
-
-        /* try to parse it into a data tree */
-        if (lyd_parse_data_mem(pctx->ctx, any->value.mem, LYD_LYB, parser_options, 0, &iter) == LY_SUCCESS) {
-            /* successfully parsed */
-            free(any->value.mem);
-            any->value.tree = iter;
-            any->value_type = LYD_ANYDATA_DATATREE;
-        }
-
-        /* turn logging on again */
-        ly_temp_log_options(prev_lo);
-    }
 
     switch (any->value_type) {
     case LYD_ANYDATA_DATATREE:
@@ -610,7 +630,7 @@ json_print_any_content(struct jsonpr_ctx *pctx, struct lyd_node_any *any)
         prev_parent = pctx->parent;
         prev_opts = pctx->options;
         pctx->parent = &any->node;
-        pctx->options &= ~LYD_PRINT_WITHSIBLINGS;
+        pctx->options &= ~LYD_PRINT_SIBLINGS;
         LY_LIST_FOR(any->value.tree, iter) {
             ret = json_print_node(pctx, iter);
             LY_CHECK_ERR_RET(ret, LEVEL_DEC, ret);
@@ -653,10 +673,6 @@ json_print_any_content(struct jsonpr_ctx *pctx, struct lyd_node_any *any)
             json_print_string(pctx->out, any->value.str);
         }
         break;
-    case LYD_ANYDATA_LYB:
-        /* LYB format is not supported */
-        LOGWRN(pctx->ctx, "Unable to print anydata content (type %d) as JSON.", any->value_type);
-        break;
     }
 
     return LY_SUCCESS;
@@ -676,14 +692,17 @@ json_print_inner(struct jsonpr_ctx *pctx, const struct lyd_node *node)
     struct lyd_node *child;
     const struct lyd_node *prev_parent;
     struct lyd_node_opaq *opaq = NULL;
-    ly_bool has_content = 0;
+    const struct lysc_node *snode;
+    ly_bool has_content = 0, no_child_print = 1;
 
     LY_LIST_FOR(lyd_child(node), child) {
         if (lyd_node_should_print(child, pctx->options)) {
+            no_child_print = 0;
             break;
         }
     }
-    if (node->meta || child) {
+    if (node->meta || child || ((pctx->options & LYD_PRINT_EMPTY_LEAF_LIST) &&
+            json_print_next_empty_leaf_list(pctx, lysc_node_child(node->schema), lyd_child(node)))) {
         has_content = 1;
     }
     if (!node->schema) {
@@ -710,6 +729,18 @@ json_print_inner(struct jsonpr_ctx *pctx, const struct lyd_node *node)
     }
     pctx->parent = prev_parent;
 
+    if (no_child_print && (pctx->options & LYD_PRINT_EMPTY_LEAF_LIST)) {
+        snode = json_print_next_empty_leaf_list(pctx, lysc_node_child(node->schema), NULL);
+        while (snode) {
+            /* print an empty (leaf-)list */
+            LY_CHECK_RET(json_print_leaf_list_empty(pctx, snode));
+            pctx->level_printed = pctx->level;
+
+            /* next iter */
+            snode = json_print_next_empty_leaf_list(pctx, snode->next, NULL);
+        }
+    }
+
     LEVEL_DEC;
     if (DO_FORMAT && has_content) {
         ly_print_(pctx->out, "\n%*s}", INDENT);
@@ -731,7 +762,7 @@ json_print_inner(struct jsonpr_ctx *pctx, const struct lyd_node *node)
 static int
 json_print_container(struct jsonpr_ctx *pctx, const struct lyd_node *node)
 {
-    LY_CHECK_RET(json_print_member(pctx, node, 0));
+    LY_CHECK_RET(json_print_member(pctx, node, NULL, 0));
     LY_CHECK_RET(json_print_inner(pctx, node));
 
     return LY_SUCCESS;
@@ -747,7 +778,7 @@ json_print_container(struct jsonpr_ctx *pctx, const struct lyd_node *node)
 static int
 json_print_any(struct jsonpr_ctx *pctx, const struct lyd_node *node)
 {
-    LY_CHECK_RET(json_print_member(pctx, node, 0));
+    LY_CHECK_RET(json_print_member(pctx, node, NULL, 0));
     LY_CHECK_RET(json_print_any_content(pctx, (struct lyd_node_any *)node));
     LEVEL_PRINTED;
 
@@ -772,7 +803,7 @@ json_print_array_is_last_inst(struct jsonpr_ctx *pctx, const struct lyd_node *no
         return 0;
     }
 
-    if ((pctx->root == node) && !(pctx->options & LYD_PRINT_WITHSIBLINGS)) {
+    if ((pctx->root == node) && !(pctx->options & LYD_PRINT_SIBLINGS)) {
         /* the only printed instance */
         return 1;
     }
@@ -799,10 +830,10 @@ json_print_array_is_last_inst(struct jsonpr_ctx *pctx, const struct lyd_node *no
 static LY_ERR
 json_print_leaf_list(struct jsonpr_ctx *pctx, const struct lyd_node *node)
 {
-    const struct lys_module *wdmod = NULL;
+    const struct lys_module *df_mod = NULL;
 
     if (!is_open_array(pctx, node)) {
-        LY_CHECK_RET(json_print_member(pctx, node, 0));
+        LY_CHECK_RET(json_print_member(pctx, node, NULL, 0));
         LY_CHECK_RET(json_print_array_open(pctx, node));
         if (node->schema->nodetype == LYS_LEAFLIST) {
             ly_print_(pctx->out, "%*s", INDENT);
@@ -822,10 +853,12 @@ json_print_leaf_list(struct jsonpr_ctx *pctx, const struct lyd_node *node)
         if (!pctx->first_leaflist) {
             if (((node->flags & LYD_DEFAULT) && (pctx->options & (LYD_PRINT_WD_ALL_TAG | LYD_PRINT_WD_IMPL_TAG))) ||
                     ((pctx->options & LYD_PRINT_WD_ALL_TAG) && lyd_is_default(node))) {
-                /* we have implicit OR explicit default node, get with-defaults module */
-                wdmod = ly_ctx_get_module_implemented(LYD_CTX(node), "ietf-netconf-with-defaults");
+                /* we have implicit OR explicit default node, print only if we have with-defaults module */
+                if (ly_ctx_get_module_implemented(LYD_CTX(node), "ietf-netconf-with-defaults")) {
+                    df_mod = ly_ctx_get_module_implemented(LYD_CTX(node), "default");
+                }
             }
-            if (wdmod || node_has_printable_meta(node)) {
+            if (df_mod || node_has_printable_meta(node)) {
                 /* we will be printing metadata for these siblings */
                 pctx->first_leaflist = node;
             }
@@ -839,6 +872,34 @@ json_print_leaf_list(struct jsonpr_ctx *pctx, const struct lyd_node *node)
     return LY_SUCCESS;
 }
 
+/**
+ * @brief Print empty leaf-list or list instance.
+ *
+ * @param[in] ctx JSON printer context.
+ * @param[in] snode Schema node to print.
+ * @return LY_ERR value.
+ */
+static LY_ERR
+json_print_leaf_list_empty(struct jsonpr_ctx *pctx, const struct lysc_node *snode)
+{
+    assert(snode->nodetype & (LYS_LEAFLIST | LYS_LIST));
+
+    LY_CHECK_RET(json_print_member(pctx, NULL, snode, 0));
+
+    /* avoid an empty line */
+    if (!(pctx->options & LY_PRINT_SHRINK)) {
+        pctx->options |= LY_PRINT_SHRINK;
+        LY_CHECK_RET(json_print_array_open(pctx, (void *)snode));
+        pctx->options &= ~LY_PRINT_SHRINK;
+    } else {
+        LY_CHECK_RET(json_print_array_open(pctx, (void *)snode));
+    }
+
+    json_print_array_close(pctx);
+
+    return LY_SUCCESS;
+}
+
 /**
  * @brief Print leaf-list's metadata or opaque nodes attributes.
  * This function is supposed to be called when the leaf-list array is closed.
@@ -850,14 +911,16 @@ static LY_ERR
 json_print_meta_attr_leaflist(struct jsonpr_ctx *pctx)
 {
     const struct lyd_node *prev, *node, *iter;
-    const struct lys_module *wdmod = NULL, *iter_wdmod;
+    const struct lys_module *df_mod = NULL, *iter_dfmod;
     const struct lyd_node_opaq *opaq = NULL;
 
     assert(pctx->first_leaflist);
 
     if (pctx->options & (LYD_PRINT_WD_ALL_TAG | LYD_PRINT_WD_IMPL_TAG)) {
-        /* get with-defaults module */
-        wdmod = ly_ctx_get_module_implemented(pctx->ctx, "ietf-netconf-with-defaults");
+        /* we have implicit OR explicit default node, print only if we have with-defaults module */
+        if (ly_ctx_get_module_implemented(pctx->ctx, "ietf-netconf-with-defaults")) {
+            df_mod = ly_ctx_get_module_implemented(pctx->ctx, "default");
+        }
     }
 
     /* node is the first instance of the leaf-list */
@@ -866,7 +929,7 @@ json_print_meta_attr_leaflist(struct jsonpr_ctx *pctx)
             node = prev, prev = node->prev) {}
 
     if (node->schema) {
-        LY_CHECK_RET(json_print_member(pctx, node, 1));
+        LY_CHECK_RET(json_print_member(pctx, node, NULL, 1));
     } else {
         opaq = (struct lyd_node_opaq *)node;
         LY_CHECK_RET(json_print_member2(pctx, lyd_parent(node), opaq->format, &opaq->name, 1));
@@ -877,16 +940,16 @@ json_print_meta_attr_leaflist(struct jsonpr_ctx *pctx)
     LY_LIST_FOR(node, iter) {
         PRINT_COMMA;
         if (iter->schema && ((iter->flags & LYD_DEFAULT) || ((pctx->options & LYD_PRINT_WD_ALL_TAG) && lyd_is_default(iter)))) {
-            iter_wdmod = wdmod;
+            iter_dfmod = df_mod;
         } else {
-            iter_wdmod = NULL;
+            iter_dfmod = NULL;
         }
-        if ((iter->schema && (node_has_printable_meta(iter) || iter_wdmod)) || (opaq && opaq->attr)) {
+        if ((iter->schema && (node_has_printable_meta(iter) || iter_dfmod)) || (opaq && opaq->attr)) {
             ly_print_(pctx->out, "%*s%s", INDENT, DO_FORMAT ? "{\n" : "{");
             LEVEL_INC;
 
             if (iter->schema) {
-                LY_CHECK_RET(json_print_metadata(pctx, iter, iter_wdmod));
+                LY_CHECK_RET(json_print_metadata(pctx, iter, iter_dfmod));
             } else {
                 LY_CHECK_RET(json_print_attribute(pctx, (struct lyd_node_opaq *)iter));
             }
@@ -980,16 +1043,50 @@ json_print_opaq(struct jsonpr_ctx *pctx, const struct lyd_node_opaq *node)
     return LY_SUCCESS;
 }
 
+/**
+ * @brief Return the following empty (leaf-)lists if no predecessor is printed.
+ *
+ * @param[in] pctx JSON printer context.
+ * @param[in] snode First schema node to consider.
+ * @param[in] siblings Siblings to check for (leaf-)list existence.
+ * @return Next (leaf-)list without instances.
+ * @return NULL if no (leaf-)list without instances matches snode or following nodes until a node is printed.
+ */
+static const struct lysc_node *
+json_print_next_empty_leaf_list(struct jsonpr_ctx *pctx, const struct lysc_node *snode, const struct lyd_node *siblings)
+{
+    struct lyd_node *match;
+
+    /* look for following empty (leaf-)lists */
+    LY_LIST_FOR(snode, snode) {
+        if (!lyd_find_sibling_schema(siblings, snode, &match) && lyd_node_should_print(match, pctx->options)) {
+            /* instance exists and will be printed */
+            break;
+        }
+
+        if (snode->nodetype & (LYS_LEAFLIST | LYS_LIST)) {
+            assert(!match);
+            return snode;
+        }
+    }
+
+    /* no empty (leaf-)list to print */
+    return NULL;
+}
+
 /**
  * @brief Print all the types of data node including its metadata.
  *
- * @param[in] ctx JSON printer context.
+ * @param[in] pctx JSON printer context.
  * @param[in] node Data node to print.
  * @return LY_ERR value.
  */
 static LY_ERR
 json_print_node(struct jsonpr_ctx *pctx, const struct lyd_node *node)
 {
+    const struct lysc_node *snode;
+    ly_bool last_inst = 0;
+
     if (!lyd_node_should_print(node, pctx->options)) {
         if (json_print_array_is_last_inst(pctx, node)) {
             json_print_array_close(pctx);
@@ -1022,6 +1119,23 @@ json_print_node(struct jsonpr_ctx *pctx, const struct lyd_node *node)
             LOGINT(pctx->ctx);
             return EXIT_FAILURE;
         }
+
+        if (pctx->options & LYD_PRINT_EMPTY_LEAF_LIST) {
+            /* first check whether the last instance of a schema node has been printed */
+            if (!node->next || (node->schema != node->next->schema)) {
+                last_inst = 1;
+            }
+        }
+
+        if (last_inst) {
+            snode = json_print_next_empty_leaf_list(pctx, node->schema->next, node);
+            while (snode) {
+                /* print an empty (leaf-)list */
+                LY_CHECK_RET(json_print_leaf_list_empty(pctx, snode));
+
+                snode = json_print_next_empty_leaf_list(pctx, snode->next, node);
+            }
+        }
     }
 
     pctx->level_printed = pctx->level;
@@ -1061,7 +1175,7 @@ json_print_data(struct ly_out *out, const struct lyd_node *root, uint32_t option
     LY_LIST_FOR(root, node) {
         pctx.root = node;
         LY_CHECK_RET(json_print_node(&pctx, node));
-        if (!(options & LYD_PRINT_WITHSIBLINGS)) {
+        if (!(options & LYD_PRINT_SIBLINGS)) {
             break;
         }
     }
