@@ -26,6 +26,7 @@
 #include "out_internal.h"
 #include "parser_data.h"
 #include "plugins_exts/metadata.h"
+#include "plugins_internal.h"
 #include "plugins_types.h"
 #include "printer_data.h"
 #include "printer_internal.h"
@@ -192,7 +193,7 @@ static void
 xml_print_meta(struct xmlpr_ctx *pctx, const struct lyd_node *node)
 {
     struct lyd_meta *meta;
-    const struct lys_module *mod;
+    const struct lys_module *df_mod = NULL, *mod;
     struct ly_set ns_list = {0};
     LY_ARRAY_COUNT_TYPE u;
     ly_bool dynamic, filter_attrs = 0;
@@ -204,9 +205,11 @@ xml_print_meta(struct xmlpr_ctx *pctx, const struct lyd_node *node)
         if (((node->flags & LYD_DEFAULT) && (pctx->options & (LYD_PRINT_WD_ALL_TAG | LYD_PRINT_WD_IMPL_TAG))) ||
                 ((pctx->options & LYD_PRINT_WD_ALL_TAG) && lyd_is_default(node))) {
             /* we have implicit OR explicit default node, print attribute only if context include with-defaults schema */
-            mod = ly_ctx_get_module_latest(LYD_CTX(node), "ietf-netconf-with-defaults");
-            if (mod) {
-                ly_print_(pctx->out, " %s:default=\"true\"", xml_print_ns(pctx, mod->ns, mod->prefix, 0));
+            if (ly_ctx_get_module_latest(LYD_CTX(node), "ietf-netconf-with-defaults")) {
+                df_mod = ly_ctx_get_module_latest(LYD_CTX(node), "default");
+            }
+            if (df_mod) {
+                ly_print_(pctx->out, " %s:default=\"true\"", xml_print_ns(pctx, df_mod->ns, df_mod->prefix, 0));
             }
         }
     }
@@ -233,7 +236,8 @@ xml_print_meta(struct xmlpr_ctx *pctx, const struct lyd_node *node)
         ly_set_add(&ns_list, NULL, 0, NULL);
 
         /* print the value */
-        value = meta->value.realtype->plugin->print(LYD_CTX(node), &meta->value, LY_VALUE_XML, &ns_list, &dynamic, NULL);
+        value = LYSC_GET_TYPE_PLG(meta->value.realtype->plugin_ref)->print(LYD_CTX(node),
+                &meta->value, LY_VALUE_XML, &ns_list, &dynamic, NULL);
 
         /* print namespaces connected with the value's prefixes */
         for (i = 1; i < ns_list.count; ++i) {
@@ -358,8 +362,8 @@ xml_print_term(struct xmlpr_ctx *pctx, const struct lyd_node_term *node)
     }
 
     /* print the value */
-    value = ((struct lysc_node_leaf *)node->schema)->type->plugin->print(LYD_CTX(node), &node->value, LY_VALUE_XML,
-            &ns_list, &dynamic, NULL);
+    value = LYSC_GET_TYPE_PLG(((struct lysc_node_leaf *)node->schema)->type->plugin_ref)->print(LYD_CTX(node),
+            &node->value, LY_VALUE_XML, &ns_list, &dynamic, NULL);
     LY_CHECK_ERR_GOTO(!value, rc = LY_EINVAL, cleanup);
 
     /* print node opening */
@@ -433,7 +437,7 @@ xml_print_anydata(struct xmlpr_ctx *pctx, const struct lyd_node_any *node)
 {
     struct lyd_node_any *any = (struct lyd_node_any *)node;
     struct lyd_node *iter;
-    uint32_t prev_opts, *prev_lo, temp_lo = 0;
+    uint32_t prev_opts;
     LY_ERR ret;
 
     if ((node->schema->nodetype == LYS_ANYDATA) && (node->value_type != LYD_ANYDATA_DATATREE)) {
@@ -448,28 +452,11 @@ no_content:
         ly_print_(pctx->out, "/>%s", DO_FORMAT ? "\n" : "");
         return LY_SUCCESS;
     } else {
-        if (any->value_type == LYD_ANYDATA_LYB) {
-            /* turn logging off */
-            prev_lo = ly_temp_log_options(&temp_lo);
-
-            /* try to parse it into a data tree */
-            if (lyd_parse_data_mem((struct ly_ctx *)LYD_CTX(node), any->value.mem, LYD_LYB,
-                    LYD_PARSE_ONLY | LYD_PARSE_OPAQ | LYD_PARSE_STRICT, 0, &iter) == LY_SUCCESS) {
-                /* successfully parsed */
-                free(any->value.mem);
-                any->value.tree = iter;
-                any->value_type = LYD_ANYDATA_DATATREE;
-            }
-
-            /* turn logging on again */
-            ly_temp_log_options(prev_lo);
-        }
-
         switch (any->value_type) {
         case LYD_ANYDATA_DATATREE:
             /* close opening tag and print data */
             prev_opts = pctx->options;
-            pctx->options &= ~LYD_PRINT_WITHSIBLINGS;
+            pctx->options &= ~LYD_PRINT_SIBLINGS;
             LEVEL_INC;
 
             ly_print_(pctx->out, ">%s", DO_FORMAT ? "\n" : "");
@@ -498,10 +485,6 @@ no_content:
             }
             ly_print_(pctx->out, ">%s", any->value.str);
             break;
-        case LYD_ANYDATA_LYB:
-            /* LYB format is not supported */
-            LOGWRN(pctx->ctx, "Unable to print anydata content (type %d) as XML.", any->value_type);
-            goto no_content;
         }
 
         /* closing tag */
@@ -635,7 +618,7 @@ xml_print_data(struct ly_out *out, const struct lyd_node *root, uint32_t options
     /* content */
     LY_LIST_FOR(root, node) {
         LY_CHECK_RET(xml_print_node(&pctx, node));
-        if (!(options & LYD_PRINT_WITHSIBLINGS)) {
+        if (!(options & LYD_PRINT_SIBLINGS)) {
             break;
         }
     }
