@@ -4,7 +4,7 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief internal functions for YANG schema trees.
  *
- * Copyright (c) 2015 - 2024 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -206,14 +206,14 @@ LY_ERR ly_dup_prefix_data(const struct ly_ctx *ctx, LY_VALUE_FORMAT format, cons
  *
  * @param[in] ctx libyang context.
  * @param[in] value Value to be parsed.
- * @param[in] value_len Length of the @p value.
+ * @param[in] value_size Size of the @p value in bytes.
  * @param[in] format Format of the prefixes in the value.
  * @param[in] prefix_data Format-specific data for resolving any prefixes (see ::ly_resolve_prefix).
  * @param[in,out] format_p Resulting format of the prefixes.
  * @param[in,out] prefix_data_p Resulting prefix data for the value in format @p format_p.
  * @return LY_ERR value.
  */
-LY_ERR ly_store_prefix_data(const struct ly_ctx *ctx, const void *value, size_t value_len, LY_VALUE_FORMAT format,
+LY_ERR ly_store_prefix_data(const struct ly_ctx *ctx, const void *value, uint32_t value_size, LY_VALUE_FORMAT format,
         const void *prefix_data, LY_VALUE_FORMAT *format_p, void **prefix_data_p);
 
 /**
@@ -230,36 +230,23 @@ const char *ly_format2str(LY_VALUE_FORMAT format);
  * Hash is calculated and new node flag is set.
  *
  * @param[in] schema Schema node of the new data node.
- * @param[in] value String value to be parsed.
- * @param[in] value_len Length of @p value, must be set correctly.
+ * @param[in] value Value to be parsed.
+ * @param[in] value_size_bits Size of @p value in bits, must be set correctly.
  * @param[in] is_utf8 Whether @p value is a valid UTF-8 string, if applicable.
  * @param[in] store_only Whether to perform storing operation only.
  * @param[in,out] dynamic Flag if @p value is dynamically allocated, is adjusted when @p value is consumed.
  * @param[in] format Input format of @p value.
  * @param[in] prefix_data Format-specific data for resolving any prefixes (see ::ly_resolve_prefix).
  * @param[in] hints [Value hints](@ref lydvalhints) from the parser regarding the value type.
- * @param[out] incomplete Whether the value needs to be resolved.
+ * @param[in] top_ext Extension instance containing the definition of the data being created.
+ * @param[out] incomplete Whether the value needs to be resolved, optional.
  * @param[out] node Created node.
  * @return LY_SUCCESS on success.
- * @return LY_EINCOMPLETE in case data tree is needed to finish the validation.
  * @return LY_ERR value if an error occurred.
  */
-LY_ERR lyd_create_term(const struct lysc_node *schema, const char *value, size_t value_len, ly_bool is_utf8,
+LY_ERR lyd_create_term(const struct lysc_node *schema, const void *value, uint32_t value_size_bits, ly_bool is_utf8,
         ly_bool store_only, ly_bool *dynamic, LY_VALUE_FORMAT format, void *prefix_data, uint32_t hints,
-        ly_bool *incomplete, struct lyd_node **node);
-
-/**
- * @brief Create a term (leaf/leaf-list) node from a parsed value by duplicating it.
- *
- * Hash is calculated and new node flag is set.
- *
- * @param[in] schema Schema node of the new data node.
- * @param[in] val Parsed value to use.
- * @param[out] node Created node.
- * @return LY_SUCCESS on success.
- * @return LY_ERR value if an error occurred.
- */
-LY_ERR lyd_create_term2(const struct lysc_node *schema, const struct lyd_value *val, struct lyd_node **node);
+        const struct lysc_ext_instance *top_ext, ly_bool *incomplete, struct lyd_node **node);
 
 /**
  * @brief Create an inner (container/list/RPC/action/notification) node.
@@ -284,12 +271,13 @@ LY_ERR lyd_create_inner(const struct lysc_node *schema, struct lyd_node **node);
  * @param[in] predicates Compiled key list predicates.
  * @param[in] vars Array of defined variables to use in predicates, may be NULL.
  * @param[in] store_only Whether to perform storing operation only.
+ * @param[in] top_ext Extension instance containing the definition of the data being created.
  * @param[out] node Created node.
  * @return LY_SUCCESS on success.
  * @return LY_ERR value if an error occurred.
  */
 LY_ERR lyd_create_list(const struct lysc_node *schema, const struct ly_path_predicate *predicates,
-        const struct lyxp_var *vars, ly_bool store_only, struct lyd_node **node);
+        const struct lyxp_var *vars, ly_bool store_only, const struct lysc_ext_instance *top_ext, struct lyd_node **node);
 
 /**
  * @brief Create a list with all its keys (cannot be used for key-less list).
@@ -300,12 +288,13 @@ LY_ERR lyd_create_list(const struct lysc_node *schema, const struct ly_path_pred
  * @param[in] keys Key list predicates.
  * @param[in] keys_len Length of @p keys.
  * @param[in] store_only Whether to perform storing operation only.
+ * @param[in] top_ext Extension instance containing the definition of the data being created.
  * @param[out] node Created node.
  * @return LY_SUCCESS on success.
  * @return LY_ERR value if an error occurred.
  */
 LY_ERR lyd_create_list2(const struct lysc_node *schema, const char *keys, size_t keys_len, ly_bool store_only,
-        struct lyd_node **node);
+        const struct lysc_ext_instance *top_ext, struct lyd_node **node);
 
 /**
  * @brief Create an anyxml/anydata node.
@@ -347,8 +336,8 @@ LY_ERR lyd_create_any(const struct lysc_node *schema, const void *value, LYD_ANY
  * @return LY_SUCCESS on success.
  * @return LY_ERR value if an error occurred.
  */
-LY_ERR lyd_create_opaq(const struct ly_ctx *ctx, const char *name, size_t name_len, const char *prefix, size_t pref_len,
-        const char *module_key, size_t module_key_len, const char *value, size_t value_len, ly_bool *dynamic,
+LY_ERR lyd_create_opaq(const struct ly_ctx *ctx, const char *name, uint32_t name_len, const char *prefix, uint32_t pref_len,
+        const char *module_key, uint32_t module_key_len, const char *value, uint32_t value_len, ly_bool *dynamic,
         LY_VALUE_FORMAT format, void *val_prefix_data, uint32_t hints, struct lyd_node **node);
 
 /**
@@ -375,17 +364,19 @@ LY_ERR lyd_change_term_val(struct lyd_node *term, struct lyd_value *val, ly_bool
  * @param[in,out] first First sibling.
  * @param[in] sparent Schema parent of the siblings, NULL if schema of @p parent can be used.
  * @param[in] mod Module of the default values, NULL for nested siblings.
+ * @param[in] top_ext Extension instance containing the definition of the data being created.
  * @param[in] node_when Optional set to add nodes with "when" conditions into.
  * @param[in] node_types Optional set to add nodes with unresolved types into.
- * @param[in] ext_node Optional set to add nodes with extension instance node callbacks into.
+ * @param[in] ext_val Optional set to add nodes with extension instances into.
  * @param[in] impl_opts Implicit options (@ref implicitoptions).
  * @param[in,out] getnext_ht Getnext HT to use, new @p sparent is added to it.
  * @param[in,out] diff Validation diff.
  * @return LY_ERR value.
  */
 LY_ERR lyd_new_implicit(struct lyd_node *parent, struct lyd_node **first, const struct lysc_node *sparent,
-        const struct lys_module *mod, struct ly_set *node_when, struct ly_set *node_types, struct ly_set *ext_node,
-        uint32_t impl_opts, struct ly_ht *getnext_ht, struct lyd_node **diff);
+        const struct lys_module *mod, const struct lysc_ext_instance *top_ext, struct ly_set *node_when,
+        struct ly_set *node_types, struct ly_set *ext_val, uint32_t impl_opts, struct ly_ht *getnext_ht,
+        struct lyd_node **diff);
 
 /**
  * @brief Check the existence and create any non-existing implicit children, recursively for containers.
@@ -394,17 +385,19 @@ LY_ERR lyd_new_implicit(struct lyd_node *parent, struct lyd_node **first, const
  * @param[in,out] first First sibling.
  * @param[in] sparent Schema parent of the siblings, NULL if schema of @p parent can be used.
  * @param[in] mod Module of the default values, NULL for nested siblings.
+ * @param[in] top_ext Extension instance containing the definition of the data being created.
  * @param[in] node_when Optional set to add nodes with "when" conditions into.
  * @param[in] node_types Optional set to add nodes with unresolved types into.
- * @param[in] ext_node Optional set to add nodes with extension instance node callbacks into.
+ * @param[in] ext_val Optional set to add nodes with extension instances into.
  * @param[in] impl_opts Implicit options (@ref implicitoptions).
  * @param[in,out] getnext_ht Getnext HT to use, new @p sparent is added to it.
  * @param[in,out] diff Validation diff.
  * @return LY_ERR value.
  */
 LY_ERR lyd_new_implicit_r(struct lyd_node *parent, struct lyd_node **first, const struct lysc_node *sparent,
-        const struct lys_module *mod, struct ly_set *node_when, struct ly_set *node_types, struct ly_set *ext_node,
-        uint32_t impl_opts, struct ly_ht *getnext_ht, struct lyd_node **diff);
+        const struct lys_module *mod, const struct lysc_ext_instance *top_ext, struct ly_set *node_when,
+        struct ly_set *node_types, struct ly_set *ext_val, uint32_t impl_opts, struct ly_ht *getnext_ht,
+        struct lyd_node **diff);
 
 /**
  * @brief Find the next node, before which to insert the new node.
@@ -502,16 +495,26 @@ void lyd_insert_meta(struct lyd_node *parent, struct lyd_meta *meta, ly_bool cle
  */
 void lyd_unlink_meta_single(struct lyd_meta *meta);
 
+/**
+ * @brief Get the annotation definition in the module.
+ *
+ * @param[in] mod Metadata module (with the annotation definition).
+ * @param[in] name Metadata name.
+ * @param[in] name_len Length of @p name, must be set correctly.
+ * @return YANG extension annotation instance.
+ */
+struct lysc_ext_instance *lyd_get_meta_annotation(const struct lys_module *mod, const char *name, size_t name_len);
+
 /**
  * @brief Create and insert a metadata (last) into a parent.
  *
  * @param[in] parent Parent of the metadata, can be NULL.
  * @param[in,out] meta Metadata list to add at its end if @p parent is NULL, returned created attribute.
  * @param[in] mod Metadata module (with the annotation definition).
- * @param[in] name Attribute name.
+ * @param[in] name Metadata name.
  * @param[in] name_len Length of @p name, must be set correctly.
- * @param[in] value String value to be parsed.
- * @param[in] value_len Length of @p value, must be set correctly.
+ * @param[in] value Value to be parsed.
+ * @param[in] value_size_bits Size of @p value in bits, must be set correctly.
  * @param[in] is_utf8 Whether @p value is a valid UTF-8 string, if applicable.
  * @param[in] store_only Whether to perform storing operation only.
  * @param[in,out] dynamic Flag if @p value is dynamically allocated, is adjusted when @p value is consumed.
@@ -519,6 +522,7 @@ void lyd_unlink_meta_single(struct lyd_meta *meta);
  * @param[in] prefix_data Format-specific data for resolving any prefixes (see ::ly_resolve_prefix).
  * @param[in] hints [Value hints](@ref lydvalhints) from the parser regarding the value type.
  * @param[in] ctx_node Value context node, may be NULL for metadata.
+ * @param[in] top_ext Extension instance containing the definition of the data being created.
  * @param[in] clear_dflt Whether to clear dflt flag starting from @p parent, recursively all NP containers.
  * @param[out] incomplete Whether the value needs to be resolved.
  * @return LY_SUCCESS on success.
@@ -526,9 +530,9 @@ void lyd_unlink_meta_single(struct lyd_meta *meta);
  * @return LY_ERR value if an error occurred.
  */
 LY_ERR lyd_create_meta(struct lyd_node *parent, struct lyd_meta **meta, const struct lys_module *mod, const char *name,
-        size_t name_len, const char *value, size_t value_len, ly_bool is_utf8, ly_bool store_only, ly_bool *dynamic,
-        LY_VALUE_FORMAT format, void *prefix_data, uint32_t hints, const struct lysc_node *ctx_node, ly_bool clear_dflt,
-        ly_bool *incomplete);
+        uint32_t name_len, const void *value, uint32_t value_size_bits, ly_bool is_utf8, ly_bool store_only, ly_bool *dynamic,
+        LY_VALUE_FORMAT format, void *prefix_data, uint32_t hints, const struct lysc_node *ctx_node,
+        const struct lysc_ext_instance *top_ext, ly_bool clear_dflt, ly_bool *incomplete);
 
 /**
  * @brief Create a copy of the metadata.
@@ -572,8 +576,8 @@ void lyd_insert_attr(struct lyd_node *parent, struct lyd_attr *attr);
  * @return LY_ERR value on error.
  */
 LY_ERR lyd_create_attr(struct lyd_node *parent, struct lyd_attr **attr, const struct ly_ctx *ctx, const char *name,
-        size_t name_len, const char *prefix, size_t prefix_len, const char *module_key, size_t module_key_len,
-        const char *value, size_t value_len, ly_bool *dynamic, LY_VALUE_FORMAT format, void *val_prefix_data, uint32_t hints);
+        uint32_t name_len, const char *prefix, uint32_t prefix_len, const char *module_key, uint32_t module_key_len,
+        const char *value, uint32_t value_len, ly_bool *dynamic, LY_VALUE_FORMAT format, void *val_prefix_data, uint32_t hints);
 
 /**
  * @brief Store and canonize the given @p value into @p val according to the schema node type rules.
@@ -582,7 +586,7 @@ LY_ERR lyd_create_attr(struct lyd_node *parent, struct lyd_attr **attr, const st
  * @param[in,out] val Storage for the value.
  * @param[in] type Type of the value.
  * @param[in] value Value to be parsed, must not be NULL.
- * @param[in] value_len Length of the give @p value, must be set correctly.
+ * @param[in] value_size_bits Size of @p value in bits, must be set correctly.
  * @param[in] is_utf8 Whether @p value is a valid UTF-8 string, if applicable.
  * @param[in] store_only Whether to perform storing operation only.
  * @param[in,out] dynamic Flag if @p value is dynamically allocated, is adjusted when @p value is consumed.
@@ -590,13 +594,15 @@ LY_ERR lyd_create_attr(struct lyd_node *parent, struct lyd_attr **attr, const st
  * @param[in] prefix_data Format-specific data for resolving any prefixes (see ::ly_resolve_prefix).
  * @param[in] hints [Value hints](@ref lydvalhints) from the parser.
  * @param[in] ctx_node Context schema node.
+ * @param[in] top_ext Extension instance containing the definition of the data being stored.
  * @param[out] incomplete Optional, set if the value also needs to be resolved.
  * @return LY_SUCCESS on success,
  * @return LY_ERR value on error.
  */
 LY_ERR lyd_value_store(const struct ly_ctx *ctx, struct lyd_value *val, const struct lysc_type *type, const void *value,
-        size_t value_len, ly_bool is_utf8, ly_bool store_only, ly_bool *dynamic, LY_VALUE_FORMAT format, void *prefix_data,
-        uint32_t hints, const struct lysc_node *ctx_node, ly_bool *incomplete);
+        uint32_t value_size_bits, ly_bool is_utf8, ly_bool store_only, ly_bool *dynamic, LY_VALUE_FORMAT format,
+        void *prefix_data, uint32_t hints, const struct lysc_node *ctx_node, const struct lysc_ext_instance *top_ext,
+        ly_bool *incomplete);
 
 /**
  * @brief Validate previously incompletely stored value.
@@ -606,11 +612,12 @@ LY_ERR lyd_value_store(const struct ly_ctx *ctx, struct lyd_value *val, const st
  * @param[in,out] val Stored value to resolve.
  * @param[in] ctx_node Context node for the resolution.
  * @param[in] tree Data tree for the resolution.
+ * @param[in] top_ext Extension instance containing the definition of the data being validated.
  * @return LY_SUCCESS on success,
  * @return LY_ERR value on error.
  */
 LY_ERR lyd_value_validate_incomplete(const struct ly_ctx *ctx, const struct lysc_type *type, struct lyd_value *val,
-        const struct lyd_node *ctx_node, const struct lyd_node *tree);
+        const struct lyd_node *ctx_node, const struct lyd_node *tree, const struct lysc_ext_instance *top_ext);
 
 /**
  * @brief Check type restrictions applicable to the particular leaf/leaf-list with the given string @p value.
@@ -620,16 +627,44 @@ LY_ERR lyd_value_validate_incomplete(const struct ly_ctx *ctx, const struct lysc
  *
  * @param[in] ctx libyang context for logging (function does not log errors when @p ctx is NULL)
  * @param[in] node Schema node for the @p value.
- * @param[in] value String value to be checked, expected to be in JSON format.
- * @param[in] value_len Length of the given @p value (mandatory).
+ * @param[in] value Value to be checked in @p format.
+ * @param[in] value_size_bits Size of the given @p value in bits.
  * @param[in] format Value prefix format.
  * @param[in] prefix_data Format-specific data for resolving any prefixes (see ::ly_resolve_prefix).
  * @param[in] hints Value encoding hints.
+ * @param[in] top_ext Extension instance containing the definition of the data being validated.
  * @return LY_SUCCESS on success
  * @return LY_ERR value if an error occurred.
  */
-LY_ERR ly_value_validate(const struct ly_ctx *ctx, const struct lysc_node *node, const char *value, size_t value_len,
-        LY_VALUE_FORMAT format, void *prefix_data, uint32_t hints);
+LY_ERR ly_value_validate(const struct ly_ctx *ctx, const struct lysc_node *node, const void *value,
+        uint32_t value_size_bits, LY_VALUE_FORMAT format, void *prefix_data, uint32_t hints,
+        const struct lysc_ext_instance *top_ext);
+
+/**
+ * @brief Check type restrictions applicable to the particular leaf/leaf-list with the given string @p value.
+ *
+ * The given node is not modified in any way - it is just checked if the @p value can be set to the node.
+ *
+ * @param[in] schema Schema node of the @p value.
+ * @param[in] value String value to be checked, it is expected to be in JSON format.
+ * @param[in] value_len Length of the given @p value (mandatory).
+ * @param[in] format Value prefix format.
+ * @param[in] prefix_data Format-specific data for resolving any prefixes (see ::ly_resolve_prefix).
+ * @param[in] hints Value hints, bitmap of LYD_VALHINT_* values.
+ * @param[in] ctx_node Optional data tree context node for the value (leafref target, instance-identifier).
+ * If not set and is required for the validation to complete, ::LY_EINCOMPLETE is be returned.
+ * @param[in] top_ext Extension instance containing the definition of the data being validated.
+ * @param[in] log Whether to log errors or not.
+ * @param[out] realtype Optional real type of @p value.
+ * @param[out] canonical Optional canonical value of @p value (in the dictionary).
+ * @return LY_SUCCESS on success
+ * @return LY_EINCOMPLETE in case the @p ctx_node is not provided and it was needed to finish the validation
+ * (e.g. due to require-instance).
+ * @return LY_ERR value if an error occurred.
+ */
+LY_ERR lyd_value_validate3(const struct lysc_node *schema, const char *value, size_t value_len,
+        LY_VALUE_FORMAT format, void *prefix_data, uint32_t hints, const struct lyd_node *ctx_node,
+        const struct lysc_ext_instance *top_ext, int log, const struct lysc_type **realtype, const char **canonical);
 
 /**
  * @defgroup datahash Data nodes hash manipulation
@@ -774,6 +809,6 @@ void lyd_unlink_ignore_lyds(struct lyd_node **first_sibling, struct lyd_node *no
  * @return LY_ENOT if the string does not match;
  * @return LY_ERR on error.
  */
-LY_ERR ly_pattern_code_match(pcre2_code *pcode, const char *str, size_t str_len, struct ly_err_item **err);
+LY_ERR ly_pattern_code_match(const pcre2_code *pcode, const char *str, size_t str_len, struct ly_err_item **err);
 
 #endif /* LY_TREE_DATA_INTERNAL_H_ */
